@@ -9,7 +9,6 @@
 #include "gettext.h"
 
 using namespace std;
-using namespace boost;
 using namespace cnoid;
 
 
@@ -33,9 +32,11 @@ FileBar* FileBar::instance()
 FileBar::FileBar()
     : ToolBar(N_("FileBar"))
 {
+    setVisibleByDefault(true);
+    
     addButton(QIcon(":/Base/icons/projectsave.png"), _("Save the project"))
         ->sigClicked().connect(
-            bind(&ProjectManager::overwriteCurrentProject, ProjectManager::instance()));
+            boost::bind(&ProjectManager::overwriteCurrentProject, ProjectManager::instance()));
 }
 
 
