@@ -1,5 +1,5 @@
 /**
-  @author Shin'ichiro Nakaoka
+   @author Shin'ichiro Nakaoka
 */
 
 #include "FolderItem.h"
@@ -14,13 +14,6 @@ FolderItem::FolderItem()
 }
 
 
-FolderItem::FolderItem(const std::string& name)
-    : Item(name)
-{
-
-}
-
-
 FolderItem::FolderItem(const FolderItem& org)
     : Item(org)
 {
@@ -34,7 +27,7 @@ FolderItem::~FolderItem()
 }
 
 
-ItemPtr FolderItem::doDuplicate() const
+Item* FolderItem::doDuplicate() const
 {
     return new FolderItem(*this);
 }
@@ -52,7 +45,7 @@ bool FolderItem::restore(const Archive& archive)
 }
 
 
-void FolderItem::initialize(ExtensionManager* ext)
+void FolderItem::initializeClass(ExtensionManager* ext)
 {
     static bool initialized = false;
     if(!initialized){
