@@ -8,9 +8,9 @@
 #include "TimeBar.h"
 #include "ToolBarArea.h"
 #include "ScrollBar.h"
-#include "ConnectionSet.h"
 #include <cnoid/View>
 #include <cnoid/MainWindow>
+#include <cnoid/ConnectionSet>
 #include <boost/bind.hpp>
 #include <boost/format.hpp>
 #include <boost/dynamic_bitset.hpp>
@@ -32,218 +32,222 @@ inline int isfinite(double x) { return _finite(x); }
 #include "gettext.h"
 
 using namespace std;
-using namespace boost;
 using namespace cnoid;
 
 
 namespace {
 
-    const bool TRACE_FUNCTIONS = false;
+const bool TRACE_FUNCTIONS = false;
         
-    const size_t MAX_NUM_HISTORIES = 10;
+const size_t MAX_NUM_HISTORIES = 10;
 
-    struct EditHistory
-    {
-        EditHistory() { frame = 0; }
-        int frame;
-        vector<double> orgValues;
-        vector<double> newValues;
-    };
-    typedef shared_ptr<EditHistory> EditHistoryPtr;
-    typedef deque<EditHistoryPtr> EditHistoryList;
+struct EditHistory
+{
+    EditHistory() { frame = 0; }
+    int frame;
+    vector<double> orgValues;
+    vector<double> newValues;
+};
+typedef boost::shared_ptr<EditHistory> EditHistoryPtr;
+typedef deque<EditHistoryPtr> EditHistoryList;
 }
 
 
 namespace cnoid {
 
-    class GraphDataHandlerImpl
-    {
-    public:
-        GraphDataHandlerImpl();
+class GraphDataHandlerImpl
+{
+public:
+    GraphDataHandlerImpl();
         
-        boost::signal<void()> sigDataUpdated;
+    Signal<void()> sigDataUpdated;
 
-        /**
-           The size of this array is the actual data size + 2 and the actual data is
-           stored from the second elements. The prepended / appended elements are
-           same as the initial / last elements of the actual data, and the elements
-           are used for calculating velocities.
-        */
-        vector<double> values;
-        int numFrames; // the actual number of frames (values.size() - 2)
+    /**
+       The size of this array is the actual data size + 2 and the actual data is
+       stored from the second elements. The prepended / appended elements are
+       same as the initial / last elements of the actual data, and the elements
+       are used for calculating velocities.
+    */
+    vector<double> values;
+    int numFrames; // the actual number of frames (values.size() - 2)
         
-        int prevNumValues;
-        double offset;
-        double stepRatio;
-        double lowerValueLimit;
-        double upperValueLimit;
-        double lowerVelocityLimit;
-        double upperVelocityLimit;
+    int prevNumValues;
+    double offset;
+    double stepRatio;
+    double lowerValueLimit;
+    double upperValueLimit;
+    double lowerVelocityLimit;
+    double upperVelocityLimit;
 
-        std::string label;
-        float r, g, b;
+    int id;
+    std::string label;
+    float r, g, b;
 
-        EditHistoryList editHistories;
-        int currentHistory;
+    EditHistoryList editHistories;
+    int currentHistory;
 
-        dynamic_bitset<> controlPointMask;
-        bool isControlPointUpdateNeeded;
+    boost::dynamic_bitset<> controlPointMask;
+    bool isControlPointUpdateNeeded;
 
-        GraphDataHandler::DataRequestCallback dataRequestCallback;
-        GraphDataHandler::DataModifiedCallback dataModifiedCallback;
-    };
+    GraphDataHandler::DataRequestCallback dataRequestCallback;
+    GraphDataHandler::DataModifiedCallback dataModifiedCallback;
+};
 
 
-    class GraphWidgetImpl : public signals::trackable
-    {
-    public:
+class GraphWidgetImpl
+{
+public:
 
-        GraphWidget* self;
-        View* parentView;
-        QWidget* screen;
-        DoubleScrollBar* hScrollbar;
-        DoubleScrollBar* vScrollbar;
+    GraphWidget* self;
+    View* parentView;
+    QWidget* screen;
+    DoubleScrollBar* hScrollbar;
+    DoubleScrollBar* vScrollbar;
 
-        QPen pen;
+    QPen pen;
 
-        QVector<qreal> cursorDashes;
-        QVector<qreal> limitValueDashes;
-        QPolygonF polyline;
+    QVector<qreal> cursorDashes;
+    QVector<qreal> limitValueDashes;
+    QPolygonF polyline;
 
-        GraphWidget::Mode mode;
-        GraphWidget::EditMode editMode;
+    GraphWidget::Mode mode;
+    GraphWidget::EditMode editMode;
 
-        std::vector<GraphDataHandlerPtr> handlers;
-        ConnectionSet connections;
+    std::vector<GraphDataHandlerPtr> handlers;
+    ConnectionSet connections;
 
-        bool isOrgValueVisible;
-        bool isVelocityVisible;
-        bool isAccelerationVisible;
+    bool isOrgValueVisible;
+    bool isVelocityVisible;
+    bool isAccelerationVisible;
 
-        bool isReconfigurationNeeded;
-        bool isDomainUpdateNeeded;
+    bool isReconfigurationNeeded;
+    bool isDomainUpdateNeeded;
 
-        double domainLowerX;
-        double domainUpperX;
+    double domainLowerX;
+    double domainUpperX;
 
-        double rangeLowerY;
-        double rangeUpperY;
+    double rangeLowerY;
+    double rangeUpperY;
 
-        double cursorX;
-        double screenCursorX;
+    double cursorX;
+    double screenCursorX;
 
-        double leftX;
-        double centerY;
-        double scaleX;
-        double scaleY;
+    double currentScreenX;
+    double currentScreenY;
 
-        double visibleWidth;
-        double visibleHeight;
-        double visibleHeightHalf;
-        double bottomY;
-        double topY;
+    double leftX;
+    double centerY;
+    double scaleX;
+    double scaleY;
 
-        double screenMarginX;
-        double screenCenterY;
+    double visibleWidth;
+    double visibleHeight;
+    double visibleHeightHalf;
+    double bottomY;
+    double topY;
 
-        double screenWidth;
-        double screenHeight;
+    double screenMarginX;
+    double screenCenterY;
 
-        double lineWidth;
+    double screenWidth;
+    double screenHeight;
 
-        bool isLimitVisible;
-        bool isGridOn;
-        double gridWidth;
-        double gridHeight;
+    double lineWidth;
 
-        GraphDataHandlerImpl* editTarget;
-        bool isControlPointsHighlighted;
-        int controlPointStep;
-        int controlPointOffset;
+    bool isLimitVisible;
+    bool isGridOn;
+    double gridWidth;
+    double gridHeight;
 
-        enum { DRAG_NONE, DRAG_CURSOR, DRAG_EDIT, DRAG_ZOOM, DRAG_MOVE } dragState;
+    GraphDataHandlerImpl* editTarget;
+    bool isControlPointsHighlighted;
+    int controlPointStep;
+    int controlPointOffset;
 
-        double pressedScreenX;
-        double pressedScreenY;
-        double dragPrevScreenX;
-        double dragPrevScreenY;
+    enum { DRAG_NONE, DRAG_CURSOR, DRAG_EDIT, DRAG_ZOOM, DRAG_MOVE } dragState;
 
-        double dragOrgScaleX;
-        double dragOrgScaleY;
-        double dragOrgLeftX;
-        double dragOrgCenterY;
+    double pressedScreenX;
+    double pressedScreenY;
+    double dragPrevScreenX;
+    double dragPrevScreenY;
 
-        bool isEditBufferedUpdateMode;
-        int editedFrameBegin;
-        int editedFrameEnd;
+    double dragOrgScaleX;
+    double dragOrgScaleY;
+    double dragOrgLeftX;
+    double dragOrgCenterY;
 
-        GraphWidget::ScrollMode autoScrollMode;
+    bool isEditBufferedUpdateMode;
+    int editedFrameBegin;
+    int editedFrameEnd;
 
-        TimeBar* timeBar;
-        bool timeBarSyncMode;
-        boost::signals::connection timeChangedConnetion;
+    GraphWidget::ScrollMode autoScrollMode;
 
-        QLabel statusLabel;
+    TimeBar* timeBar;
+    bool timeBarSyncMode;
+    Connection timeChangedConnetion;
 
-        GraphWidgetImpl(GraphWidget* self, View* parentView);
-        ~GraphWidgetImpl();
+    QLabel statusLabel;
 
-        void onActivated();
-        void onDeactivated();
+    GraphWidgetImpl(GraphWidget* self, View* parentView);
+    ~GraphWidgetImpl();
 
-        void addDataHandler(GraphDataHandlerPtr handler);
-        void clearDataHandlers();
-        void updateData(GraphDataHandlerPtr handler);
-        bool setCursorPosition(double x, bool enableTimeBarSync, bool forceTimeChange);
-        bool setCursorPositionWithScreenX(double screenX, bool enableTimeBarSync, bool forceTimeChange);
+    void onActivated();
+    void onDeactivated();
 
-        void setTimeBarSyncMode(bool on);
-        void enableTimeBarSync(bool on);
-        void setAutoScrollMode(GraphWidget::ScrollMode mode);
-        void setVerticalValueRange(double lower, double upper);
-        void showLimits(bool show);
-        void showRulers(bool show);
-        void showGrid(bool show);
-        void setGridSize(double width, double height);
-        void setControlPointStep(int step, int offset);
-        void highlightControlPoints(bool on);
-        void changeMode(GraphWidget::Mode mode);
-        void changeEditMode(GraphWidget::EditMode mode);
+    void addDataHandler(GraphDataHandlerPtr handler);
+    void clearDataHandlers();
+    void updateData(GraphDataHandlerPtr handler);
+    bool setCursorPosition(double x, bool enableTimeBarSync, bool forceTimeChange);
+    bool setCursorPositionWithScreenX(double screenX, bool enableTimeBarSync, bool forceTimeChange);
+
+    void setTimeBarSyncMode(bool on);
+    void enableTimeBarSync(bool on);
+    void setAutoScrollMode(GraphWidget::ScrollMode mode);
+    void setVerticalValueRange(double lower, double upper);
+    void showLimits(bool show);
+    void showRulers(bool show);
+    void showGrid(bool show);
+    void setGridSize(double width, double height);
+    void setControlPointStep(int step, int offset);
+    void highlightControlPoints(bool on);
+    void changeMode(GraphWidget::Mode mode);
+    void changeEditMode(GraphWidget::EditMode mode);
         
-        void setupConfiguration();
-        void updateDomain();
+    void setupConfiguration();
+    void updateDomain();
 
-        bool onFocusInEvent(QFocusEvent* event);
-        bool onScreenResizeEvent(QResizeEvent* event);
-        bool onScreenMouseButtonPressEvent(QMouseEvent* event);
-        bool onScreenMouseButtonReleaseEvent(QMouseEvent* event);
-        bool onScreenMouseMoveEvent(QMouseEvent* event);
-        bool onScreenKeyPressEvent(QKeyEvent* event);
-        bool onScreenKeyReleaseEvent(QKeyEvent* event);
-        void zoomScreen(int screenX, int screenY);
-        void moveScreen(double screenX, double screenY);
-        void startTrajectoryEdit();
-        void finishTrajectoryEdit();
-        void doTrajectoryEdit(int screenX, int screenY, Qt::KeyboardModifiers modifiers);
-        void storeOrgValuesToHistory(int frameBegin, int frameEnd);
-        void undoTrajectoryEdit();
-        void redoTrajectoryEdit();
-        void selectEditTargetByClicking(double screenX, double screenY);
-        bool onScreenPaintEvent(QPaintEvent* event);
-        void drawTrajectory(QPainter& painter, const QRect& rect, GraphDataHandlerImpl* data);
-        void drawLimits(QPainter& painter, GraphDataHandlerImpl* data);
-        void updateControlPoints(GraphDataHandlerImpl* data);
-        void drawGrid(QPainter& painter);
-        void onHScrollbarChanged(double value);
-        void onVScrollbarChanged(double value);
-        bool storeState(Archive& archive);
-        bool restoreState(const Archive& archive);        
-    };
+    bool onFocusInEvent(QFocusEvent* event);
+    bool onScreenResizeEvent(QResizeEvent* event);
+    bool onScreenMouseButtonPressEvent(QMouseEvent* event);
+    bool onScreenMouseButtonReleaseEvent(QMouseEvent* event);
+    bool onScreenMouseMoveEvent(QMouseEvent* event);
+    bool onScreenKeyPressEvent(QKeyEvent* event);
+    bool onScreenKeyReleaseEvent(QKeyEvent* event);
+    void zoomScreen(int screenX, int screenY);
+    void moveScreen(double screenX, double screenY);
+    void startTrajectoryEdit();
+    void finishTrajectoryEdit();
+    void doTrajectoryEdit(int screenX, int screenY, Qt::KeyboardModifiers modifiers);
+    void storeOrgValuesToHistory(int frameBegin, int frameEnd);
+    void undoTrajectoryEdit();
+    void redoTrajectoryEdit();
+    void selectEditTargetByClicking(double screenX, double screenY);
+    bool onScreenPaintEvent(QPaintEvent* event);
+    void drawTrajectory(QPainter& painter, const QRect& rect, GraphDataHandlerImpl* data);
+    void drawLimits(QPainter& painter, GraphDataHandlerImpl* data);
+    void updateControlPoints(GraphDataHandlerImpl* data);
+    void drawGrid(QPainter& painter);
+    void onHScrollbarChanged(double value);
+    void onVScrollbarChanged(double value);
+    bool storeState(Archive& archive);
+    bool restoreState(const Archive& archive);        
+};
 }
 
 
 GraphDataHandler::GraphDataHandler()
 {
+    id = -1;
     impl = new GraphDataHandlerImpl();
 }
 
@@ -273,6 +277,9 @@ GraphDataHandler::~GraphDataHandler()
 }
 
 
+
+
+
 void GraphDataHandler::setColor(float r, float g, float b)
 {
     impl->r = r;
@@ -374,11 +381,11 @@ GraphWidgetImpl::GraphWidgetImpl(GraphWidget* self, View* parentView)
 
     hScrollbar = new DoubleScrollBar(Qt::Horizontal, self);
     hScrollbar->sigValueChanged().connect(
-        bind(&GraphWidgetImpl::onHScrollbarChanged, this, _1));
+        boost::bind(&GraphWidgetImpl::onHScrollbarChanged, this, _1));
     
     vScrollbar = new DoubleScrollBar(Qt::Vertical, self);
     vScrollbar->sigValueChanged().connect(
-        bind(&GraphWidgetImpl::onVScrollbarChanged, this, _1));
+        boost::bind(&GraphWidgetImpl::onVScrollbarChanged, this, _1));
 
     QGridLayout* grid = new QGridLayout(self);
     grid->setSpacing(0);
@@ -386,6 +393,9 @@ GraphWidgetImpl::GraphWidgetImpl(GraphWidget* self, View* parentView)
     grid->addWidget(screen, 0, 0);
     grid->addWidget(hScrollbar, 1, 0);
     grid->addWidget(vScrollbar, 0, 1);
+    grid->setColumnStretch(0, 1);
+    grid->setRowStretch(0, 1);
+    grid->setSizeConstraint(QLayout::SetNoConstraint);
     self->setLayout(grid);
     
     isOrgValueVisible = true;
@@ -402,6 +412,9 @@ GraphWidgetImpl::GraphWidgetImpl(GraphWidget* self, View* parentView)
     leftX = 0.0;
     centerY = 0.0;
 
+    currentScreenX = 0.0;
+    currentScreenY = 0.0;
+
     rangeUpperY = 10.0;
     rangeLowerY = -rangeUpperY;
 
@@ -425,8 +438,8 @@ GraphWidgetImpl::GraphWidgetImpl(GraphWidget* self, View* parentView)
     timeBar = TimeBar::instance();
     timeBarSyncMode = true;
 
-    parentView->sigActivated().connect(bind(&GraphWidgetImpl::onActivated, this));
-    parentView->sigDeactivated().connect(bind(&GraphWidgetImpl::onDeactivated, this));
+    parentView->sigActivated().connect(boost::bind(&GraphWidgetImpl::onActivated, this));
+    parentView->sigDeactivated().connect(boost::bind(&GraphWidgetImpl::onDeactivated, this));
 
     statusLabel.setAlignment(Qt::AlignLeft);
     QFont font = statusLabel.font();
@@ -474,7 +487,7 @@ void GraphWidgetImpl::addDataHandler(GraphDataHandlerPtr handler)
     
     handlers.push_back(handler);
     connections.add(handler->impl->sigDataUpdated.connect(
-                        bind(&GraphWidgetImpl::updateData, this, handler)));
+                        boost::bind(&GraphWidgetImpl::updateData, this, handler)));
 
     isReconfigurationNeeded = true;
     isDomainUpdateNeeded = true;
@@ -671,7 +684,7 @@ void GraphWidgetImpl::enableTimeBarSync(bool on)
     } else {
         if(on && parentView->isActive()){
             timeChangedConnetion = timeBar->sigTimeChanged().connect(
-                bind(&GraphWidgetImpl::setCursorPosition, this, _1, false, false));
+                boost::bind(&GraphWidgetImpl::setCursorPosition, this, _1, false, false));
             setCursorPosition(timeBar->time(), false, false);
         }
     }
@@ -1102,42 +1115,42 @@ bool GraphWidgetImpl::onScreenMouseButtonReleaseEvent(QMouseEvent* event)
 
 bool GraphWidgetImpl::onScreenMouseMoveEvent(QMouseEvent* event)
 {
-    const double screenX = event->x();
-    const double screenY = event->y();
+    currentScreenX = event->x();
+    currentScreenY = event->y();
 
     switch(dragState){
 
     case DRAG_CURSOR:
-        setCursorPositionWithScreenX(screenX, true, false);
+        setCursorPositionWithScreenX(currentScreenX, true, false);
         break;
 
     case DRAG_EDIT:
-        doTrajectoryEdit(screenX, screenY, event->modifiers());
+        doTrajectoryEdit(currentScreenX, currentScreenY, event->modifiers());
         break;
 
     case DRAG_ZOOM:
-        zoomScreen(screenX, screenY);
+        zoomScreen(currentScreenX, currentScreenY);
         break;
 
     case DRAG_MOVE:
-        moveScreen(screenX, screenY);
+        moveScreen(currentScreenX, currentScreenY);
         break;
 
     default:
         break;
     }
 
-    double x = leftX + (screenX - screenMarginX) / scaleX;
-    double y = - centerY - (screenY - screenCenterY) / scaleY;
+    double x = leftX + (currentScreenX - screenMarginX) / scaleX;
+    double y = - centerY - (currentScreenY - screenCenterY) / scaleY;
 
     //hRuler.property_position() = x;
     //vRuler.property_position() = y;
 
-    static format f(_("Graph: Position = (%1$.5f, %2$.5f)"));
+    static boost::format f(_("Graph: Position = (%1$.5f, %2$.5f)"));
     statusLabel.setText(str(f % x % y).c_str());
 
-    dragPrevScreenX = screenX;
-    dragPrevScreenY = screenY;
+    dragPrevScreenX = currentScreenX;
+    dragPrevScreenY = currentScreenY;
     
     return true;
 }
@@ -1180,6 +1193,16 @@ bool GraphWidgetImpl::onScreenKeyPressEvent(QKeyEvent* event)
         case Qt::Key_L:
             changeEditMode(GraphWidget::LINE_MODE);
             break;
+
+        case Qt::Key_Space:
+        {
+            QMouseEvent mouseEvent(QEvent::MouseButtonPress,
+                                   QPoint(currentScreenX, currentScreenY),
+                                   Qt::MidButton, Qt::MidButton,
+                                   event->modifiers());
+            unprocessed = !onScreenMouseButtonPressEvent(&mouseEvent);
+        }
+        break;
         
         default:
             unprocessed = true;
@@ -1203,6 +1226,16 @@ bool GraphWidgetImpl::onScreenKeyReleaseEvent(QKeyEvent* event)
         }
         break;
 
+    case Qt::Key_Space:
+    {
+        QMouseEvent mouseEvent(QEvent::MouseButtonPress,
+                               QPoint(currentScreenX, currentScreenY),
+                               Qt::MidButton, Qt::MidButton,
+                               event->modifiers());
+        unprocessed = !onScreenMouseButtonReleaseEvent(&mouseEvent);
+    }
+    break;
+
     default:
         unprocessed = true;
         break;
@@ -1556,7 +1589,7 @@ bool GraphWidgetImpl::onScreenPaintEvent(QPaintEvent* event)
     painter.setClipping(true);
 
     pen.setStyle(Qt::SolidLine);
-    pen.setWidthF(lineWidth);
+    pen.setWidthF(1.0);
     painter.setPen(pen);
 
     // draw the grid
@@ -1575,12 +1608,18 @@ bool GraphWidgetImpl::onScreenPaintEvent(QPaintEvent* event)
             drawLimits(painter, handlers[i]->impl);
         }
     }
-        
+
+    pen.setWidthF(lineWidth);
+    painter.setPen(pen);
+    
     for(size_t i=0; i < handlers.size(); ++i){
         GraphDataHandlerImpl* data = handlers[i]->impl;
         drawTrajectory(painter, event->rect(), data);
     }
 
+    pen.setWidthF(1.0);
+    painter.setPen(pen);
+    
     // draw the cursor
     pen.setColor(QColor(25, 25, 25, 230));
     pen.setDashPattern(cursorDashes);
@@ -1757,7 +1796,7 @@ void GraphWidgetImpl::drawTrajectory
                 if(frame < 0){
                     frame = 0;
                 }
-                dynamic_bitset<>& mask = data->controlPointMask;
+                boost::dynamic_bitset<>& mask = data->controlPointMask;
                 while(frame < frame_end){
                     if(mask[frame]){
                         double px = screenOffsetX + (frame - frame_begin) * xratio;
@@ -1871,6 +1910,15 @@ void GraphWidgetImpl::onVScrollbarChanged(double value)
 }
 
 
+bool GraphWidget::saveImage(const std::string& filename)
+{
+    QPixmap pixmap(impl->screen->size());
+    impl->screen->render(&pixmap);
+    pixmap.save(filename.c_str());
+    return true;
+}
+
+
 bool GraphWidget::storeState(Archive& archive)
 {
     return impl->storeState(archive);
