@@ -7,18 +7,30 @@
 
 using namespace cnoid;
 
-void LazySignalBase::request()
+
+LazySignalBase::LazySignalBase()
+    : LazyCaller(boost::bind(&LazySignalBase::doEmit, this))
 {
-    if(!isIdleEventPending){
-        isIdleEventPending = true;
-        callLater(boost::bind(&LazySignalBase::onIdle, this), priority);
-    }
+
 }
 
 
-bool LazySignalBase::onIdle()
+LazySignalBase::LazySignalBase(boost::function<void()> emitFunction, int priority)
+    : LazyCaller(boost::bind(&LazySignalBase::doEmit, this), priority),
+      emitFunction(emitFunction)
+{
+
+}
+
+
+void LazySignalBase::request()
+{
+    (*this)();
+}
+
+
+bool LazySignalBase::doEmit()
 {
-    isIdleEventPending = false;
     for(size_t i=0; i < connectionsToBlock.size(); ++i){
         connectionsToBlock[i].block();
     }
