@@ -10,7 +10,7 @@
 
 using namespace cnoid;
 
-template<> void MultiSeqItem<MultiAffine3Seq>::initialize(ExtensionManager* ext)
+template<> void MultiSeqItem<MultiAffine3Seq>::initializeClass(ExtensionManager* ext)
 {
     ext->itemManager().registerClass<MultiAffine3SeqItem>(N_("MultiAffine3SeqItem"));
     
