@@ -16,7 +16,7 @@ static bool loadPlainSeqFormat(MultiValueSeqItem* item, const std::string& filen
     if(item->seq()->loadPlainFormat(filename)){
         return true;
     } else {
-        os << item->seq()->ioErrorMessage();
+        os << item->seq()->seqMessage();
         return false;
     }
 }
@@ -27,13 +27,13 @@ static bool saveAsPlainSeqFormat(MultiValueSeqItem* item, const std::string& fil
     if(item->seq()->saveAsPlainFormat(filename)){
         return true;
     } else {
-        os << item->seq()->ioErrorMessage();
+        os << item->seq()->seqMessage();
         return false;
     }
 }
 
 
-template<> void MultiSeqItem<MultiValueSeq>::initialize(ExtensionManager* ext)
+template<> void MultiSeqItem<MultiValueSeq>::initializeClass(ExtensionManager* ext)
 {
     ext->itemManager().registerClass<MultiValueSeqItem>(N_("MultiValueSeqItem"));
 
@@ -41,8 +41,8 @@ template<> void MultiSeqItem<MultiValueSeq>::initialize(ExtensionManager* ext)
         new MultiSeqItemCreationPanel(_("Number of values in a frame")));
     
     ext->itemManager().addLoaderAndSaver<MultiValueSeqItem>(
-        _("Plain format of a multi value sequence"), "PLAIN-MULTI-VALUE-SEQ", "*",
-        bind(loadPlainSeqFormat, _1, _2, _3), bind(saveAsPlainSeqFormat, _1, _2, _3), 
+        _("Plain Format of a Multi Value Sequence"), "PLAIN-MULTI-VALUE-SEQ", "*",
+        boost::bind(loadPlainSeqFormat, _1, _2, _3), boost::bind(saveAsPlainSeqFormat, _1, _2, _3), 
         ItemManager::PRIORITY_CONVERSION);
 }
 
