@@ -11,10 +11,10 @@
 
 namespace cnoid {
 
-    typedef MultiSeqItem<MultiValueSeq> MultiValueSeqItem;
-    typedef MultiValueSeqItem::Ptr MultiValueSeqItemPtr;
+typedef MultiSeqItem<MultiValueSeq> MultiValueSeqItem;
+typedef MultiValueSeqItem::Ptr MultiValueSeqItemPtr;
 
-    template<> void MultiSeqItem<MultiValueSeq>::initialize(ExtensionManager* ext);
+template<> void MultiSeqItem<MultiValueSeq>::initializeClass(ExtensionManager* ext);
 }
 
 #endif
