@@ -11,214 +11,235 @@
 #include "LazyCaller.h"
 #include "SpinBox.h"
 #include "Slider.h"
-#include "Button.h"
-#include <QDialog>
+#include "Buttons.h"
+#include "CheckBox.h"
+#include "Dialog.h"
+#include <QDialogButtonBox>
 #include <QTime>
-#include <QCheckBox>
-#include <QLayout>
 #include <boost/bind.hpp>
 #include <boost/lexical_cast.hpp>
+#include <cmath>
 #include <limits>
 #include <iostream>
-
 #include "gettext.h"
 
 using namespace std;
-using namespace boost;
 using namespace cnoid;
 
 namespace {
 
-    const bool TRACE_FUNCTIONS = false;
+const bool TRACE_FUNCTIONS = false;
     
-    inline double myNearByInt(double x)
-    {
+inline double myNearByInt(double x)
+{
 #ifdef Q_OS_WIN32
-        double u = ceil(x);
-        double l = floor(x);
-        if(fabs(u - x) < fabs(x - l)){
-            return u;
-        } else {
-            return l;
-        }
-#else
-        return nearbyint(x);
-#endif
+    double u = ceil(x);
+    double l = floor(x);
+    if(fabs(u - x) < fabs(x - l)){
+        return u;
+    } else {
+        return l;
     }
+#else
+    return nearbyint(x);
+#endif
+}
 
-    class SetupDialog : public QDialog
-    {
-    public:
-        SpinBox* frameRateSpin;
-        SpinBox* playbackFrameRateSpin;
-        DoubleSpinBox* playbackSpeedScaleSpin;
-        QCheckBox* beatModeCheck;
-        DoubleSpinBox* tempoSpin;
-        SpinBox* beatcSpin;
-        SpinBox* beatmSpin;
-        DoubleSpinBox* beatOffsetSpin;
+class ConfigDialog : public Dialog
+{
+public:
+    SpinBox frameRateSpin;
+    SpinBox playbackFrameRateSpin;
+    CheckBox idleLoopDrivenCheck;
+    DoubleSpinBox playbackSpeedScaleSpin;
+    CheckBox fillLevelSyncCheck;
+    CheckBox autoExpandCheck;
+    QCheckBox beatModeCheck;
+    DoubleSpinBox tempoSpin;
+    SpinBox beatcSpin;
+    SpinBox beatmSpin;
+    DoubleSpinBox beatOffsetSpin;
 
-        SetupDialog()
-            : QDialog(MainWindow::instance()) {
+    ConfigDialog() {
+        setWindowTitle(_("Time Bar Config"));
 
-            setWindowTitle(_("Time Bar Setup"));
+        QVBoxLayout* vbox = new QVBoxLayout();
+        setLayout(vbox);
+        
+        QHBoxLayout* hbox = new QHBoxLayout();
+        hbox->addWidget(new QLabel(_("Internal frame rate")));
+        frameRateSpin.setAlignment(Qt::AlignCenter);
+        frameRateSpin.setRange(1, 10000);
+        hbox->addWidget(&frameRateSpin);
+        hbox->addStretch();
+        vbox->addLayout(hbox);
 
-            QVBoxLayout* vbox = new QVBoxLayout();
-            setLayout(vbox);
+        hbox = new QHBoxLayout();
+        hbox->addWidget(new QLabel(_("Playback frame rate")));
+        playbackFrameRateSpin.setAlignment(Qt::AlignCenter);
+        playbackFrameRateSpin.setRange(0, 1000);
+        playbackFrameRateSpin.setValue(50);
+        hbox->addWidget(&playbackFrameRateSpin);
+        hbox->addStretch();
+        vbox->addLayout(hbox);
 
-            QHBoxLayout* hbox = new QHBoxLayout();
-            hbox->addWidget(new QLabel(_("Internal frame rate")));
-            frameRateSpin = new SpinBox();
-            frameRateSpin->setAlignment(Qt::AlignCenter);
-            frameRateSpin->setRange(1, 9999);
-            hbox->addWidget(frameRateSpin);
-            hbox->addStretch();
-            vbox->addLayout(hbox);
-
-            hbox = new QHBoxLayout();
-            hbox->addWidget(new QLabel(_("Playback frame rate")));
-            playbackFrameRateSpin = new SpinBox();
-            playbackFrameRateSpin->setAlignment(Qt::AlignCenter);
-            playbackFrameRateSpin->setRange(0, 999);
-            playbackFrameRateSpin->setValue(50);
-            hbox->addWidget(playbackFrameRateSpin);
-            hbox->addStretch();
-            vbox->addLayout(hbox);
-
-            hbox = new QHBoxLayout();
-            hbox->addWidget(new QLabel(_("Playback speed scale")));
-            playbackSpeedScaleSpin = new DoubleSpinBox();
-            playbackSpeedScaleSpin->setAlignment(Qt::AlignCenter);
-            playbackSpeedScaleSpin->setDecimals(1);
-            playbackSpeedScaleSpin->setRange(0.1, 9.9);
-            playbackSpeedScaleSpin->setSingleStep(0.1);
-            playbackSpeedScaleSpin->setValue(1.0);
-            hbox->addWidget(playbackSpeedScaleSpin);
-            hbox->addStretch();
-            vbox->addLayout(hbox);
-
-            /*
-            hbox = new QHBoxLayout();
-            vbox->addLayout(hbox);
+        hbox = new QHBoxLayout();
+        idleLoopDrivenCheck.setText(_("Idle loop driven mode"));
+        hbox->addWidget(&idleLoopDrivenCheck);
+        hbox->addStretch();
+        vbox->addLayout(hbox);
             
-            beatModeCheck = new QCheckBox(_("Beat mode"));
-            hbox->addWidget(beatModeCheck);
+        hbox = new QHBoxLayout();
+        hbox->addWidget(new QLabel(_("Playback speed scale")));
+        playbackSpeedScaleSpin.setAlignment(Qt::AlignCenter);
+        playbackSpeedScaleSpin.setDecimals(1);
+        playbackSpeedScaleSpin.setRange(0.1, 99.9);
+        playbackSpeedScaleSpin.setSingleStep(0.1);
+        playbackSpeedScaleSpin.setValue(1.0);
+        hbox->addWidget(&playbackSpeedScaleSpin);
+        hbox->addStretch();
+        vbox->addLayout(hbox);
 
-            beatcSpin = new SpinBox();
-            beatcSpin->setRange(1, 99);
-            hbox->addWidget(beatcSpin);
+        hbox = new QHBoxLayout();
+        fillLevelSyncCheck.setText(_("Sync with ongoing updates"));
+        fillLevelSyncCheck.setChecked(true);
+        hbox->addWidget(&fillLevelSyncCheck);
+        hbox->addStretch();
+        vbox->addLayout(hbox);
 
-            hbox->addWidget(new QLabel("/"));
+        hbox = new QHBoxLayout();
+        autoExpandCheck.setText(_("Automatically expand the time range"));
+        autoExpandCheck.setChecked(true);
+        hbox->addWidget(&autoExpandCheck);
+        hbox->addStretch();
+        vbox->addLayout(hbox);
+            
+        /*
+          hbox = new QHBoxLayout();
+          vbox->addLayout(hbox);
+            
+          beatModeCheck = new QCheckBox(_("Beat mode"));
+          hbox->addWidget(beatModeCheck);
 
-            beatmSpin = new SpinBox();
-            beatmSpin->setRange(1, 99);
-            hbox->addWidget(beatmSpin);
+          beatcSpin = new SpinBox();
+          beatcSpin->setRange(1, 99);
+          hbox->addWidget(beatcSpin);
 
-            hbox->addStretch();
-            hbox = new QHBoxLayout();
-            vbox->addLayout(hbox);
+          hbox->addWidget(new QLabel("/"));
 
-            hbox->addWidget(new QLabel(_("Tempo")));
-            tempoSpin = new DoubleSpinBox();
-            tempoSpin->setRange(1.0, 999.99);
-            tempoSpin->setDecimals(2);
-            hbox->addWidget(tempoSpin);
+          beatmSpin = new SpinBox();
+          beatmSpin->setRange(1, 99);
+          hbox->addWidget(beatmSpin);
 
-            hbox->addWidget(new QLabel(_("Offset")));
-            beatOffsetSpin = new DoubleSpinBox();
-            beatOffsetSpin->setRange(-9.99, 9.99);
-            beatOffsetSpin->setDecimals(2);
-            beatOffsetSpin->setSingleStep(0.1);
-            hbox->addWidget(beatOffsetSpin);
-            hbox->addWidget(new QLabel(_("[s]")));
+          hbox->addStretch();
+          hbox = new QHBoxLayout();
+          vbox->addLayout(hbox);
 
-            hbox->addStretch();
-            */
+          hbox->addWidget(new QLabel(_("Tempo")));
+          tempoSpin = new DoubleSpinBox();
+          tempoSpin->setRange(1.0, 999.99);
+          tempoSpin->setDecimals(2);
+          hbox->addWidget(tempoSpin);
 
-            PushButton* okButton = new PushButton(_("&OK"));
-            okButton->setDefault(true);
+          hbox->addWidget(new QLabel(_("Offset")));
+          beatOffsetSpin = new DoubleSpinBox();
+          beatOffsetSpin->setRange(-9.99, 9.99);
+          beatOffsetSpin->setDecimals(2);
+          beatOffsetSpin->setSingleStep(0.1);
+          hbox->addWidget(beatOffsetSpin);
+          hbox->addWidget(new QLabel(_("[s]")));
 
-            vbox->addStretch();
-        }
-    };
+          hbox->addStretch();
+        */
+
+        vbox->addStretch();
+
+        PushButton* okButton = new PushButton(_("&OK"));
+        okButton->setDefault(true);
+        QDialogButtonBox* buttonBox = new QDialogButtonBox(this);
+        buttonBox->addButton(okButton, QDialogButtonBox::AcceptRole);
+        connect(buttonBox,SIGNAL(accepted()), this, SLOT(accept()));
+        vbox->addWidget(okButton);
+    }
+};
 
 }
 
 namespace cnoid {
 
-    class TimeBarImpl : public QObject
-    {
-    public:
-        TimeBarImpl(TimeBar* self);
-        ~TimeBarImpl();
+class TimeBarImpl : public QObject
+{
+public:
+    TimeBarImpl(TimeBar* self);
+    ~TimeBarImpl();
 
-        bool setTime(double time, bool calledFromPlaybackLoop, QWidget* callerWidget = 0);
-        void onTimeSpinChanged(double value);
-        bool onTimeSliderChangeValue(double value);
+    bool setTime(double time, bool calledFromPlaybackLoop, QWidget* callerWidget = 0);
+    void onTimeSpinChanged(double value);
+    bool onTimeSliderChangeValue(double value);
 
-        void setTimeRange(double minTime, double maxTime);
-        void setFrameRate(double rate);
-        void updateTimeProperties(bool forceUpdate);
-        void onPlaybackSpeedScaleChanged();
-        void onPlaybackFrameRateChanged();
-        void onPlayActivated();
-        void onResumeActivated();
-        void startPlayback();
-        void stopPlayback(bool isStoppedManually);
-        int startFillLevelUpdate();
-        void updateFillLevel(int id, double time);
-        void updateMinFillLevel();
-        void stopFillLevelUpdate(int id);
+    void setTimeRange(double minTime, double maxTime);
+    void setFrameRate(double rate);
+    void updateTimeProperties(bool forceUpdate);
+    void onPlaybackSpeedScaleChanged();
+    void onPlaybackFrameRateChanged();
+    void onPlayActivated();
+    void onResumeActivated();
+    void startPlayback();
+    void stopPlayback(bool isStoppedManually);
+    int startFillLevelUpdate();
+    void updateFillLevel(int id, double time);
+    void updateMinFillLevel();
+    void stopFillLevelUpdate(int id);
 
-        void onTimeRangeSpinsChanged();
-        void onFrameRateSpinChanged();
+    void onTimeRangeSpinsChanged();
+    void onFrameRateSpinChanged();
 
-        virtual void timerEvent(QTimerEvent* event);
+    virtual void timerEvent(QTimerEvent* event);
         
-        void onRefreshButtonClicked();
+    void onRefreshButtonClicked();
 
-        bool storeState(Archive& archive);
-        bool restoreState(const Archive& archive);
+    bool storeState(Archive& archive);
+    bool restoreState(const Archive& archive);
 
-        TimeBar* self;
-        ostream& os;
-        SetupDialog setup;
+    TimeBar* self;
+    ostream& os;
+    ConfigDialog config;
 
-        ToolButton* stopResumeButton;
-        ToolButton* frameModeToggle;
-        QIcon resumeIcon;
-        QIcon stopIcon;
+    ToolButton* stopResumeButton;
+    ToolButton* frameModeToggle;
+    QIcon resumeIcon;
+    QIcon stopIcon;
         
-        DoubleSpinBox* timeSpin;
-        Slider* timeSlider;
-        DoubleSpinBox* minTimeSpin;
-        DoubleSpinBox* maxTimeSpin;
-        int decimals;
-        double minTime;
-        double maxTime;
-        double playbackSpeedScale;
-        double playbackFrameRate;
-        double animationTimeOffset;
-        int timerId;
-        QTime timer;
-        bool repeatMode;
-        bool isDoingPlayback;
-        map<int, double> fillLevelMap;
-        double fillLevel;
-        bool isFillLevelActive;
+    DoubleSpinBox* timeSpin;
+    Slider* timeSlider;
+    DoubleSpinBox* minTimeSpin;
+    DoubleSpinBox* maxTimeSpin;
+    int decimals;
+    double minTime;
+    double maxTime;
+    double playbackSpeedScale;
+    double playbackFrameRate;
+    double animationTimeOffset;
+    int timerId;
+    QTime timer;
+    bool repeatMode;
+    bool isDoingPlayback;
+    map<int, double> fillLevelMap;
+    double fillLevel;
+    bool isFillLevelActive;
 
-        boost::signal<bool(double time), TimeBar::LogicalProduct> sigPlaybackInitialized;
-        boost::signal<void(double time)> sigPlaybackStarted;
-        boost::signal<bool(double time), TimeBar::LogicalSum> sigTimeChanged;
-        boost::signal<void(double time, bool isStoppedManually)> sigPlaybackStopped;
-    };
+    Signal<bool(double time), LogicalProduct> sigPlaybackInitialized;
+    Signal<void(double time)> sigPlaybackStarted;
+    Signal<bool(double time), LogicalSum> sigTimeChanged;
+    Signal<void(double time, bool isStoppedManually)> sigPlaybackStopped;
+};
 }
 
 
 static void onSigOptionsParsed(boost::program_options::variables_map& v)
 {
     if(v.count("start-playback")){
-        callLater(bind(&TimeBar::startPlayback, TimeBar::instance()));
+        callLater(boost::bind(&TimeBar::startPlayback, TimeBar::instance()));
     }
 }
 
@@ -229,8 +250,9 @@ void TimeBar::initialize(ExtensionManager* ext)
     if(!initialized){
         ext->addToolBar(TimeBar::instance());
 
-        ext->optionManager().addOption("start-playback", "start playback automatically");
-        ext->optionManager().sigOptionsParsed().connect(onSigOptionsParsed);
+        ext->optionManager()
+            .addOption("start-playback", "start playback automatically")
+            .sigOptionsParsed().connect(onSigOptionsParsed);
             
         initialized = true;
     }
@@ -257,6 +279,7 @@ TimeBarImpl::TimeBarImpl(TimeBar* self)
       resumeIcon(QIcon(":/Base/icons/resume.png")),
       stopIcon(QIcon(":/Base/icons/stop.png"))
 {
+    self->setVisibleByDefault(true);
     self->setStretchable(true);
     
     self->time_ = 0.0;
@@ -271,28 +294,29 @@ TimeBarImpl::TimeBarImpl(TimeBar* self)
     isFillLevelActive = false;
 
     self->addButton(QIcon(":/Base/icons/play.png"), _("Start animation"))
-        ->sigClicked().connect(bind(&TimeBarImpl::onPlayActivated, this));
+        ->sigClicked().connect(boost::bind(&TimeBarImpl::onPlayActivated, this));
 
     stopResumeButton = self->addButton(resumeIcon, _("Resume animation"));
     stopResumeButton->setIcon(resumeIcon);
-    stopResumeButton->sigClicked().connect(bind(&TimeBarImpl::onResumeActivated, this));
+    stopResumeButton->sigClicked().connect(boost::bind(&TimeBarImpl::onResumeActivated, this));
 
     self->addButton(QIcon(":/Base/icons/refresh.png"), _("Refresh state at the current time"))
-        ->sigClicked().connect(bind(&TimeBarImpl::onRefreshButtonClicked, this));
+        ->sigClicked().connect(boost::bind(&TimeBarImpl::onRefreshButtonClicked, this));
     
     timeSpin = new DoubleSpinBox();
     timeSpin->setAlignment(Qt::AlignCenter);
-    timeSpin->sigValueChanged().connect(bind(&TimeBarImpl::onTimeSpinChanged, this, _1));
+    timeSpin->sigValueChanged().connect(boost::bind(&TimeBarImpl::onTimeSpinChanged, this, _1));
     self->addWidget(timeSpin);
 
     timeSlider = new Slider(Qt::Horizontal);
-    timeSlider->sigValueChanged().connect(bind(&TimeBarImpl::onTimeSliderChangeValue, this, _1));
+    timeSlider->sigValueChanged().connect(boost::bind(&TimeBarImpl::onTimeSliderChangeValue, this, _1));
+    timeSlider->setMinimumWidth(timeSlider->sizeHint().width());
     self->addWidget(timeSlider);
 
     minTimeSpin = new DoubleSpinBox();
     minTimeSpin->setAlignment(Qt::AlignCenter);
     minTimeSpin->setRange(-999.0, 999.0);
-    minTimeSpin->sigValueChanged().connect(bind(&TimeBarImpl::onTimeRangeSpinsChanged, this));
+    minTimeSpin->sigValueChanged().connect(boost::bind(&TimeBarImpl::onTimeRangeSpinsChanged, this));
     self->addWidget(minTimeSpin);
 
     self->addLabel(" : ");
@@ -300,18 +324,18 @@ TimeBarImpl::TimeBarImpl(TimeBar* self)
     maxTimeSpin = new DoubleSpinBox();
     maxTimeSpin->setAlignment(Qt::AlignCenter);
     maxTimeSpin->setRange(-999.0, 999.0);
-    maxTimeSpin->sigValueChanged().connect(bind(&TimeBarImpl::onTimeRangeSpinsChanged, this));
+    maxTimeSpin->sigValueChanged().connect(boost::bind(&TimeBarImpl::onTimeRangeSpinsChanged, this));
     self->addWidget(maxTimeSpin);
 
-    self->addButton(QIcon(":/Base/icons/setup.png"), _("Open the setup dialog"))
-        ->sigClicked().connect(bind(&QDialog::show, &setup));
+    self->addButton(QIcon(":/Base/icons/setup.png"), _("Show the config dialog"))
+        ->sigClicked().connect(boost::bind(&QDialog::show, &config));
 
-    setup.frameRateSpin->sigValueChanged().connect(bind(&TimeBarImpl::onFrameRateSpinChanged, this));
-    setup.playbackFrameRateSpin->sigValueChanged().connect(bind(&TimeBarImpl::onPlaybackFrameRateChanged, this));
-    setup.playbackSpeedScaleSpin->sigValueChanged().connect(bind(&TimeBarImpl::onPlaybackSpeedScaleChanged, this));
+    config.frameRateSpin.sigValueChanged().connect(boost::bind(&TimeBarImpl::onFrameRateSpinChanged, this));
+    config.playbackFrameRateSpin.sigValueChanged().connect(boost::bind(&TimeBarImpl::onPlaybackFrameRateChanged, this));
+    config.playbackSpeedScaleSpin.sigValueChanged().connect(boost::bind(&TimeBarImpl::onPlaybackSpeedScaleChanged, this));
 
-    playbackSpeedScale = setup.playbackSpeedScaleSpin->value();
-    playbackFrameRate = setup.playbackFrameRateSpin->value();
+    playbackSpeedScale = config.playbackSpeedScaleSpin.value();
+    playbackFrameRate = config.playbackFrameRateSpin.value();
 
     updateTimeProperties(true);
 }
@@ -329,13 +353,13 @@ TimeBarImpl::~TimeBarImpl()
 }
 
 
-SignalProxy< boost::signal<bool(double time), TimeBar::LogicalProduct> > TimeBar::sigPlaybackInitialized()
+SignalProxy<bool(double time), LogicalProduct> TimeBar::sigPlaybackInitialized()
 {
     return impl->sigPlaybackInitialized;
 }
 
 
-SignalProxy< boost::signal<void(double time)> > TimeBar::sigPlaybackStarted()
+SignalProxy<void(double time)> TimeBar::sigPlaybackStarted()
 {
     return impl->sigPlaybackStarted;
 }
@@ -348,13 +372,13 @@ SignalProxy< boost::signal<void(double time)> > TimeBar::sigPlaybackStarted()
    and return false if the time is not valid. The example of the latter case is that
    the time is over the length of the data processed in the function.
 */
-SignalProxy< boost::signal<bool(double time), TimeBar::LogicalSum> > TimeBar::sigTimeChanged()
+SignalProxy<bool(double time), LogicalSum> TimeBar::sigTimeChanged()
 {
     return impl->sigTimeChanged;
 }
 
 
-SignalProxy< boost::signal<void(double time, bool isStoppedManually)> > TimeBar::sigPlaybackStopped()
+SignalProxy<void(double time, bool isStoppedManually)> TimeBar::sigPlaybackStopped()
 {
     return impl->sigPlaybackStopped;
 }
@@ -382,7 +406,10 @@ bool TimeBarImpl::setTime(double time, bool calledFromPlaybackLoop, QWidget* cal
 
     const double newTime = myNearByInt(time * self->frameRate_) / self->frameRate_;
 
-    if(newTime == self->time_){
+    // When the optimization is enabled,
+    // the result of (newTime == self->time_) sometimes becomes false,
+    // so here the following judgement is used.
+    if(fabs(newTime - self->time_) < 1.0e-14){
         if(calledFromPlaybackLoop){
             return true;
         }
@@ -390,6 +417,20 @@ bool TimeBarImpl::setTime(double time, bool calledFromPlaybackLoop, QWidget* cal
             return false;
         }
     }
+
+    if(newTime > maxTime && config.autoExpandCheck.isChecked()){
+        maxTime = newTime;
+        timeSpin->blockSignals(true);
+        timeSlider->blockSignals(true);
+        maxTimeSpin->blockSignals(true);
+        timeSpin->setRange(minTime, maxTime);
+        const double r = pow(10.0, decimals);
+        timeSlider->setRange((int)myNearByInt(minTime * r), (int)myNearByInt(maxTime * r));
+        maxTimeSpin->setValue(maxTime);
+        maxTimeSpin->blockSignals(false);
+        timeSlider->blockSignals(false);
+        timeSpin->blockSignals(false);
+    }
         
     self->time_ = newTime;
 
@@ -403,7 +444,7 @@ bool TimeBarImpl::setTime(double time, bool calledFromPlaybackLoop, QWidget* cal
         timeSlider->setValue((int)myNearByInt(self->time_ * pow(10.0, decimals)));
         timeSlider->blockSignals(false);
     }
-    
+
     return sigTimeChanged(self->time_);
 }
 
@@ -413,6 +454,9 @@ void TimeBarImpl::onTimeSpinChanged(double value)
     if(TRACE_FUNCTIONS){
         cout << "TimeBarImpl::onTimeSpinChanged()" << endl;
     }
+    if(isDoingPlayback){
+        stopPlayback(true);
+    }
     setTime(value, false, timeSpin);
 }
 
@@ -422,6 +466,9 @@ bool TimeBarImpl::onTimeSliderChangeValue(double value)
     if(TRACE_FUNCTIONS){
         cout << "TimeBarImpl::onTimeSliderChanged(): value = " << value << endl;
     }
+    if(isDoingPlayback){
+        stopPlayback(true);
+    }
     setTime(value / pow(10.0, decimals), false, timeSlider);
     return true;
 }
@@ -436,8 +483,10 @@ void TimeBar::setFrameRate(double rate)
 void TimeBarImpl::setFrameRate(double rate)
 {
     if(rate > 0.0){
-        self->frameRate_ = rate;
-        updateTimeProperties(false);
+        if(self->frameRate_ != rate){
+            self->frameRate_ = rate;
+            updateTimeProperties(true);
+        }
     }
 }
 
@@ -463,7 +512,7 @@ void TimeBar::setTimeRange(double min, double max)
 void TimeBarImpl::setTimeRange(double minTime, double maxTime)
 {
     this->minTime = minTime;
-    this->maxTime = std::max(minTime, maxTime);
+    this->maxTime = maxTime;
     updateTimeProperties(false);
 }
 
@@ -474,13 +523,14 @@ void TimeBarImpl::updateTimeProperties(bool forceUpdate)
     timeSlider->blockSignals(true);
     minTimeSpin->blockSignals(true);
     maxTimeSpin->blockSignals(true);
-    setup.frameRateSpin->blockSignals(true);
+    config.frameRateSpin.blockSignals(true);
     
-    double timeStep = 1.0 / self->frameRate_;
+    const double timeStep = 1.0 / self->frameRate_;
     decimals = static_cast<int>(ceil(log10(self->frameRate_)));
-    double r = pow(10.0, decimals);
+    const double r = pow(10.0, decimals);
 
-    if(forceUpdate || (minTime != timeSpin->minimum() || maxTime != timeSpin->maximum())){
+    if(forceUpdate ||
+       (minTime != timeSpin->minimum() || maxTime != timeSpin->maximum())){
         timeSpin->setRange(minTime, maxTime);
         timeSlider->setRange((int)myNearByInt(minTime * r), (int)myNearByInt(maxTime * r));
     }
@@ -490,9 +540,9 @@ void TimeBarImpl::updateTimeProperties(bool forceUpdate)
     timeSlider->setSingleStep(timeStep * r);
     minTimeSpin->setValue(minTime);
     maxTimeSpin->setValue(maxTime);
-    setup.frameRateSpin->setValue(self->frameRate_);
+    config.frameRateSpin.setValue(self->frameRate_);
 
-    setup.frameRateSpin->blockSignals(false);
+    config.frameRateSpin.blockSignals(false);
     maxTimeSpin->blockSignals(false);
     minTimeSpin->blockSignals(false);
     timeSlider->blockSignals(false);
@@ -504,7 +554,7 @@ void TimeBarImpl::updateTimeProperties(bool forceUpdate)
     
 void TimeBarImpl::onPlaybackSpeedScaleChanged()
 {
-    playbackSpeedScale = setup.playbackSpeedScaleSpin->value();
+    playbackSpeedScale = config.playbackSpeedScaleSpin.value();
     
     if(isDoingPlayback){
         startPlayback();
@@ -514,13 +564,19 @@ void TimeBarImpl::onPlaybackSpeedScaleChanged()
 
 double TimeBar::playbackSpeedScale() const
 {
-    return impl->setup.playbackSpeedScaleSpin->value();
+    return impl->config.playbackSpeedScaleSpin.value();
+}
+
+
+void TimeBar::setPlaybackSpeedScale(double scale)
+{
+    impl->config.playbackSpeedScaleSpin.setValue(scale);
 }
 
 
 void TimeBarImpl::onPlaybackFrameRateChanged()
 {
-    playbackFrameRate = setup.playbackFrameRateSpin->value();
+    playbackFrameRate = config.playbackFrameRateSpin.value();
     
     if(isDoingPlayback){
         startPlayback();
@@ -530,7 +586,13 @@ void TimeBarImpl::onPlaybackFrameRateChanged()
 
 double TimeBar::playbackFrameRate() const
 {
-    return impl->setup.playbackFrameRateSpin->value();
+    return impl->config.playbackFrameRateSpin.value();
+}
+
+
+void TimeBar::setPlaybackFrameRate(double rate)
+{
+    impl->config.playbackFrameRateSpin.setValue(rate);
 }
 
 
@@ -584,8 +646,12 @@ void TimeBarImpl::startPlayback()
             const static QString tip(_("Stop animation"));
             stopResumeButton->setIcon(stopIcon);
             stopResumeButton->setToolTip(tip);
-
-            timerId = startTimer((int)myNearByInt(1000.0 / playbackFrameRate));
+            if(config.idleLoopDrivenCheck.isChecked()){
+                timerId = startTimer(0);
+            } else {
+                timerId = startTimer((int)myNearByInt(1000.0 / playbackFrameRate));
+            }
+            
             timer.start();
         }
     }
@@ -688,14 +754,40 @@ void TimeBarImpl::stopFillLevelUpdate(int id)
 }
 
 
+void TimeBar::setFillLevelSync(bool on)
+{
+    impl->config.fillLevelSyncCheck.setChecked(on);
+}
+
+
+void TimeBar::startPlaybackFromFillLevel()
+{
+    if(isDoingPlayback()){
+        stopPlayback();
+    }
+    setTime(impl->fillLevel);
+    startPlayback();
+}
+
+
+double TimeBar::realPlaybackTime() const
+{
+    if(impl->isDoingPlayback){
+        return impl->animationTimeOffset + impl->playbackSpeedScale * (impl->timer.elapsed() / 1000.0);
+    } else {
+        return time_;
+    }
+}
+
+
 void TimeBarImpl::timerEvent(QTimerEvent* event)
 {
     double time = animationTimeOffset + playbackSpeedScale * (timer.elapsed() / 1000.0);
 
     bool doStopAtLastFillLevel = false;
     if(isFillLevelActive){
-        if(time > fillLevel){
-            animationTimeOffset -= (time - fillLevel);
+        if(config.fillLevelSyncCheck.isChecked() || (time > fillLevel)){
+            animationTimeOffset += (fillLevel - time);
             time = fillLevel;
             if(fillLevelMap.empty()){
                 doStopAtLastFillLevel = true;
@@ -722,7 +814,7 @@ void TimeBarImpl::onTimeRangeSpinsChanged()
 
 void TimeBarImpl::onFrameRateSpinChanged()
 {
-    setFrameRate(setup.frameRateSpin->value());
+    setFrameRate(config.frameRateSpin.value());
 }
 
 
@@ -734,21 +826,11 @@ void TimeBarImpl::onRefreshButtonClicked()
 }
 
 
-QSize TimeBar::minimumSizeHint () const
+int TimeBar::stretchableDefaultWidth() const
 {
-    QSize s = ToolBar::minimumSizeHint();
-    s.setWidth(s.width() * 2);
-    return s;
+    return sizeHint().width() + impl->timeSlider->sizeHint().width() * 5;
 }
-
-
-QSize TimeBar::sizeHint () const
-{
-    QSize s = ToolBar::minimumSizeHint();
-    s.setWidth(s.width() * 3);
-    return s;
-}
-
+    
 
 bool TimeBar::storeState(Archive& archive)
 {
@@ -761,7 +843,12 @@ bool TimeBarImpl::storeState(Archive& archive)
     archive.write("minTime", minTime);
     archive.write("maxTime", maxTime);
     archive.write("frameRate", self->frameRate_);
+    archive.write("playbackFrameRate", playbackFrameRate);
+    archive.write("idleLoopDrivenMode", config.idleLoopDrivenCheck.isChecked());
     archive.write("currentTime", self->time_);
+    archive.write("speedScale", playbackSpeedScale);
+    archive.write("syncToOngoingUpdates", config.fillLevelSyncCheck.isChecked());
+    archive.write("autoExpansion", config.autoExpandCheck.isChecked());
     return true;
 }
 
@@ -776,10 +863,18 @@ bool TimeBarImpl::restoreState(const Archive& archive)
 {
     archive.read("minTime", minTime);
     archive.read("maxTime", maxTime);
-    archive.read("frameRate", self->frameRate_);
     archive.read("currentTime", self->time_);
 
-    updateTimeProperties(false);
+    config.playbackFrameRateSpin.setValue(archive.get("playbackFrameRate", playbackFrameRate));
+    config.idleLoopDrivenCheck.setChecked(archive.get("idleLoopDrivenMode", config.idleLoopDrivenCheck.isChecked()));
+    config.playbackSpeedScaleSpin.setValue(archive.get("speedScale", playbackSpeedScale));
+    config.fillLevelSyncCheck.setChecked(archive.get("syncToOngoingUpdates", config.fillLevelSyncCheck.isChecked()));
+    config.autoExpandCheck.setChecked(archive.get("autoExpansion", config.autoExpandCheck.isChecked()));
+
+    double prevFrameRate = self->frameRate_;
+    archive.read("frameRate", self->frameRate_);
+
+    updateTimeProperties(self->frameRate_ != prevFrameRate);
     
     return true;
 }
