@@ -7,106 +7,122 @@
 #include "ToolBar.h"
 #include "MainWindow.h"
 #include "LazyCaller.h"
-#include <cnoid/YamlNodes>
+#include "Menu.h"
+#include <cnoid/ValueTree>
 #include <QResizeEvent>
+#include <QApplication>
 #include <boost/bind.hpp>
 #include <boost/shared_ptr.hpp>
 #include <deque>
 #include <vector>
 #include <list>
+#include <set>
 #include <limits>
 #include <algorithm>
-
 #include <iostream>
 
 using namespace std;
-using namespace boost;
 using namespace cnoid;
 
 namespace {
 
-    const bool DEBUG_MODE = false;
+const bool DEBUG_MODE = false;
 
-    typedef std::vector<ToolBar*> ToolBarList;
-        
-    class ToolBarRow
-    {
-    public:
-        ToolBarRow(ToolBarArea* toolBarArea) :
-            toolBarArea(toolBarArea),
-            separator(toolBarArea) {
-            height = 0;
-            separator.hide();
-        }
-
-        ToolBarArea* toolBarArea;
-        ToolBarList toolBars;
-        HSeparator separator;
-        int height;
-    };
-    typedef shared_ptr<ToolBarRow> ToolBarRowPtr;
+typedef std::vector<ToolBar*> ToolBarArray;
 
 
-    // for archiving
-    struct LayoutState
-    {
-    public:
-        string name;
-        int desiredX;
-        int layoutPriority;
-    };
+class ToolBarRow
+{
+public:
+    ToolBarRow(ToolBarArea* toolBarArea) :
+        toolBarArea(toolBarArea),
+        separator(toolBarArea) {
+        height = 0;
+        separator.hide();
+    }
 
-    typedef list<LayoutState> RowLayoutState;
-    typedef list<RowLayoutState> AreaLayoutState;
+    ToolBarArea* toolBarArea;
+    ToolBarArray toolBars;
+    HSeparator separator;
+    int height;
+};
+typedef boost::shared_ptr<ToolBarRow> ToolBarRowPtr;
+
+
+// for archiving
+struct LayoutState
+{
+public:
+    string name;
+    int desiredX;
+    int layoutPriority;
+};
+
+typedef list<LayoutState> RowLayoutState;
+typedef list<RowLayoutState> AreaLayoutState;
 }
 
 
 namespace cnoid {
 
-    class ToolBarAreaImpl
-    {
-    public:
-        ToolBarAreaImpl(ToolBarArea* self);
-        ~ToolBarAreaImpl();
+class ToolBarAreaImpl
+{
+public:
+    ToolBarAreaImpl(ToolBarArea* self);
+    ~ToolBarAreaImpl();
 
-        void getAllToolBars(std::vector<ToolBar*>& bars);
-        void storeLayout(YamlMappingPtr layout);
-        void restoreLayout(const YamlMappingPtr layout);
-        bool addToolBar(ToolBar* toolBar);
-        void removeToolBar(ToolBar* toolBar);
-        void dragToolBar(ToolBar* toolBar, const QPoint& globalPos);
+    bool addToolBar(ToolBar* toolBar);
+    void removeToolBar(ToolBar* toolBar);
+    void hideToolBar(ToolBar* toolBar);
+    void setVisibilityMenuItems(Menu* menu);
+    void onVisiblityCheckToggled(ToolBar* toolBar, bool on);
+    void storeLayout(MappingPtr& archive);
+    void restoreLayout(MappingPtr& archive);
+    void resetLayout(MappingPtr& archive);
+    void dragToolBar(ToolBar* toolBar, const QPoint& globalPos);
         
-        void setNewToolBars();
-        void expandStrechableBars
-        (ToolBarRowPtr& row, int portion, int shift, int barIndex, int numStrechables, int lastSpace);
+    void setNewToolBars();
+    void expandStrechableBars
+    (ToolBarRowPtr& row, int portion, int shift, int barIndex, int numStrechables, int lastSpace);
 
-        void useSpace(ToolBarRowPtr& row, int space, int shift, int barIndex, int numStrechables, int allspace );
-        void setNewToolBar(ToolBar* toolBar, vector<int>& numStrechablesOfRow);
-        void layoutToolBars();
-        void layoutToolBarRow(ToolBarRowPtr toolBarRow, int& io_rowTop, bool isNewBottomRow);
-        void layoutToolBarRowWithDraggedToolBar(ToolBarList& toolBars, int rowTop, int rowHeight);
-        int normalizeLayoutPriorities(ToolBarList& toolBars);
-        void layoutToolBarRowPart(ToolBarList& toolBars, int rowTop, int rowHeight, int partLeft, int partRight);
+    void useSpace(ToolBarRowPtr& row, int space, int shift, int barIndex, int numStrechables, int allspace );
+    void setNewToolBar(ToolBar* toolBar, vector<int>& numStrechablesOfRow);
+    void layoutToolBars();
+    void layoutToolBarRow(ToolBarRowPtr toolBarRow, int& io_rowTop, bool isNewBottomRow);
+    void layoutToolBarRowWithDraggedToolBar(ToolBarArray& toolBars, int rowTop, int rowHeight);
+    int normalizeLayoutPriorities(ToolBarArray& toolBars);
+    void layoutToolBarRowPart(ToolBarArray& toolBars, int rowTop, int rowHeight, int partLeft, int partRight);
 
-        void resizeEvent(QResizeEvent* event);
+    void resizeEvent(QResizeEvent* event);
 
-        ToolBarArea* self;
+    ToolBarArea* self;
 
-        bool isBeforeDoingInitialLayout;
-        ToolBarList newToolBars;
-        deque<ToolBarRowPtr> toolBarRows;
-        ToolBarRowPtr spilledToolBarRow;
-        YamlMappingPtr initialLayout;
-
-        int areaHeight;
-        int prevAreaHeight;
-
-        ToolBar* draggedToolBar;
-        bool draggedToolBarHasNotBeenInserted;
-        int dragY;
-
-        LazyCaller layoutToolBarsCaller;
+    bool isBeforeDoingInitialLayout;
+    
+    int defaultOrderIndex;
+    
+    struct DefaultOrderCmp {
+        bool operator() (ToolBar* bar1, ToolBar* bar2) {
+            return (bar1->defaultOrderIndex < bar2->defaultOrderIndex);
+        }
     };
+        
+    set<ToolBar*> toolBars;
+    ToolBarArray newToolBarsToShow;
+    deque<ToolBarRowPtr> toolBarRows;
+    ToolBarRowPtr spilledToolBarRow;
+    MappingPtr initialLayout;
+
+    int areaHeight;
+    int prevAreaHeight;
+
+    ToolBar* draggedToolBar;
+    bool draggedToolBarHasNotBeenInserted;
+    int dragY;
+
+    LazyCaller layoutToolBarsLater;
+};
+
 }
 
 
@@ -120,9 +136,10 @@ ToolBarArea::ToolBarArea(QWidget* parent)
 ToolBarAreaImpl::ToolBarAreaImpl(ToolBarArea* self)
     : self(self),
       spilledToolBarRow(new ToolBarRow(self)),
-      layoutToolBarsCaller(bind(&ToolBarAreaImpl::layoutToolBars, this))
+      layoutToolBarsLater(boost::bind(&ToolBarAreaImpl::layoutToolBars, this))
 {
     isBeforeDoingInitialLayout = true;
+    defaultOrderIndex = 0;
     
     draggedToolBar = 0;
     draggedToolBarHasNotBeenInserted = false;
@@ -144,23 +161,24 @@ ToolBarAreaImpl::~ToolBarAreaImpl()
 }
 
 
-std::vector<ToolBar*> ToolBarArea::getAllToolBars()
+void ToolBarArea::getAllToolBars(std::vector<ToolBar*>& out_toolBars)
 {
-    std::vector<ToolBar*> bars;
-    impl->getAllToolBars(bars);
-    return bars;
+    out_toolBars.clear();
+    for(set<ToolBar*>::iterator p = impl->toolBars.begin(); p != impl->toolBars.end(); ++p){
+        out_toolBars.push_back(*p);
+    }
 }
 
 
-void ToolBarAreaImpl::getAllToolBars(std::vector<ToolBar*>& bars)
+void ToolBarArea::getVisibleToolBars(std::vector<ToolBar*>& out_toolBars)
 {
-    bars.clear();
+    out_toolBars.clear();
     
-    for(size_t i=0; i < toolBarRows.size(); ++i){
-        ToolBarRowPtr& row = toolBarRows[i];
+    for(size_t i=0; i < impl->toolBarRows.size(); ++i){
+        ToolBarRowPtr& row = impl->toolBarRows[i];
         vector<ToolBar*>& toolBars = row->toolBars;
         for(size_t j=0; j < toolBars.size(); ++j){
-            bars.push_back(toolBars[j]);
+            out_toolBars.push_back(toolBars[j]);
         }
     }
 }
@@ -174,13 +192,25 @@ bool ToolBarArea::addToolBar(ToolBar* toolBar)
 
 bool ToolBarAreaImpl::addToolBar(ToolBar* toolBar)
 {
+    if(DEBUG_MODE){
+        cout << "ToolBarAreaImpl::addToolBar()" << endl;
+    }
+        
     if(toolBar){
-        if(!toolBar->toolBarArea()){
-            toolBar->toolBarArea_ = self;
-            newToolBars.push_back(toolBar);
+        set<ToolBar*>::iterator p = toolBars.find(toolBar);
+        if(p == toolBars.end()){
+            toolBars.insert(toolBar);
 
+            toolBar->toolBarArea_ = self;
+            toolBar->defaultOrderIndex = defaultOrderIndex++;
+            toolBar->hide();
+            toolBar->setParent(self);
+
+            if(toolBar->isVisibleByDefault()){
+                newToolBarsToShow.push_back(toolBar);
+            }
             if(!isBeforeDoingInitialLayout){
-                layoutToolBarsCaller.request();
+                layoutToolBarsLater();
             }
             return true;
         }
@@ -199,30 +229,80 @@ void ToolBarArea::removeToolBar(ToolBar* toolBar)
 
 void ToolBarAreaImpl::removeToolBar(ToolBar* toolBar)
 {
-    for(size_t i=0; i < toolBarRows.size(); ++i){
-        ToolBarRowPtr row = toolBarRows[i];
-        vector<ToolBar*>& toolBars = row->toolBars;
-        for(size_t j=0; j < toolBars.size(); ++j){
-            if(toolBars[j] == toolBar){
-                toolBar->hide();
-                toolBar->setParent(0);
-                toolBar->toolBarArea_ = 0;
-                toolBars.erase(toolBars.begin() + j);
+    set<ToolBar*>::iterator p = toolBars.find(toolBar);
+    if(p != toolBars.end()){
+        hideToolBar(toolBar);
+        toolBar->setParent(0);
+        toolBar->toolBarArea_ = 0;
+        toolBars.erase(p);
+    }
+}
 
-                if(!isBeforeDoingInitialLayout){
-                    layoutToolBarsCaller.request();
+
+void ToolBarAreaImpl::hideToolBar(ToolBar* toolBar)
+{
+    if(toolBar->isVisible()){
+        for(size_t i=0; i < toolBarRows.size(); ++i){
+            ToolBarRowPtr row = toolBarRows[i];
+            vector<ToolBar*>& toolBars = row->toolBars;
+            for(size_t j=0; j < toolBars.size(); ++j){
+                if(toolBars[j] == toolBar){
+                    toolBar->hide();
+                    toolBars.erase(toolBars.begin() + j);
+                    if(!isBeforeDoingInitialLayout){
+                        layoutToolBarsLater();
+                    }
+                    return;
                 }
-                return;
             }
         }
     }
 }
 
 
-void ToolBarArea::setInitialLayout(YamlMappingPtr layout)
+void ToolBarArea::setVisibilityMenuItems(Menu* menu)
+{
+    impl->setVisibilityMenuItems(menu);
+}
+
+
+void ToolBarAreaImpl::setVisibilityMenuItems(Menu* menu)
+{
+    menu->clear();
+
+    vector<ToolBar*> sorted(toolBars.size());
+    std::copy(toolBars.begin(), toolBars.end(), sorted.begin());
+    std::sort(sorted.begin(), sorted.end(), DefaultOrderCmp());
+    
+    for(size_t i=0; i < sorted.size(); ++i){
+        ToolBar* toolBar = sorted[i];
+        Action* action = new Action(menu);
+        action->setText(toolBar->windowTitle());
+        action->setCheckable(true);
+        if(toolBar->isVisible()){
+            action->setChecked(true);
+        }
+        action->sigToggled().connect(boost::bind(&ToolBarAreaImpl::onVisiblityCheckToggled, this, toolBar, _1));
+        menu->addAction(action);
+    }
+}
+
+
+void ToolBarAreaImpl::onVisiblityCheckToggled(ToolBar* toolBar, bool on)
+{
+    if(on){
+        newToolBarsToShow.push_back(toolBar);
+        layoutToolBarsLater();
+    } else {
+        hideToolBar(toolBar);
+    }
+}
+
+
+void ToolBarArea::setInitialLayout(MappingPtr archive)
 {
     if(impl->isBeforeDoingInitialLayout){
-        impl->initialLayout = layout;
+        impl->initialLayout = archive;
     }
 }
 
@@ -239,11 +319,19 @@ void ToolBarAreaImpl::resizeEvent(QResizeEvent* event)
     if(DEBUG_MODE){
         cout << "ToolBarAreaImpl::resizeEvent(" <<
             event->size().width() << ", " << event->size().height() << ")";
-        cout << ", isVisible =" << self->isVisible() << endl;
+        cout << ", isVisible =" << self->isVisible();
+        cout << ", MainWindows's windowState() = " << MainWindow::instance()->windowState() << endl;
     }
 
     if(isBeforeDoingInitialLayout){
-        if(!(MainWindow::instance()->windowState() & (Qt::WindowMaximized | Qt::WindowFullScreen)) || self->isVisible()){
+#ifdef Q_OS_MAC
+        bool doLayout = true;
+#else
+        bool doLayout =
+            !(MainWindow::instance()->windowState() & (Qt::WindowMaximized | Qt::WindowFullScreen)) ||
+            self->isVisible();
+#endif
+        if(doLayout){
             isBeforeDoingInitialLayout = false;
             if(initialLayout){
                 restoreLayout(initialLayout);
@@ -253,17 +341,33 @@ void ToolBarAreaImpl::resizeEvent(QResizeEvent* event)
             }
         }
     } else {
-        if(event->size().width() != event->oldSize().width()){
+        if(event->oldSize().width() >= 0 && event->size().width() != event->oldSize().width()){
             layoutToolBars();
         }
     }
 }
 
 
+bool ToolBarArea::event(QEvent* event)
+{
+    if(false){ // Is this needed?
+        if(event->type() == QEvent::LayoutRequest){
+            if(DEBUG_MODE){
+                cout << "ToolBarArea::event(QEvent::LayoutRequest)" << endl;
+            }
+            impl->layoutToolBars();
+            return true;
+        }
+    }
+    return QWidget::event(event);
+}
+
+
 void ToolBarArea::doInitialLayout()
 {
     if(DEBUG_MODE){
         cout << "ToolBarAreaImpl::doInitialLayout()" << endl;
+        cout << "width = " << width() << endl;
     }
     
     if(impl->isBeforeDoingInitialLayout){
@@ -278,105 +382,57 @@ void ToolBarArea::doInitialLayout()
 }
 
 
-bool ToolBarArea::event(QEvent* event)
+void ToolBarArea::restoreLayout(MappingPtr archive)
 {
-    if(event->type() == QEvent::LayoutRequest){
-        if(DEBUG_MODE){
-            cout << "ToolBarArea::event(QEvent::LayoutRequest)" << endl;
-        }
-        impl->layoutToolBars();
-        return true;
-    }
-    return QWidget::event(event);
+    impl->restoreLayout(archive);
 }
 
 
-void ToolBarArea::storeLayout(YamlMappingPtr layout)
-{
-    impl->storeLayout(layout);
-}
-
-
-void ToolBarAreaImpl::storeLayout(YamlMappingPtr layout)
-{
-    YamlMapping* layoutOfToolBars = layout->createMapping("layoutOfToolBars");
-    YamlSequence* rows = layoutOfToolBars->createSequence("rows");
-    
-    for(size_t i=0; i < toolBarRows.size(); ++i){
-        ToolBarList& toolBars = toolBarRows[i]->toolBars;
-        if(!toolBars.empty()){
-            YamlSequence* bars = new YamlSequence();
-            for(ToolBarList::iterator p = toolBars.begin(); p != toolBars.end(); ++p){
-                ToolBar* toolBar = *p;
-                YamlMapping* state = new YamlMapping();
-                state->setFlowStyle(true);
-                state->write("name", toolBar->objectName().toStdString(), YAML_DOUBLE_QUOTED);
-                state->write("x", toolBar->desiredX);
-                state->write("priority", toolBar->layoutPriority);
-                bars->append(state);
-            }
-            rows->append(bars);
-        }
-    }
-}
-
-
-void ToolBarArea::restoreLayout(const YamlMappingPtr layout)
-{
-    impl->restoreLayout(layout);
-}
-
-
-void ToolBarAreaImpl::restoreLayout(const YamlMappingPtr layout)
+void ToolBarAreaImpl::restoreLayout(MappingPtr& archive)
 {
     if(DEBUG_MODE){
         cout << "ToolBarAreaImpl::restoreLayout()" << endl;
     }
 
     if(isBeforeDoingInitialLayout){
-        initialLayout = layout;
+        initialLayout = archive;
         return;
     }
-       
-    const YamlMappingPtr layoutOfToolBars = layout->findMapping("layoutOfToolBars");
+
+    const MappingPtr layoutOfToolBars = archive->findMapping("layoutOfToolBars");
     if(!layoutOfToolBars->isValid()){
         layoutToolBars();
         return;
     }
-    
+
     // make the map from name to toolBar
-    typedef map<QString, ToolBar*> ToolBarMap;
-    ToolBarMap toolBarMap;
-    for(size_t i=0; i < toolBarRows.size(); ++i){
-        ToolBarList& toolBars = toolBarRows[i]->toolBars;
-        for(ToolBarList::iterator q = toolBars.begin(); q != toolBars.end(); ++q){
-            ToolBar* toolBar = *q;
-            toolBarMap[toolBar->windowTitle()] = toolBar;
-        }
+    typedef map<QString, ToolBar*> NameToToolBarMap;
+    NameToToolBarMap nameToToolBarMap;
+    for(set<ToolBar*>::iterator p = toolBars.begin(); p != toolBars.end(); ++p){
+        ToolBar* toolBar = *p;
+        toolBar->hide();
+        nameToToolBarMap[toolBar->objectName()] = toolBar;
     }
+
+    newToolBarsToShow.clear();
+    
     toolBarRows.clear();
 
-    for(size_t i=0; i < newToolBars.size(); ++i){
-        toolBarMap[newToolBars[i]->windowTitle()] = newToolBars[i];
-        newToolBars[i]->setParent(self);
-    }
-    newToolBars.clear();
-
-    const YamlSequence* rows = layoutOfToolBars->get("rows").toSequence();
+    const Listing* rows = layoutOfToolBars->get("rows").toListing();
 
     for(int i=0; i < rows->size(); ++i){
         ToolBarRowPtr row(new ToolBarRow(self));
         toolBarRows.push_back(row);
-        const YamlSequence* bars = rows->get(i).toSequence();
+        const Listing* bars = rows->at(i)->toListing();
         for(int j=0; j < bars->size(); ++j){
-            const YamlMapping* state = bars->get(j).toMapping();
-            ToolBarMap::iterator it = toolBarMap.find(QString(state->get("name").toString().c_str()));
-            if(it != toolBarMap.end()){
+            const Mapping* state = bars->at(j)->toMapping();
+            NameToToolBarMap::iterator it = nameToToolBarMap.find(QString(state->get("name").toString().c_str()));
+            if(it != nameToToolBarMap.end()){
                 ToolBar* toolBar = it->second;
-                toolBarMap.erase(it);
                 row->toolBars.push_back(toolBar);
                 toolBar->desiredX = state->get("x").toInt();
                 toolBar->layoutPriority = state->get("priority").toInt();
+                nameToToolBarMap.erase(it);
             }
         }
         if(row->toolBars.empty()){
@@ -384,14 +440,74 @@ void ToolBarAreaImpl::restoreLayout(const YamlMappingPtr layout)
         }
     }
 
-    for(ToolBarMap::iterator p = toolBarMap.begin(); p != toolBarMap.end(); ++p){
-        newToolBars.push_back(p->second);
-    }
-
     layoutToolBars();
 }
 
 
+void ToolBarArea::resetLayout(MappingPtr archive)
+{
+    impl->resetLayout(archive);
+}
+
+
+void ToolBarAreaImpl::resetLayout(MappingPtr& archive)
+{
+    if(DEBUG_MODE){
+        cout << "ToolBarAreaImpl::resetLayout()" << endl;
+    }
+    newToolBarsToShow.clear();
+    for(set<ToolBar*>::iterator p = toolBars.begin(); p != toolBars.end(); ++p){
+        ToolBar* toolBar = *p;
+        toolBar->hide();
+        if(toolBar->isVisibleByDefault()){
+            newToolBarsToShow.push_back(toolBar);
+        }
+    }
+    std::sort(newToolBarsToShow.begin(), newToolBarsToShow.end(), DefaultOrderCmp());
+    
+    toolBarRows.clear();
+    layoutToolBars();
+    
+    self->removeLayout(archive);
+}
+
+
+void ToolBarArea::removeLayout(MappingPtr archive)
+{
+    archive->remove("layoutOfToolBars");
+}
+
+
+void ToolBarArea::storeLayout(MappingPtr archive)
+{
+    impl->storeLayout(archive);
+}
+
+
+void ToolBarAreaImpl::storeLayout(MappingPtr& archive)
+{
+    Mapping* layoutOfToolBars = archive->createMapping("layoutOfToolBars");
+    Listing* rows = layoutOfToolBars->createListing("rows");
+    
+    for(size_t i=0; i < toolBarRows.size(); ++i){
+        ToolBarArray& toolBars = toolBarRows[i]->toolBars;
+        if(!toolBars.empty()){
+            Listing* bars = new Listing();
+            for(ToolBarArray::iterator p = toolBars.begin(); p != toolBars.end(); ++p){
+                ToolBar* toolBar = *p;
+                Mapping* state = new Mapping();
+                state->setFlowStyle(true);
+                state->write("name", toolBar->objectName().toStdString(), DOUBLE_QUOTED);
+                state->write("x", toolBar->desiredX);
+                state->write("priority", toolBar->layoutPriority);
+                bars->append(state);
+            }
+            rows->append(bars);
+        }
+    }
+}
+
+
 void ToolBarArea::dragToolBar(ToolBar* toolBar, const QPoint& globalPos)
 {
     impl->dragToolBar(toolBar, globalPos);
@@ -421,10 +537,10 @@ void ToolBarAreaImpl::setNewToolBars()
 {
     vector<int> numStrechablesOfRow;
 
-    for(size_t i=0; i < newToolBars.size(); ++i){
-        setNewToolBar(newToolBars[i], numStrechablesOfRow);
+    for(size_t i=0; i < newToolBarsToShow.size(); ++i){
+        setNewToolBar(newToolBarsToShow[i], numStrechablesOfRow);
     }
-    newToolBars.clear();
+    newToolBarsToShow.clear();
 
     for(size_t i=0; i < numStrechablesOfRow.size(); ++i){
         int numStrechables = numStrechablesOfRow[i];
@@ -481,7 +597,7 @@ void ToolBarAreaImpl::setNewToolBar(ToolBar* toolBar, vector<int>& numStrechable
                 if(width <= lastSpace){
                     toolBar->desiredX = lastX + 1;
                     if(toolBar->isStretchable()){
-                        width = std::min(toolBar->sizeHint().width(), lastSpace);
+                        width = std::min(toolBar->stretchableDefaultWidth(), lastSpace);
                     }
                     toolBarRow = existingRow;
                     rowIndex = i;
@@ -501,8 +617,7 @@ void ToolBarAreaImpl::setNewToolBar(ToolBar* toolBar, vector<int>& numStrechable
         if(toolBar->isStretchable()){
             numStrechablesOfRow[rowIndex]++;
         }
-        toolBar->setParent(self);
-        toolBar->setGeometry(toolBar->desiredX, 0, width, toolBar->sizeHint().height());
+        toolBar->setGeometry(toolBar->desiredX, 0, width, toolBar->minimumSizeHint().height());
         toolBar->show();
     }
 }
@@ -514,7 +629,7 @@ void ToolBarAreaImpl::layoutToolBars()
         cout << "ToolBarAreaImpl::layoutToolBars()" << endl;
     }
 
-    if(!newToolBars.empty()){
+    if(!newToolBarsToShow.empty()){
         setNewToolBars();
     }
 
@@ -559,17 +674,17 @@ void ToolBarAreaImpl::layoutToolBarRow(ToolBarRowPtr toolBarRow, int& io_rowTop,
     int rowHeight = 0;
 
     bool draggedToolBarExists = false;
-    ToolBarList& toolBars = toolBarRow->toolBars;
+    ToolBarArray& toolBars = toolBarRow->toolBars;
 
     // calculate the row height and remove the dragged tool bar if it is originally in this row
-    ToolBarList::iterator p = toolBars.begin();
+    ToolBarArray::iterator p = toolBars.begin();
     while(p != toolBars.end()){
         ToolBar* toolBar = *p;
         if(toolBar == draggedToolBar){
             draggedToolBarExists = true;
             p = toolBars.erase(p);
         } else {
-            rowHeight = std::max(rowHeight, toolBar->sizeHint().height());
+            rowHeight = std::max(rowHeight, toolBar->minimumSizeHint().height());
             ++p;
         }
     }
@@ -580,14 +695,14 @@ void ToolBarAreaImpl::layoutToolBarRow(ToolBarRowPtr toolBarRow, int& io_rowTop,
     if(draggedToolBarHasNotBeenInserted){
         int bottomBorder = 0;
         if(rowHeight == 0 && draggedToolBarExists){
-            int h = draggedToolBar->sizeHint().height();
+            int h = draggedToolBar->minimumSizeHint().height();
             bottomBorder = io_rowTop + h * 4 / 5;
         } else {
             bottomBorder = io_rowTop + rowHeight;
         }
         if(dragY < bottomBorder || isNewBottomRow){
             isDraggedToolBarMovingToThisRow = true;
-            rowHeight = std::max(rowHeight, draggedToolBar->sizeHint().height());
+            rowHeight = std::max(rowHeight, draggedToolBar->minimumSizeHint().height());
             layoutToolBarRowWithDraggedToolBar(toolBars, io_rowTop, rowHeight);
         }
     }
@@ -599,13 +714,6 @@ void ToolBarAreaImpl::layoutToolBarRow(ToolBarRowPtr toolBarRow, int& io_rowTop,
             layoutToolBarRowPart(toolBars, io_rowTop, rowHeight, 0, self->width());
         }
 
-        for(ToolBarList::iterator p = toolBars.begin(); p != toolBars.end(); ++p){
-            ToolBar* toolBar = *p;
-            if(toolBar->height() != rowHeight){
-                toolBar->setGeometry(toolBar->x(), io_rowTop, toolBar->width(), rowHeight);
-            }
-        }
-    
         io_rowTop += rowHeight;
 
         HSeparator& sep = toolBarRow->separator;
@@ -617,7 +725,7 @@ void ToolBarAreaImpl::layoutToolBarRow(ToolBarRowPtr toolBarRow, int& io_rowTop,
 }
 
 
-void ToolBarAreaImpl::layoutToolBarRowWithDraggedToolBar(ToolBarList& toolBars, int rowTop, int rowHeight)
+void ToolBarAreaImpl::layoutToolBarRowWithDraggedToolBar(ToolBarArray& toolBars, int rowTop, int rowHeight)
 {
     int maxPriorty = normalizeLayoutPriorities(toolBars);
     draggedToolBar->layoutPriority = maxPriorty + 1;
@@ -643,13 +751,13 @@ void ToolBarAreaImpl::layoutToolBarRowWithDraggedToolBar(ToolBarList& toolBars,
 /**
    @return The maximum priority in the normalized priorities
 */
-int ToolBarAreaImpl::normalizeLayoutPriorities(ToolBarList& toolBars)
+int ToolBarAreaImpl::normalizeLayoutPriorities(ToolBarArray& toolBars)
 {
     int maxPriority = 0;
 
     if(!toolBars.empty()){
 
-        ToolBarList sortedToolBars(toolBars);
+        ToolBarArray sortedToolBars(toolBars);
         std::sort(sortedToolBars.begin(), sortedToolBars.end(), ToolBar::LayoutPriorityCmp());
 
         int prevOldPriority = -1;
@@ -672,7 +780,7 @@ int ToolBarAreaImpl::normalizeLayoutPriorities(ToolBarList& toolBars)
 
 
 void ToolBarAreaImpl::layoutToolBarRowPart
-(ToolBarList& toolBars, int rowTop, int rowHeight, int partLeft, int partRight)
+(ToolBarArray& toolBars, int rowTop, int rowHeight, int partLeft, int partRight)
 {
     // find the index of the tool bar with the maximum priority
     int pivotIndex = 0;
@@ -684,14 +792,14 @@ void ToolBarAreaImpl::layoutToolBarRowPart
         }
     }
 
-    ToolBarList leftPartToolBars;
+    ToolBarArray leftPartToolBars;
     int pivotAreaLeft = partLeft;
     for(int i=0; i < pivotIndex; ++i){
         leftPartToolBars.push_back(toolBars[i]);
         pivotAreaLeft += toolBars[i]->minimumSizeHint().width();
     }
 
-    ToolBarList rightPartToolBars;
+    ToolBarArray rightPartToolBars;
     int pivotAreaRight = partRight;
     for(size_t i = pivotIndex + 1; i < toolBars.size(); ++i){
         rightPartToolBars.push_back(toolBars[i]);
@@ -741,5 +849,3 @@ void ToolBarAreaImpl::layoutToolBarRowPart
     pivotToolBar->setGeometry(x, y, width, height);
     pivotToolBar->show();
 }
-
-
