@@ -16,9 +16,8 @@
 #define NOMINMAX
 #endif
 
-#include "World.h"
-#include "Body.h"
-#include "Link.h"
+#include "ABMWorld.h"
+#include "ABMBody.h"
 #include "LinkTraverse.h"
 #include "ForwardDynamicsCBM.h"
 #include "ConstraintForceSolver.h"
@@ -36,7 +35,6 @@
 
 
 using namespace std;
-using namespace boost;
 using namespace tvmet;
 using namespace boost::numeric::ublas;
 using namespace cnoid;
@@ -110,226 +108,226 @@ static const bool CFS_PUT_NUM_CONTACT_POINTS = false;
 
 namespace cnoid
 {
-    class CFSImpl
-    {
-    public:
+class CFSImpl
+{
+public:
 
-        CFSImpl(WorldBase& world);
+    CFSImpl(WorldBase& world);
 
-        bool addCollisionCheckLinkPair
-        (int bodyIndex1, Link* link1, int bodyIndex2, Link* link2, double muStatic, double muDynamic, double culling_thresh, double epsilon);
+    bool addCollisionCheckLinkPair
+    (int bodyIndex1, Link* link1, int bodyIndex2, Link* link2, double muStatic, double muDynamic, double culling_thresh, double epsilon);
 
-        void initialize(void);
-        void solve(CollisionSequence& corbaCollisionSequence);
-        inline void clearExternalForces();
+    void initialize(void);
+    void solve(CollisionSequence& corbaCollisionSequence);
+    inline void clearExternalForces();
 
 #undef PI
 #ifdef __WIN32__       // Visual C++ bug
-        static const double PI;
-        static const double PI_2;
+    static const double PI;
+    static const double PI_2;
 #elif defined(__APPLE__)
 #define PI M_PI
 #define PI_2 M_PI/2
 #else
-        static const double PI   = 3.14159265358979323846;
-        static const double PI_2 = 1.57079632679489661923;
+    static const double PI   = 3.14159265358979323846;
+    static const double PI_2 = 1.57079632679489661923;
 #endif
 
-        WorldBase& world;
+    WorldBase& world;
 
-        bool isConstraintForceOutputMode;
-        bool useBuiltinCollisionDetector;
+    bool isConstraintForceOutputMode;
+    bool useBuiltinCollisionDetector;
 
-        struct ConstraintPoint {
-            int globalIndex;
-            Vector3 point;
-            Vector3 normalTowardInside[2];
-            Vector3 defaultAccel[2];
-            double normalProjectionOfRelVelocityOn0;
-            double depth; // position error in the case of a connection point
+    struct ConstraintPoint {
+        int globalIndex;
+        Vector3 point;
+        Vector3 normalTowardInside[2];
+        Vector3 defaultAccel[2];
+        double normalProjectionOfRelVelocityOn0;
+        double depth; // position error in the case of a connection point
 
-            double mu;
-            Vector3 relVelocityOn0;
-            int globalFrictionIndex;
-            int numFrictionVectors;
-            Vector3 frictionVector[4][2];
-        };
-        typedef std::vector<ConstraintPoint> ConstraintPointArray;
+        double mu;
+        Vector3 relVelocityOn0;
+        int globalFrictionIndex;
+        int numFrictionVectors;
+        Vector3 frictionVector[4][2];
+    };
+    typedef std::vector<ConstraintPoint> ConstraintPointArray;
 
-        struct LinkData
-        {
-            Vector3	dvo;
-            Vector3	dw;
-            Vector3	pf0;
-            Vector3	ptau0;
-            double  uu;
-            double  uu0;
-            double  ddq;
-            int     numberToCheckAccelCalcSkip;
-            int     parentIndex;
-            Link*   link;
-        };
-        typedef std::vector<LinkData> LinkDataArray;
+    struct LinkData
+    {
+        Vector3	dvo;
+        Vector3	dw;
+        Vector3	pf0;
+        Vector3	ptau0;
+        double  uu;
+        double  uu0;
+        double  ddq;
+        int     numberToCheckAccelCalcSkip;
+        int     parentIndex;
+        Link*   link;
+    };
+    typedef std::vector<LinkData> LinkDataArray;
 
-        struct BodyData
-        {
-            BodyPtr body;
-            bool isStatic;
-            bool hasConstrainedLinks;
-            bool isTestForceBeingApplied;
-            LinkDataArray linksData;
+    struct BodyData
+    {
+        BodyPtr body;
+        bool isStatic;
+        bool hasConstrainedLinks;
+        bool isTestForceBeingApplied;
+        LinkDataArray linksData;
 
-            Vector3 dpf;
-            Vector3 dptau;
+        Vector3 dpf;
+        Vector3 dptau;
 
-            /// only used by the ForwardDynamicsMM mode
-            Vector3* rootLinkPosRef;
-
-            /**
-               If the body includes high-gain mode joints,
-               the ForwardDynamisMM object of the body is set to this pointer.
-               The pointer is null when all the joints are torque mode and
-               the forward dynamics is calculated by ABM.
-            */
-            ForwardDynamicsMMPtr forwardDynamicsMM;
-        };
-
-        std::vector<BodyData> bodiesData;
-
-        class LinkPair : public ColdetModelPair
-        {
-        public:
-            int index;
-            bool isSameBodyPair;
-            int bodyIndex[2];
-            BodyData* bodyData[2];
-            Link* link[2];
-            LinkData* linkData[2];
-            ConstraintPointArray constraintPoints;
-
-            double muStatic;
-            double muDynamic;
-            double culling_thresh;
-            double epsilon;
-
-            Body::LinkConnection* connection;
-
-        };
-        typedef intrusive_ptr<LinkPair> LinkPairPtr;
-        typedef std::vector<LinkPairPtr> LinkPairArray;
-
-        LinkPairArray collisionCheckLinkPairs;
-        LinkPairArray connectedLinkPairs;
-
-        std::vector<LinkPair*> constrainedLinkPairs;
+        /// only used by the ForwardDynamicsMM mode
+        Vector3* rootLinkPosRef;
 
         /**
-           globalNumConstraintVectors = globalNumContactNormalVectors + globalNumConnectionVectors
+           If the body includes high-gain mode joints,
+           the ForwardDynamisMM object of the body is set to this pointer.
+           The pointer is null when all the joints are torque mode and
+           the forward dynamics is calculated by ABM.
         */
-        int globalNumConstraintVectors;
+        ForwardDynamicsMMPtr forwardDynamicsMM;
+    };
 
-        int globalNumContactNormalVectors;
-        int globalNumConnectionVectors;
-        int globalNumFrictionVectors;
+    std::vector<BodyData> bodiesData;
 
-        int prevGlobalNumConstraintVectors;
-        int prevGlobalNumFrictionVectors;
+    class LinkPair : public ColdetModelPair
+    {
+    public:
+        int index;
+        bool isSameBodyPair;
+        int bodyIndex[2];
+        BodyData* bodyData[2];
+        Link* link[2];
+        LinkData* linkData[2];
+        ConstraintPointArray constraintPoints;
 
-        bool areThereImpacts;
-        int numUnconverged;
+        double muStatic;
+        double muDynamic;
+        double culling_thresh;
+        double epsilon;
 
-        // Mlcp * solution + b   _|_  solution
-
-        typedef boost::numeric::ublas::matrix<double, ublas::row_major> rmdmatrix;
-        rmdmatrix Mlcp;
-
-        // constant acceleration term when no external force is applied
-        dvector an0;
-        dvector at0;
-
-        // constant vector of LCP
-        dvector b;
-
-        // contact force solution: normal forces at contact points
-        dvector solution;
-
-        // random number generator
-        variate_generator<mt19937, uniform_real<> > randomAngle;
-
-        // for special version of gauss sidel iterative solver
-        std::vector<int> frictionIndexToContactIndex;
-        dvector contactIndexToMu;
-        dvector mcpHi;
-
-        int  maxNumGaussSeidelIteration;
-        int  numGaussSeidelInitialIteration;
-        double gaussSeidelMaxRelError;
-
-        int numGaussSeidelTotalLoops;
-        int numGaussSeidelTotalCalls;
-
-        void setConstraintPoints(CollisionSequence& collisions);
-        void setContactConstraintPoints(LinkPair& linkPair, CollisionPointSequence& collisionPoints);
-        void setFrictionVectors(ConstraintPoint& constraintPoint);
-        bool setConnectionConstraintPoints(LinkPair& linkPair);
-        void putContactPoints();
-        void solveImpactConstraints();
-        void initMatrices();
-        void setAccelCalcSkipInformation();
-        void setDefaultAccelerationVector();
-        void setAccelerationMatrix();
-        void initABMForceElementsWithNoExtForce(BodyData& bodyData);
-        void calcABMForceElementsWithTestForce(BodyData& bodyData, Link* linkToApplyForce, const Vector3& f, const Vector3& tau);
-        void calcAccelsABM(BodyData& bodyData, int constraintIndex);
-        void calcAccelsMM(BodyData& bodyData, int constraintIndex);
-
-        void extractRelAccelsOfConstraintPoints
-        (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, int testForceIndex, int constraintIndex);
-
-        void extractRelAccelsFromLinkPairCase1
-        (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, LinkPair& linkPair, int testForceIndex, int constraintIndex);
-        void extractRelAccelsFromLinkPairCase2
-        (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, LinkPair& linkPair, int iTestForce, int iDefault, int testForceIndex, int constraintIndex);
-        void extractRelAccelsFromLinkPairCase3
-        (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, LinkPair& linkPair, int testForceIndex, int constraintIndex);
-
-        void copySymmetricElementsOfAccelerationMatrix
-        (matrix_range<rmdmatrix>& Knn, matrix_range<rmdmatrix>& Ktn, matrix_range<rmdmatrix>& Knt, matrix_range<rmdmatrix>& Ktt);
-
-        void clearSingularPointConstraintsOfClosedLoopConnections();
-		
-        void setConstantVectorAndMuBlock();
-        void addConstraintForceToLinks();
-        void addConstraintForceToLink(LinkPair* linkPair, int ipair);
-
-        void solveMCPByProjectedGaussSeidel
-        (const rmdmatrix& M, const dvector& b, dvector& x);
-        void solveMCPByProjectedGaussSeidelInitial
-        (const rmdmatrix& M, const dvector& b, dvector& x, const int numIteration);
-        void solveMCPByProjectedGaussSeidelMain
-        (const rmdmatrix& M, const dvector& b, dvector& x, const int numIteration);
-        double solveMCPByProjectedGaussSeidelErrorCheck
-        (const rmdmatrix& M, const dvector& b, dvector& x);
-
-        void checkLCPResult(rmdmatrix& M, dvector& b, dvector& x);
-        void checkMCPResult(rmdmatrix& M, dvector& b, dvector& x);
-
-#ifdef USE_PIVOTING_LCP
-        bool callPathLCPSolver(rmdmatrix& Mlcp, dvector& b, dvector& solution);
-
-        // for PATH solver
-        std::vector<double> lb;
-        std::vector<double> ub;
-        std::vector<int> m_i;
-        std::vector<int> m_j;
-        std::vector<double> m_ij;
-#endif
+        Body::LinkConnection* connection;
 
     };
+    typedef boost::shared_ptr<LinkPair> LinkPairPtr;
+    typedef std::vector<LinkPairPtr> LinkPairArray;
+
+    LinkPairArray collisionCheckLinkPairs;
+    LinkPairArray connectedLinkPairs;
+
+    std::vector<LinkPair*> constrainedLinkPairs;
+
+    /**
+       globalNumConstraintVectors = globalNumContactNormalVectors + globalNumConnectionVectors
+    */
+    int globalNumConstraintVectors;
+
+    int globalNumContactNormalVectors;
+    int globalNumConnectionVectors;
+    int globalNumFrictionVectors;
+
+    int prevGlobalNumConstraintVectors;
+    int prevGlobalNumFrictionVectors;
+
+    bool areThereImpacts;
+    int numUnconverged;
+
+    // Mlcp * solution + b   _|_  solution
+
+    typedef boost::numeric::ublas::matrix<double, ublas::row_major> rmdmatrix;
+    rmdmatrix Mlcp;
+
+    // constant acceleration term when no external force is applied
+    dvector an0;
+    dvector at0;
+
+    // constant vector of LCP
+    dvector b;
+
+    // contact force solution: normal forces at contact points
+    dvector solution;
+
+    // random number generator
+    variate_generator<mt19937, uniform_real<> > randomAngle;
+
+    // for special version of gauss sidel iterative solver
+    std::vector<int> frictionIndexToContactIndex;
+    dvector contactIndexToMu;
+    dvector mcpHi;
+
+    int  maxNumGaussSeidelIteration;
+    int  numGaussSeidelInitialIteration;
+    double gaussSeidelMaxRelError;
+
+    int numGaussSeidelTotalLoops;
+    int numGaussSeidelTotalCalls;
+
+    void setConstraintPoints(CollisionSequence& collisions);
+    void setContactConstraintPoints(LinkPair& linkPair, CollisionPointSequence& collisionPoints);
+    void setFrictionVectors(ConstraintPoint& constraintPoint);
+    bool setConnectionConstraintPoints(LinkPair& linkPair);
+    void putContactPoints();
+    void solveImpactConstraints();
+    void initMatrices();
+    void setAccelCalcSkipInformation();
+    void setDefaultAccelerationVector();
+    void setAccelerationMatrix();
+    void initABMForceElementsWithNoExtForce(BodyData& bodyData);
+    void calcABMForceElementsWithTestForce(BodyData& bodyData, Link* linkToApplyForce, const Vector3& f, const Vector3& tau);
+    void calcAccelsABM(BodyData& bodyData, int constraintIndex);
+    void calcAccelsMM(BodyData& bodyData, int constraintIndex);
+
+    void extractRelAccelsOfConstraintPoints
+    (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, int testForceIndex, int constraintIndex);
+
+    void extractRelAccelsFromLinkPairCase1
+    (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, LinkPair& linkPair, int testForceIndex, int constraintIndex);
+    void extractRelAccelsFromLinkPairCase2
+    (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, LinkPair& linkPair, int iTestForce, int iDefault, int testForceIndex, int constraintIndex);
+    void extractRelAccelsFromLinkPairCase3
+    (matrix_range<rmdmatrix>& Kxn, matrix_range<rmdmatrix>& Kxt, LinkPair& linkPair, int testForceIndex, int constraintIndex);
+
+    void copySymmetricElementsOfAccelerationMatrix
+    (matrix_range<rmdmatrix>& Knn, matrix_range<rmdmatrix>& Ktn, matrix_range<rmdmatrix>& Knt, matrix_range<rmdmatrix>& Ktt);
+
+    void clearSingularPointConstraintsOfClosedLoopConnections();
+		
+    void setConstantVectorAndMuBlock();
+    void addConstraintForceToLinks();
+    void addConstraintForceToLink(LinkPair* linkPair, int ipair);
+
+    void solveMCPByProjectedGaussSeidel
+    (const rmdmatrix& M, const dvector& b, dvector& x);
+    void solveMCPByProjectedGaussSeidelInitial
+    (const rmdmatrix& M, const dvector& b, dvector& x, const int numIteration);
+    void solveMCPByProjectedGaussSeidelMain
+    (const rmdmatrix& M, const dvector& b, dvector& x, const int numIteration);
+    double solveMCPByProjectedGaussSeidelErrorCheck
+    (const rmdmatrix& M, const dvector& b, dvector& x);
+
+    void checkLCPResult(rmdmatrix& M, dvector& b, dvector& x);
+    void checkMCPResult(rmdmatrix& M, dvector& b, dvector& x);
+
+#ifdef USE_PIVOTING_LCP
+    bool callPathLCPSolver(rmdmatrix& Mlcp, dvector& b, dvector& solution);
+
+    // for PATH solver
+    std::vector<double> lb;
+    std::vector<double> ub;
+    std::vector<int> m_i;
+    std::vector<int> m_j;
+    std::vector<double> m_ij;
+#endif
+
+};
 #ifdef __WIN32__
 
-    const double CFSImpl::PI   = 3.14159265358979323846;
-    const double CFSImpl::PI_2 = 1.57079632679489661923;
+const double CFSImpl::PI   = 3.14159265358979323846;
+const double CFSImpl::PI_2 = 1.57079632679489661923;
 #endif
 };
 
@@ -386,7 +384,7 @@ bool CFSImpl::addCollisionCheckLinkPair
     int index;
     int isRegistered;
 
-    tie(index, isRegistered) = world.getIndexOfLinkPairs(link1, link2);
+    boost::tie(index, isRegistered) = world.getIndexOfLinkPairs(link1, link2);
 
     if(index >= 0){
 
@@ -1568,9 +1566,9 @@ void CFSImpl::addConstraintForceToLinks()
     for(int i=0; i < n; ++i){
         LinkPair* linkPair = constrainedLinkPairs[i];
         for(int j=0; j < 2; ++j){
-           // if(!linkPair->link[j]->isRoot() || linkPair->link[j]->jointType != Link::FIXED_JOINT){
-                addConstraintForceToLink(linkPair, j);
-           // }
+            // if(!linkPair->link[j]->isRoot() || linkPair->link[j]->jointType != Link::FIXED_JOINT){
+            addConstraintForceToLink(linkPair, j);
+            // }
         }
     }
 }
@@ -1625,18 +1623,18 @@ void CFSImpl::addConstraintForceToLink(LinkPair* linkPair, int ipair)
 
             if(fn > 0.0){
                 //for(int j=0; j < constraint.numFrictionVectors; ++j){
-                    Vector3 v(constraint.relVelocityOn0 - constraint.normalProjectionOfRelVelocityOn0 * constraint.normalTowardInside[1]);
-                    //cout << constraint.relVelocityOn0 << " ";
-                    if(ipair == 0){
-                        v = -v;
-                    }
-                    double s = norm2(v);
-                    if(s < 1.0e-3){
-                        f += - (v / 1.0e-3) * fn * constraint.mu;
-                    } else {
-                        f += - (v / s) * fn * constraint.mu;
-                    }
-                    //}
+                Vector3 v(constraint.relVelocityOn0 - constraint.normalProjectionOfRelVelocityOn0 * constraint.normalTowardInside[1]);
+                //cout << constraint.relVelocityOn0 << " ";
+                if(ipair == 0){
+                    v = -v;
+                }
+                double s = norm2(v);
+                if(s < 1.0e-3){
+                    f += - (v / 1.0e-3) * fn * constraint.mu;
+                } else {
+                    f += - (v / s) * fn * constraint.mu;
+                }
+                //}
             }
             
             f_total += f;
