@@ -5,12 +5,17 @@
 #ifndef CNOID_BODY_INVERSE_DYNAMICS_H_INCLUDED
 #define CNOID_BODY_INVERSE_DYNAMICS_H_INCLUDED
 
-#include "Body.h"
+#include <cnoid/EigenTypes>
 #include "exportdecl.h"
 
 namespace cnoid {
 
-    CNOID_EXPORT void calcInverseDynamics(const BodyPtr& body, Link* link, Vector3& out_f, Vector3& out_tau);
+class Link;
+
+/**
+   @return force being applied to the root link
+*/
+CNOID_EXPORT Vector6 calcInverseDynamics(Link* link);
 
 }
 
