@@ -2,10 +2,12 @@
    @author Shin'ichiro NAKAOKA
 */
 
+#include "KinematicFaultChecker.h"
 #include "BodyItem.h"
 #include "BodyMotionItem.h"
-#include "KinematicFaultChecker.h"
+#include "WorldItem.h"
 #include "LinkSelectionView.h"
+#include <cnoid/BodyState>
 #include <cnoid/Archive>
 #include <cnoid/MainWindow>
 #include <cnoid/MenuManager>
@@ -13,99 +15,102 @@
 #include <cnoid/ItemTreeView>
 #include <cnoid/MessageView>
 #include <cnoid/SpinBox>
-#include <cnoid/Button>
+#include <cnoid/Buttons>
+#include <cnoid/CheckBox>
+#include <cnoid/Dialog>
 #include <cnoid/Separator>
 #include <cnoid/EigenUtil>
-#include <QDialog>
+#include <cnoid/BodyCollisionDetectorUtil>
+#include <cnoid/IdPair>
 #include <QDialogButtonBox>
 #include <QBoxLayout>
 #include <QFrame>
 #include <QLabel>
 #include <boost/bind.hpp>
-#include <boost/format.hpp>
 #include <map>
 #include "gettext.h"
 
 using namespace std;
-using namespace boost;
 using namespace cnoid;
+using boost::dynamic_bitset;
+using boost::format;
 
 namespace {
 
-    bool USE_DUPLICATED_BODY = false;
+bool USE_DUPLICATED_BODY = false;
 
-    KinematicFaultChecker* checkerInstance = 0;
+KinematicFaultChecker* checkerInstance = 0;
 
-#ifdef _MSC_VER
-	inline long lround(double x) {
-		return static_cast<long>((x > 0.0) ? floor(x + 0.5) : ceil(x -0.5));
-	}
+#if defined(_MSC_VER) && _MSC_VER < 1800
+inline long lround(double x) {
+    return static_cast<long>((x > 0.0) ? floor(x + 0.5) : ceil(x -0.5));
+}
 #endif
 }
 
 namespace cnoid {
 
-    class KinematicFaultCheckerImpl : public QDialog
-    {
-    public:
-        MessageView& mes;
-        std::ostream& os;
+class KinematicFaultCheckerImpl : public Dialog
+{
+public:
+    MessageView& mes;
+    std::ostream& os;
         
-        CheckBox positionCheck;
-        DoubleSpinBox angleMarginSpin;
-        DoubleSpinBox translationMarginSpin;
-        CheckBox velocityCheck;
+    CheckBox positionCheck;
+    DoubleSpinBox angleMarginSpin;
+    DoubleSpinBox translationMarginSpin;
+    CheckBox velocityCheck;
 
-        QButtonGroup radioGroup;
-        RadioButton allJointsRadio;
-        RadioButton selectedJointsRadio;
-        RadioButton nonSelectedJointsRadio;
+    QButtonGroup radioGroup;
+    RadioButton allJointsRadio;
+    RadioButton selectedJointsRadio;
+    RadioButton nonSelectedJointsRadio;
         
-        DoubleSpinBox velocityLimitRatioSpin;
-        CheckBox collisionCheck;
+    DoubleSpinBox velocityLimitRatioSpin;
+    CheckBox collisionCheck;
 
-        CheckBox onlyTimeBarRangeCheck;
+    CheckBox onlyTimeBarRangeCheck;
 
-        int numFaults;
-        vector<int> lastPosFaultFrames;
-        vector<int> lastVelFaultFrames;
-        typedef std::map<ColdetLinkPair*, int> LastCollisionFrameMap;
-        LastCollisionFrameMap lastCollisionFrames;
+    int numFaults;
+    vector<int> lastPosFaultFrames;
+    vector<int> lastVelFaultFrames;
+    typedef std::map<IdPair<int>, int> LastCollisionFrameMap;
+    LastCollisionFrameMap lastCollisionFrames;
 
-        double frameRate;
-        double angleMargin;
-        double translationMargin;
-        double velocityLimitRatio;
+    double frameRate;
+    double angleMargin;
+    double translationMargin;
+    double velocityLimitRatio;
 
-        KinematicFaultCheckerImpl();
-        bool store(Archive& archive);
-        void restore(const Archive& archive);
-        void apply();
-        int checkFaults(
-            BodyItem* bodyItem, BodyMotionItem* motionItem, std::ostream& os,
-            bool checkPosition, bool checkVelocity, bool checkCollision,
-            dynamic_bitset<> linkSelection, double beginningTime, double endingTime);
-        void putJointPositionFault(int frame, Link* joint, std::ostream& os);
-        void putJointVelocityFault(int frame, Link* joint, std::ostream& os);
-        void putSelfCollision(int frame, ColdetLinkPair* linkPair, std::ostream& os);
-    };
+    KinematicFaultCheckerImpl();
+    bool store(Archive& archive);
+    void restore(const Archive& archive);
+    void apply();
+    int checkFaults(
+        BodyItem* bodyItem, BodyMotionItem* motionItem, std::ostream& os,
+        bool checkPosition, bool checkVelocity, bool checkCollision,
+        dynamic_bitset<> linkSelection, double beginningTime, double endingTime);
+    void putJointPositionFault(int frame, Link* joint, std::ostream& os);
+    void putJointVelocityFault(int frame, Link* joint, std::ostream& os);
+    void putSelfCollision(Body* body, int frame, const CollisionPair& collisionPair, std::ostream& os);
+};
 }
 
 
-void KinematicFaultChecker::initialize(ExtensionManager& ext)
+void KinematicFaultChecker::initialize(ExtensionManager* ext)
 {
     if(!checkerInstance){
-        checkerInstance = ext.manage(new KinematicFaultChecker());
+        checkerInstance = ext->manage(new KinematicFaultChecker());
 
-        MenuManager& mm = ext.menuManager();
+        MenuManager& mm = ext->menuManager();
         mm.setPath("/Tools");
         mm.addItem(_("Kinematic Fault Checker"))
-            ->sigTriggered().connect(bind(&KinematicFaultCheckerImpl::show, checkerInstance->impl));
+            ->sigTriggered().connect(boost::bind(&KinematicFaultCheckerImpl::show, checkerInstance->impl));
         
-        ext.connectProjectArchiver(
+        ext->setProjectArchiver(
             "KinematicFaultChecker",
-            bind(&KinematicFaultCheckerImpl::store, checkerInstance->impl, _1),
-            bind(&KinematicFaultCheckerImpl::restore, checkerInstance->impl, _1));
+            boost::bind(&KinematicFaultCheckerImpl::store, checkerInstance->impl, _1),
+            boost::bind(&KinematicFaultCheckerImpl::restore, checkerInstance->impl, _1));
     }
 }
 
@@ -129,8 +134,7 @@ KinematicFaultChecker::~KinematicFaultChecker()
 
 
 KinematicFaultCheckerImpl::KinematicFaultCheckerImpl()
-    : QDialog(MainWindow::instance()),
-      mes(*MessageView::mainInstance()),
+    : mes(*MessageView::mainInstance()),
       os(mes.cout())
 {
     setWindowTitle(_("Kinematic Fault Checker"));
@@ -221,7 +225,7 @@ KinematicFaultCheckerImpl::KinematicFaultCheckerImpl()
     applyButton->setDefault(true);
     QDialogButtonBox* buttonBox = new QDialogButtonBox(this);
     buttonBox->addButton(applyButton, QDialogButtonBox::AcceptRole);
-    applyButton->sigClicked().connect(bind(&KinematicFaultCheckerImpl::apply, this));
+    applyButton->sigClicked().connect(boost::bind(&KinematicFaultCheckerImpl::apply, this));
     
     vbox->addWidget(buttonBox);
 }
@@ -274,20 +278,20 @@ void KinematicFaultCheckerImpl::apply()
         mes.notify(_("No BodyMotionItems are selected."));
     } else {
         for(size_t i=0; i < items.size(); ++i){
-            BodyMotionItem* motionItem = items[i];
+            BodyMotionItem* motionItem = items.get(i);
             BodyItem* bodyItem = motionItem->findOwnerItem<BodyItem>();
             if(!bodyItem){
-                mes.notify(str(format(_("%1% is not owned by any BodyItem. Check skiped.")) % motionItem->name()));
+                mes.notify(str(fmt(_("%1% is not owned by any BodyItem. Check skiped.")) % motionItem->name()));
             } else {
                 mes.putln();
-                mes.notify(str(format(_("Applying the Kinematic Fault Checker to %1% ..."))
+                mes.notify(str(fmt(_("Applying the Kinematic Fault Checker to %1% ..."))
                                % motionItem->headItem()->name()));
                 
                 dynamic_bitset<> linkSelection;
                 if(selectedJointsRadio.isChecked()){
-                    linkSelection = LinkSelectionView::mainInstance()->getLinkSelection(bodyItem);
+                    linkSelection = LinkSelectionView::mainInstance()->linkSelection(bodyItem);
                 } else if(nonSelectedJointsRadio.isChecked()){
-                    linkSelection = LinkSelectionView::mainInstance()->getLinkSelection(bodyItem);
+                    linkSelection = LinkSelectionView::mainInstance()->linkSelection(bodyItem);
                     linkSelection.flip();
                 } else {
                     linkSelection.resize(bodyItem->body()->numLinks(), true);
@@ -295,7 +299,7 @@ void KinematicFaultCheckerImpl::apply()
                 
                 double beginningTime = 0.0;
                 double endingTime = motionItem->motion()->getTimeLength();
-                    std::numeric_limits<double>::max();
+                std::numeric_limits<double>::max();
                 if(onlyTimeBarRangeCheck.isChecked()){
                     TimeBar* timeBar = TimeBar::instance();
                     beginningTime = timeBar->minTime();
@@ -313,7 +317,7 @@ void KinematicFaultCheckerImpl::apply()
                     if(n == 1){
                         mes.notify(_("A fault has been detected."));
                     } else {
-                        mes.notify(str(format(_("%1% faults have been detected.")) % n));
+                        mes.notify(str(fmt(_("%1% faults have been detected.")) % n));
                     }
                 } else {
                     mes.notify(_("No faults have been detected."));
@@ -348,27 +352,39 @@ int KinematicFaultCheckerImpl::checkFaults
     BodyPtr body = bodyItem->body();
     BodyMotionPtr motion = motionItem->motion();
     MultiValueSeqPtr qseq = motion->jointPosSeq();;
-    MultiAffine3SeqPtr pseq = motion->linkPosSeq();
+    MultiSE3SeqPtr pseq = motion->linkPosSeq();
     
     if((!checkPosition && !checkVelocity && !checkCollision) || body->isStaticModel() || !qseq->getNumFrames()){
         return numFaults;
     }
-    
-    vector<ColdetLinkPairPtr>* pColdetLinkPairs;
 
-    BodyItem::KinematicState orgKinematicState;
+    BodyState orgKinematicState;
     
     if(USE_DUPLICATED_BODY){
-        body = body->duplicate();
-        pColdetLinkPairs = new vector<ColdetLinkPairPtr>(bodyItem->selfColdetPairs);
+        body = body->clone();
     } else {
         bodyItem->storeKinematicState(orgKinematicState);
-        pColdetLinkPairs = &bodyItem->selfColdetPairs;
     }
 
+    CollisionDetectorPtr collisionDetector;
+    WorldItem* worldItem = bodyItem->findOwnerItem<WorldItem>();
+    if(worldItem){
+        collisionDetector = worldItem->collisionDetector()->clone();
+    } else {
+        int index = CollisionDetector::factoryIndex("AISTCollisionDetector");
+        if(index >= 0){
+            collisionDetector = CollisionDetector::create(index);
+        } else {
+            collisionDetector = CollisionDetector::create(0);
+            os << _("A collision detector is not found. Collisions cannot be detected this time.") << endl;
+        }
+    }
+
+    addBodyToCollisionDetector(*body, *collisionDetector);
+    collisionDetector->makeReady();
+
     const int numJoints = std::min(body->numJoints(), qseq->numParts());
     const int numLinks = std::min(body->numLinks(), pseq->numParts());
-    const int numPairs = pColdetLinkPairs->size();
 
     frameRate = motion->frameRate();
     double stepRatio2 = 2.0 / frameRate;
@@ -387,8 +403,8 @@ int KinematicFaultCheckerImpl::checkFaults
 
     if(checkCollision){
         Link* root = body->rootLink();
-        root->p.setZero();
-        root->R.setIdentity();
+        root->p().setZero();
+        root->R().setIdentity();
     }
         
     for(int frame = beginningFrame; frame <= endingFrame; ++frame){
@@ -399,14 +415,14 @@ int KinematicFaultCheckerImpl::checkFaults
         for(int i=0; i < numJoints; ++i){
             Link* joint = body->joint(i);
             double q = qseq->at(frame, i);
-            joint->q = q;
-            if(joint->index >= 0 && linkSelection[joint->index]){
+            joint->q() = q;
+            if(joint->index() >= 0 && linkSelection[joint->index()]){
                 if(checkPosition){
                     bool fault = false;
-                    if(joint->jointType == Link::ROTATIONAL_JOINT){
-                        fault = (q > (joint->ulimit - angleMargin) || q < (joint->llimit + angleMargin));
-                    } else if(joint->jointType == Link::SLIDE_JOINT){
-                        fault = (q > (joint->ulimit - translationMargin) || q < (joint->llimit + translationMargin));
+                    if(joint->isRotationalJoint()){
+                        fault = (q > (joint->q_upper() - angleMargin) || q < (joint->q_lower() + angleMargin));
+                    } else if(joint->isSlideJoint()){
+                        fault = (q > (joint->q_upper() - translationMargin) || q < (joint->q_lower() + translationMargin));
                     }
                     if(fault){
                         putJointPositionFault(frame, joint, os);
@@ -414,8 +430,8 @@ int KinematicFaultCheckerImpl::checkFaults
                 }
                 if(checkVelocity){
                     double dq = (qseq->at(nextFrame, i) - qseq->at(prevFrame, i)) / stepRatio2;
-                    joint->dq = dq;
-                    if(dq > (joint->uvlimit * velocityLimitRatio) || dq < (joint->lvlimit * velocityLimitRatio)){
+                    joint->dq() = dq;
+                    if(dq > (joint->dq_upper() * velocityLimitRatio) || dq < (joint->dq_lower() * velocityLimitRatio)){
                         putJointVelocityFault(frame, joint, os);
                     }
                 }
@@ -424,29 +440,41 @@ int KinematicFaultCheckerImpl::checkFaults
 
         if(checkCollision){
 
+            Link* link = body->link(0);
+            if(!pseq->empty())
+            {
+                const SE3& p = pseq->at(frame, 0);
+                link->p() = p.translation();
+                link->R() = p.rotation().toRotationMatrix();
+            }
+            else
+            {
+                link->p() = Vector3d(0., 0., 0.);
+                link->R() = Matrix3d::Identity();
+            }
+
             body->calcForwardKinematics();
 
-            for(int i=0; i < numLinks; ++i){
-                Link* link = body->link(i);
-                const Affine3& p = pseq->at(frame, i);
-                link->p = p.translation();
-                link->R = p.linear();
-            }
-        
-            for(int i=0; i < numPairs; ++i){
-                ColdetLinkPair* linkPair = (*pColdetLinkPairs)[i].get();
-                linkPair->updatePositions();
-                linkPair->detectCollisions();
-                if(linkPair->checkCollision()){
-                    putSelfCollision(frame, linkPair, os);
+            for(int i=1; i < numLinks; ++i){
+                link = body->link(i);
+                if(!pseq->empty())
+                {
+                    const SE3& p = pseq->at(frame, i);
+                    link->p() = p.translation();
+                    link->R() = p.rotation().toRotationMatrix();
                 }
             }
+
+            for(int i=0; i < numLinks; ++i){
+                link = body->link(i);
+                collisionDetector->updatePosition(i, link->position());
+            }
+            collisionDetector->detectCollisions(
+                boost::bind(&KinematicFaultCheckerImpl::putSelfCollision, this, body.get(), frame, _1, boost::ref(os)));
         }
     }
 
-    if(USE_DUPLICATED_BODY){
-        delete pColdetLinkPairs;
-    } else {
+    if(!USE_DUPLICATED_BODY){
         bodyItem->restoreKinematicState(orgKinematicState);
     }
 
@@ -456,20 +484,20 @@ int KinematicFaultCheckerImpl::checkFaults
 
 void KinematicFaultCheckerImpl::putJointPositionFault(int frame, Link* joint, std::ostream& os)
 {
-    static format f1(_("%1$7.3f [s]: Position limit over of %2% (%3% is beyond the range (%4% , %5%) with margin %6%.)"));
-    static format f2(_("%1$7.3f [s]: Position limit over of %2% (%3% is beyond the range (%4% , %5%).)"));
+    static format f1(fmt(_("%1$7.3f [s]: Position limit over of %2% (%3% is beyond the range (%4% , %5%) with margin %6%.)")));
+    static format f2(fmt(_("%1$7.3f [s]: Position limit over of %2% (%3% is beyond the range (%4% , %5%).)")));
     
-    if(frame > lastPosFaultFrames[joint->jointId] + 1){
+    if(frame > lastPosFaultFrames[joint->jointId()] + 1){
         double q, l, u, m;
-        if(joint->jointType == Link::ROTATIONAL_JOINT){
-            q = degree(joint->q);
-            l = degree(joint->llimit);
-            u = degree(joint->ulimit);
+        if(joint->isRotationalJoint()){
+            q = degree(joint->q());
+            l = degree(joint->q_lower());
+            u = degree(joint->q_upper());
             m = degree(angleMargin);
         } else {
-            q = joint->q;
-            l = joint->llimit;
-            u = joint->ulimit;
+            q = joint->q();
+            l = joint->q_lower();
+            u = joint->q_upper();
             m = translationMargin;
         }
 
@@ -481,24 +509,24 @@ void KinematicFaultCheckerImpl::putJointPositionFault(int frame, Link* joint, st
 
         numFaults++;
     }
-    lastPosFaultFrames[joint->jointId] = frame;
+    lastPosFaultFrames[joint->jointId()] = frame;
 }
 
 
 void KinematicFaultCheckerImpl::putJointVelocityFault(int frame, Link* joint, std::ostream& os)
 {
-    static format f(_("%1$7.3f [s]: Velocity limit over of %2% (%3% is %4$.0f %% of the range (%5% , %6%).)"));
+    static format f(fmt(_("%1$7.3f [s]: Velocity limit over of %2% (%3% is %4$.0f %% of the range (%5% , %6%).)")));
     
-    if(frame > lastVelFaultFrames[joint->jointId] + 1){
+    if(frame > lastVelFaultFrames[joint->jointId()] + 1){
         double dq, l, u;
-        if(joint->jointType == Link::ROTATIONAL_JOINT){
-            dq = degree(joint->dq);
-            l = degree(joint->lvlimit);
-            u = degree(joint->uvlimit);
+        if(joint->isRotationalJoint()){
+            dq = degree(joint->dq());
+            l = degree(joint->dq_lower());
+            u = degree(joint->dq_upper());
         } else {
-            dq = joint->dq;
-            l = joint->lvlimit;
-            u = joint->uvlimit;
+            dq = joint->dq();
+            l = joint->dq_lower();
+            u = joint->dq_upper();
         }
 
         double r = (dq < 0.0) ? (dq / l) : (dq / u);
@@ -508,19 +536,20 @@ void KinematicFaultCheckerImpl::putJointVelocityFault(int frame, Link* joint, st
 
         numFaults++;
     }
-    lastVelFaultFrames[joint->jointId] = frame;
+    lastVelFaultFrames[joint->jointId()] = frame;
 }
 
 
-void KinematicFaultCheckerImpl::putSelfCollision(int frame, ColdetLinkPair* linkPair, std::ostream& os)
+void KinematicFaultCheckerImpl::putSelfCollision(Body* body, int frame, const CollisionPair& collisionPair, std::ostream& os)
 {
-    static format f(_("%1$7.3f [s]: Collision between %2% and %3%"));
+    static format f(fmt(_("%1$7.3f [s]: Collision between %2% and %3%")));
     
     bool putMessage = false;
-    LastCollisionFrameMap::iterator p = lastCollisionFrames.find(linkPair);
+    IdPair<int> idPair(collisionPair.geometryId);
+    LastCollisionFrameMap::iterator p = lastCollisionFrames.find(idPair);
     if(p == lastCollisionFrames.end()){
         putMessage = true;
-        lastCollisionFrames[linkPair] = frame;
+        lastCollisionFrames[idPair] = frame;
     } else {
         if(frame > p->second + 1){
             putMessage = true;
@@ -529,7 +558,9 @@ void KinematicFaultCheckerImpl::putSelfCollision(int frame, ColdetLinkPair* link
     }
 
     if(putMessage){
-        os << (f % (frame / frameRate) % linkPair->link(0)->name() % linkPair->link(1)->name()) << endl;
+        Link* link0 = body->link(collisionPair.geometryId[0]);
+        Link* link1 = body->link(collisionPair.geometryId[1]);
+        os << (f % (frame / frameRate) % link0->name() % link1->name()) << endl;
         numFaults++;
     }
 }
