@@ -15,6 +15,7 @@
 #include <QEvent>
 #include <QApplication>
 #include <boost/bind.hpp>
+#include <boost/make_shared.hpp>
 #include <set>
 #include <map>
 #include <deque>
@@ -23,151 +24,156 @@
 #include "gettext.h"
 
 using namespace std;
-using namespace boost;
 using namespace cnoid;
+using boost::dynamic_bitset;
 
 namespace {
-    const bool TRACE_FUNCTIONS = false;
+const bool TRACE_FUNCTIONS = false;
 }
 
 namespace cnoid {
 
-    class LinkTreeWidgetImpl
-    {
-    public:
-        LinkTreeWidgetImpl(LinkTreeWidget* self);
-        ~LinkTreeWidgetImpl();
+class LinkTreeWidgetImpl
+{
+public:
+    LinkTreeWidgetImpl(LinkTreeWidget* self);
+    ~LinkTreeWidgetImpl();
 
-        LinkTreeWidget* self;
+    LinkTreeWidget* self;
 
-        struct ColumnInfo {
-            LinkTreeWidget::ColumnDataFunction dataFunction;
-            LinkTreeWidget::ColumnSetDataFunction setDataFunction;
-            LinkTreeWidget::ColumnWidgetFunction widgetFunction;
-        };
-        vector<ColumnInfo> columnInfos;
-
-        QTreeWidgetItem* headerItem;
-        int nameColumn;
-        int jointIdColumn;
-        int rowIndexCounter;
-        int itemWidgetWidthAdjustment;
-        vector<LinkTreeItem*> customRows;
-        LinkTreeWidget::ListingMode listingMode;
-        ComboBox listingModeCombo;
-        Menu popupMenu; // This must be defined before popupMenuManager
-        MenuManager popupMenuManager;
-
-        boost::signal<void(bool isInitialCreation)> sigUpdateRequest;
-        boost::signal<void(LinkTreeItem* item, int column)> sigItemChanged;
-        boost::signal<void()> sigSelectionChanged;
-
-        int defaultExpansionLevel;
-        bool isCacheEnabled;
-        bool isArchiveOfCurrentBodyItemEnabled;
-
-        class BodyItemInfo : public Referenced {
-        public:
-            bool isRestoringTreeStateNeeded;
-
-            dynamic_bitset<> selection;
-            vector<int> selectedLinkIndices;
-            boost::signal<void()> sigSelectionChanged;
-
-            bool needTreeExpansionUpdate;
-            dynamic_bitset<> linkExpansions;
-            std::set<string> expandedParts;
-                
-            signals::connection detachedFromRootConnection;
-
-            BodyItemInfo() {
-                isRestoringTreeStateNeeded = true;
-            }
-            ~BodyItemInfo() {
-                detachedFromRootConnection.disconnect();
-            }
-            void setNumLinks(int n, bool doInitialize) {
-                if(doInitialize){
-                    selection.reset();
-                    linkExpansions.set();
-                    expandedParts.clear();
-                }
-                selection.resize(n, false);
-                linkExpansions.resize(n, true);
-            }
-        };
-        typedef intrusive_ptr<BodyItemInfo> BodyItemInfoPtr;
-
-        typedef map<BodyItemPtr, BodyItemInfoPtr> BodyItemInfoMap;
-        BodyItemInfoMap bodyItemInfoCache;
-
-        vector<LinkTreeItem*> linkIndexToItemMap;
-
-        BodyItemPtr currentBodyItem;
-        BodyItemInfoPtr currentBodyItemInfo;
-
-        boost::signal<void()> dummySigSelectionChanged; // never emitted
-        vector<int> emptyLinkIndices;
-        dynamic_bitset<> emptySelection;
-
-        void initialize();
-        void enableCache(bool on);
-        BodyItemInfoPtr getBodyItemInfo(BodyItemPtr bodyItem);
-        void onBodyItemDetachedFromRoot(BodyItem* bodyItem);
-        void clearTreeItems();
-        void setCurrentBodyItem(BodyItemPtr bodyItem, bool forceTreeUpdate);
-        void restoreTreeState();
-        void restoreSubTreeState(QTreeWidgetItem* item);
-        void restoreTreeStateSub(QTreeWidgetItem* parentItem);
-        void addChild(LinkTreeItem* parentItem, LinkTreeItem* item);
-        void addChild(LinkTreeItem* item);
-        void setLinkTree(Link* link, bool onlyJoints);
-        void setLinkTreeSub(Link* link, Link* parentLink, LinkTreeItem* parentItem, bool onlyJoints);
-        void setJointList(const BodyPtr& body);
-        void setLinkList(const BodyPtr& body);
-        void setLinkGroupTree(const BodyItemPtr bodyItem);
-        void setLinkGroupTreeSub(LinkTreeItem* parentItem, const LinkGroupPtr& linkGroup, const BodyPtr& body);
-        void addCustomRows();
-        void onListingModeChanged(int index);
-        void onSelectionChanged();
-        boost::signal<void()>& sigSelectionChangedOf(BodyItemPtr bodyItem);
-        const std::vector<int>& getSelectedLinkIndices(BodyItemPtr bodyItem);
-        const boost::dynamic_bitset<>& getLinkSelection(BodyItemPtr bodyItem);
-        void onCustomContextMenuRequested(const QPoint& pos);
-        void setExpansionState(const LinkTreeItem* item, bool on);
-        void onItemExpanded(QTreeWidgetItem* treeWidgetItem);
-        void onItemCollapsed(QTreeWidgetItem* treeWidgetItem);
-        bool makeSingleSelection(BodyItemPtr& bodyItem, int linkIndex);
-        bool storeState(Archive& archive);
-        bool restoreState(const Archive& archive);
+    struct ColumnInfo {
+        LinkTreeWidget::ColumnDataFunction dataFunction;
+        LinkTreeWidget::ColumnSetDataFunction setDataFunction;
+        LinkTreeWidget::ColumnWidgetFunction widgetFunction;
     };
-}
+    vector<ColumnInfo> columnInfos;
 
+    QTreeWidgetItem* headerItem;
+    int nameColumn;
+    int jointIdColumn;
+    int rowIndexCounter;
+    int itemWidgetWidthAdjustment;
+    vector<LinkTreeItem*> customRows;
+    bool isNameColumnMarginEnabled;
+    LinkTreeWidget::ListingMode listingMode;
+    ComboBox listingModeCombo;
+    Menu popupMenu; // This must be defined before popupMenuManager
+    MenuManager popupMenuManager;
+
+    Signal<void(bool isInitialCreation)> sigUpdateRequest;
+    Signal<void(LinkTreeItem* item, int column)> sigItemChanged;
+    Signal<void()> sigSelectionChanged;
+
+    int defaultExpansionLevel;
+    bool isCacheEnabled;
+    bool isArchiveOfCurrentBodyItemEnabled;
+
+    class BodyItemInfo {
+    public:
+        bool isRestoringTreeStateNeeded;
+
+        LinkGroupPtr linkGroup;
+
+        dynamic_bitset<> selection;
+        vector<int> selectedLinkIndices;
+        Signal<void()> sigSelectionChanged;
+
+        bool needTreeExpansionUpdate;
+        dynamic_bitset<> linkExpansions;
+        std::set<string> expandedParts;
+                
+        Connection detachedFromRootConnection;
+
+        BodyItemInfo() {
+            isRestoringTreeStateNeeded = true;
+        }
+        ~BodyItemInfo() {
+            detachedFromRootConnection.disconnect();
+        }
+        void setNumLinks(int n, bool doInitialize) {
+            if(doInitialize){
+                selection.reset();
+                linkExpansions.set();
+                expandedParts.clear();
+            }
+            selection.resize(n, false);
+            linkExpansions.resize(n, true);
+        }
+    };
+    typedef boost::shared_ptr<BodyItemInfo> BodyItemInfoPtr;
+
+    typedef map<BodyItemPtr, BodyItemInfoPtr> BodyItemInfoMap;
+    BodyItemInfoMap bodyItemInfoCache;
+
+    vector<LinkTreeItem*> linkIndexToItemMap;
+
+    BodyItemPtr currentBodyItem;
+    BodyItemInfoPtr currentBodyItemInfo;
+
+    Signal<void()> dummySigSelectionChanged; // never emitted
+    vector<int> emptyLinkIndices;
+    dynamic_bitset<> emptySelection;
+
+    void initialize();
+    void enableCache(bool on);
+    BodyItemInfoPtr getBodyItemInfo(BodyItem* bodyItem);
+    void onBodyItemDetachedFromRoot(BodyItem* bodyItem);
+    void clearTreeItems();
+    void setCurrentBodyItem(BodyItem* bodyItem, bool forceTreeUpdate);
+    void restoreTreeState();
+    void restoreSubTreeState(QTreeWidgetItem* item);
+    void restoreTreeStateSub(QTreeWidgetItem* parentItem);
+    void addChild(LinkTreeItem* parentItem, LinkTreeItem* item);
+    void addChild(LinkTreeItem* item);
+    void setLinkTree(Link* link, bool onlyJoints);
+    void setLinkTreeSub(Link* link, Link* parentLink, LinkTreeItem* parentItem, bool onlyJoints);
+    void setJointList(const BodyPtr& body);
+    void setLinkList(const BodyPtr& body);
+    void setLinkGroupTree(const BodyPtr& body, const LinkGroupPtr& linkGroup);
+    void setLinkGroupTreeSub(LinkTreeItem* parentItem, const LinkGroupPtr& linkGroup, const BodyPtr& body);
+    void addCustomRows();
+    void onListingModeChanged(int index);
+    void onSelectionChanged();
+    Signal<void()>& sigSelectionChangedOf(BodyItem* bodyItem);
+    int selectedLinkIndex(BodyItem* bodyItem) const;
+    const std::vector<int>& selectedLinkIndices(BodyItem* bodyItem);
+    const boost::dynamic_bitset<>& linkSelection(BodyItem* bodyItem);
+    void onCustomContextMenuRequested(const QPoint& pos);
+    void setExpansionState(const LinkTreeItem* item, bool on);
+    void onItemExpanded(QTreeWidgetItem* treeWidgetItem);
+    void onItemCollapsed(QTreeWidgetItem* treeWidgetItem);
+    bool makeSingleSelection(BodyItem* bodyItem, int linkIndex);
+    bool storeState(Archive& archive);
+    bool restoreState(const Archive& archive);
+};
+
+}
 
 namespace {
 
-    void jointIdData(const LinkTreeItem* item, int role, QVariant& out_value)
-    {
-        const Link* link = item->link();
-        if(role == Qt::DisplayRole && link && (link->jointId >= 0)){
-            out_value = link->jointId;
-        } else if(role == Qt::TextAlignmentRole){
-            out_value = Qt::AlignHCenter;
-        }
+void jointIdData(const LinkTreeItem* item, int role, QVariant& out_value)
+{
+    const Link* link = item->link();
+    if(role == Qt::DisplayRole && link && (link->jointId() >= 0)){
+        out_value = link->jointId();
+    } else if(role == Qt::TextAlignmentRole){
+        out_value = Qt::AlignHCenter;
+    }
         
-    }
+}
 
-    void nameData(const LinkTreeItem* item, int role, QVariant& out_value)
-    {
-        if(role == Qt::DisplayRole){
-            out_value = item->name().c_str();
-        }
+void nameData(const LinkTreeItem* item, int role, QVariant& out_value)
+{
+    if(role == Qt::DisplayRole){
+        out_value = item->nameText();
     }
 }
+}
 
 
 LinkTreeItem::LinkTreeItem(const std::string& name)
-    : name_(name)
+    : name_(name),
+      nameText_(name.c_str())
 {
     rowIndex_ = -1;
     link_ = 0;
@@ -178,16 +184,26 @@ LinkTreeItem::LinkTreeItem(const std::string& name)
 LinkTreeItem::LinkTreeItem(Link* link, LinkTreeWidgetImpl* treeImpl)
     : name_(link->name())
 {
+    if(treeImpl->isNameColumnMarginEnabled){
+        nameText_ = QString(" %1 ").arg(name_.c_str());
+    } else {
+        nameText_ = name_.c_str();
+    }
     rowIndex_ = -1;
     link_ = link;
     isLinkGroup_ = false;
-    treeImpl->linkIndexToItemMap[link->index] = this;
+    treeImpl->linkIndexToItemMap[link->index()] = this;
 }
 
 
 LinkTreeItem::LinkTreeItem(LinkGroup* linkGroup, LinkTreeWidgetImpl* treeImpl)
     : name_(linkGroup->name())
 {
+    if(treeImpl->isNameColumnMarginEnabled){
+        nameText_ = QString(" %1 ").arg(name_.c_str());
+    } else {
+        nameText_ = name_.c_str();
+    }
     rowIndex_ = -1;
     link_ = 0;
     isLinkGroup_ = true;
@@ -243,9 +259,11 @@ void LinkTreeWidgetImpl::initialize()
     isCacheEnabled = false;
     isArchiveOfCurrentBodyItemEnabled = false;
     itemWidgetWidthAdjustment = 0;
+    isNameColumnMarginEnabled = false;
     
     headerItem = new QTreeWidgetItem;
     QHeaderView* header = self->header();
+    header->setMinimumSectionSize(0);
     header->setStretchLastSection(false);
     QObject::connect(header, SIGNAL(sectionResized(int, int, int)),
                      self, SLOT(onHeaderSectionResized()));
@@ -256,13 +274,23 @@ void LinkTreeWidgetImpl::initialize()
     self->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
     self->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
 
-    nameColumn = self->addColumn(_("link"));
+    nameColumn = self->addColumn(_("Link"));
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
     header->setResizeMode(nameColumn, QHeaderView::Stretch);
+#else
+    header->setSectionResizeMode(nameColumn, QHeaderView::Stretch);
+#endif
     self->setColumnDataFunction(nameColumn, &nameData);
 
-    jointIdColumn = self->addColumn(_("id"));
+    jointIdColumn = self->addColumn(_("ID"));
     self->setColumnDataFunction(jointIdColumn, &jointIdData);
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
     header->setResizeMode(jointIdColumn, QHeaderView::ResizeToContents);
+#else
+    header->setSectionResizeMode(jointIdColumn, QHeaderView::ResizeToContents);
+#endif
+    
+    headerItem->setTextAlignment(jointIdColumn, Qt::AlignHCenter);
     self->moveVisualColumnIndex(jointIdColumn, 0);
 
     QObject::connect(self, SIGNAL(itemChanged(QTreeWidgetItem*, int)),
@@ -279,16 +307,16 @@ void LinkTreeWidgetImpl::initialize()
 
     // The order of the following labes must correspond to that of ListingMode
     listingModeCombo.enableI18n(CNOID_GETTEXT_DOMAIN_NAME);
-    listingModeCombo.addI18nItem(N_("link list"));
-    listingModeCombo.addI18nItem(N_("link tree"));
-    listingModeCombo.addI18nItem(N_("joint list"));
-    listingModeCombo.addI18nItem(N_("joint tree"));
-    listingModeCombo.addI18nItem(N_("part tree"));
+    listingModeCombo.addI18nItem(N_("Link List"));
+    listingModeCombo.addI18nItem(N_("Link Tree"));
+    listingModeCombo.addI18nItem(N_("Joint List"));
+    listingModeCombo.addI18nItem(N_("Joint Tree"));
+    listingModeCombo.addI18nItem(N_("Part Tree"));
 
     listingMode = LinkTreeWidget::LINK_LIST;
     listingModeCombo.setCurrentIndex(listingMode);
     listingModeCombo.sigCurrentIndexChanged().connect(
-        bind(&LinkTreeWidgetImpl::onListingModeChanged, this, _1));
+        boost::bind(&LinkTreeWidgetImpl::onListingModeChanged, this, _1));
 }
 
 
@@ -340,11 +368,25 @@ int LinkTreeWidget::jointIdColumn()
 }
 
 
+int LinkTreeWidget::setNumColumns(int n)
+{
+    setColumnCount(n);
+    impl->columnInfos.resize(n);
+    return 0;
+}
+
+
 int LinkTreeWidget::addColumn()
 {
     int column = impl->columnInfos.size();
     impl->columnInfos.push_back(LinkTreeWidgetImpl::ColumnInfo());
     setColumnCount(impl->columnInfos.size());
+    impl->headerItem->setText(column, QString());
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+    header()->setResizeMode(column, QHeaderView::ResizeToContents);
+#else
+    header()->setSectionResizeMode(column, QHeaderView::ResizeToContents);
+#endif
     return column;
 }
 
@@ -353,9 +395,44 @@ int LinkTreeWidget::addColumn(const QString& headerText)
 {
     int column = addColumn();
     impl->headerItem->setText(column, headerText);
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+    header()->setResizeMode(column, QHeaderView::ResizeToContents);
+#else
+    header()->setSectionResizeMode(column, QHeaderView::ResizeToContents);
+#endif
     return column;
 }
 
+
+void LinkTreeWidget::setColumnStretchResizeMode(int column)
+{
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+    header()->setResizeMode(column, QHeaderView::Stretch);
+#else
+    header()->setSectionResizeMode(column, QHeaderView::Stretch);
+#endif
+}
+
+
+void LinkTreeWidget::setColumnInteractiveResizeMode(int column)
+{
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+    header()->setResizeMode(column, QHeaderView::Interactive);
+#else
+    header()->setSectionResizeMode(column, QHeaderView::Interactive);
+#endif
+}
+
+
+void LinkTreeWidget::setColumnResizeToContentsMode(int column)
+{
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+    header()->setResizeMode(column, QHeaderView::ResizeToContents);
+#else
+    header()->setSectionResizeMode(column, QHeaderView::ResizeToContents);
+#endif
+}
+
                     
 void LinkTreeWidget::moveVisualColumnIndex(int column, int visualIndex)
 {
@@ -390,6 +467,12 @@ void LinkTreeWidget::changeEvent(QEvent* event)
 }
 
 
+void LinkTreeWidget::setNameColumnMarginEnabled(bool on)
+{
+    impl->isNameColumnMarginEnabled = on;
+}
+
+
 void LinkTreeWidget::setAlignedItemWidget
 (LinkTreeItem* item, int column, QWidget* widget, Qt::Alignment alignment)
 {
@@ -436,7 +519,7 @@ int LinkTreeWidget::numLinkTreeItems()
 }
 
 
-SignalProxy< boost::signal<void(bool isInitialCreation)> > LinkTreeWidget::sigUpdateRequest()
+SignalProxy<void(bool isInitialCreation)> LinkTreeWidget::sigUpdateRequest()
 {
     return impl->sigUpdateRequest;
 }
@@ -461,7 +544,7 @@ void LinkTreeWidget::fixListingMode(bool on)
 }
 
 
-LinkTreeWidgetImpl::BodyItemInfoPtr LinkTreeWidgetImpl::getBodyItemInfo(BodyItemPtr bodyItem)
+LinkTreeWidgetImpl::BodyItemInfoPtr LinkTreeWidgetImpl::getBodyItemInfo(BodyItem* bodyItem)
 {
     BodyItemInfoPtr info;
 
@@ -479,9 +562,10 @@ LinkTreeWidgetImpl::BodyItemInfoPtr LinkTreeWidgetImpl::getBodyItemInfo(BodyItem
             if(!isCacheEnabled){
                 bodyItemInfoCache.clear();
             }
-            info = new BodyItemInfo();
+            info = boost::make_shared<BodyItemInfo>();
+            info->linkGroup = LinkGroup::create(*bodyItem->body());
             info->detachedFromRootConnection = bodyItem->sigDetachedFromRoot().connect(
-                bind(&LinkTreeWidgetImpl::onBodyItemDetachedFromRoot, this, bodyItem.get()));
+                boost::bind(&LinkTreeWidgetImpl::onBodyItemDetachedFromRoot, this, bodyItem));
             bodyItemInfoCache[bodyItem] = info;
         }
 
@@ -496,7 +580,7 @@ LinkTreeWidgetImpl::BodyItemInfoPtr LinkTreeWidgetImpl::getBodyItemInfo(BodyItem
 
 void LinkTreeWidgetImpl::onBodyItemDetachedFromRoot(BodyItem* bodyItem)
 {
-    if(currentBodyItem.get() == bodyItem){
+    if(currentBodyItem == bodyItem){
         setCurrentBodyItem(0, false);
     }
 
@@ -507,7 +591,7 @@ void LinkTreeWidgetImpl::onBodyItemDetachedFromRoot(BodyItem* bodyItem)
     }
 }
 
-void LinkTreeWidget::setBodyItem(BodyItemPtr bodyItem)
+void LinkTreeWidget::setBodyItem(BodyItem* bodyItem)
 {
     impl->setCurrentBodyItem(bodyItem, false);
 }
@@ -515,7 +599,7 @@ void LinkTreeWidget::setBodyItem(BodyItemPtr bodyItem)
 
 BodyItem* LinkTreeWidget::bodyItem()
 {
-    return impl->currentBodyItem.get();
+    return impl->currentBodyItem;
 }
 
 
@@ -541,13 +625,15 @@ void LinkTreeWidgetImpl::clearTreeItems()
 }
 
 
-void LinkTreeWidgetImpl::setCurrentBodyItem(BodyItemPtr bodyItem, bool forceTreeUpdate)
+void LinkTreeWidgetImpl::setCurrentBodyItem(BodyItem* bodyItem, bool forceTreeUpdate)
 {
     if(TRACE_FUNCTIONS){
         cout << "LinkTreeWidgetImpl::setCurrentBodyItem()" << endl;
     }
+
+    bool currentChanged = bodyItem != currentBodyItem;
     
-    if(forceTreeUpdate || bodyItem != currentBodyItem){
+    if(forceTreeUpdate || currentChanged){
 
         self->blockSignals(true);
 
@@ -575,31 +661,36 @@ void LinkTreeWidgetImpl::setCurrentBodyItem(BodyItemPtr bodyItem, bool forceTree
             case LinkTreeWidget::LINK_LIST:
                 self->setRootIsDecorated(false);
                 setLinkList(body);
+                headerItem->setText(nameColumn, _("Link"));
                 break;
             case LinkTreeWidget::LINK_TREE:
                 self->setRootIsDecorated(true);
                 setLinkTree(body->rootLink(), false);
+                headerItem->setText(nameColumn, _("Link"));
                 break;
             case LinkTreeWidget::JOINT_LIST:
                 self->setRootIsDecorated(false);
                 setJointList(body);
+                headerItem->setText(nameColumn, _("Joint"));
                 break;
             case LinkTreeWidget::JOINT_TREE:
                 self->setRootIsDecorated(true);
                 setLinkTree(body->rootLink(), true);
+                headerItem->setText(nameColumn, _("Joint"));
                 break;
             case LinkTreeWidget::PART_TREE:
                 self->setRootIsDecorated(true);
-                setLinkGroupTree(bodyItem);
+                setLinkGroupTree(bodyItem->body(), currentBodyItemInfo->linkGroup);
+                headerItem->setText(nameColumn, _("Link"));
                 break;
             default:
                 break;
             }
 
             addCustomRows();
-
-            sigUpdateRequest(true);
         }
+
+        sigUpdateRequest(true);
     }
 }
 
@@ -638,7 +729,7 @@ void LinkTreeWidgetImpl::restoreTreeStateSub(QTreeWidgetItem* parentItem)
             const Link* link = item->link();
             if(link){
                 const dynamic_bitset<>& selection = currentBodyItemInfo->selection;
-                item->setSelected(selection[link->index]);
+                item->setSelected(selection[link->index()]);
             }
         
             if(item->childCount() > 0){ // Tree
@@ -652,7 +743,7 @@ void LinkTreeWidgetImpl::restoreTreeStateSub(QTreeWidgetItem* parentItem)
                         item->setExpanded(expanded);
                     }
                 } else if(link){
-                    expanded = currentBodyItemInfo->linkExpansions[link->index];
+                    expanded = currentBodyItemInfo->linkExpansions[link->index()];
                     item->setExpanded(expanded);
                 }
                 if(expanded){
@@ -703,7 +794,7 @@ void LinkTreeWidgetImpl::setLinkTreeSub(Link* link, Link* parentLink, LinkTreeIt
 {
     LinkTreeItem* item = 0;
 
-    if(onlyJoints && link->jointId < 0){
+    if(onlyJoints && link->jointId() < 0){
         item = parentItem;
     } else {
         item = new LinkTreeItem(link, this);
@@ -711,8 +802,8 @@ void LinkTreeWidgetImpl::setLinkTreeSub(Link* link, Link* parentLink, LinkTreeIt
         item->setExpanded(true);
     }
 
-    if(link->child){
-        for(Link* child = link->child; child; child = child->sibling){
+    if(link->child()){
+        for(Link* child = link->child(); child; child = child->sibling()){
             setLinkTreeSub(child, link, item, onlyJoints);
         }
     }
@@ -742,10 +833,8 @@ void LinkTreeWidgetImpl::setLinkList(const BodyPtr& body)
 }
 
 
-void LinkTreeWidgetImpl::setLinkGroupTree(const BodyItemPtr bodyItem)
+void LinkTreeWidgetImpl::setLinkGroupTree(const BodyPtr& body, const LinkGroupPtr& linkGroup)
 {
-    BodyPtr body = bodyItem->body();
-    LinkGroupPtr linkGroup = body->linkGroup();
     if(linkGroup){
         self->blockSignals(true);
         setLinkGroupTreeSub(0, linkGroup, body);
@@ -816,7 +905,7 @@ void LinkTreeWidgetImpl::onSelectionChanged()
         for(int i=0; i < selected.size(); ++i){
             LinkTreeItem* item = dynamic_cast<LinkTreeItem*>(selected[i]);
             if(item && item->link()){
-                currentBodyItemInfo->selection[item->link()->index] = true;
+                currentBodyItemInfo->selection[item->link()->index()] = true;
             }
         }
         currentBodyItemInfo->sigSelectionChanged();
@@ -825,7 +914,7 @@ void LinkTreeWidgetImpl::onSelectionChanged()
 }
 
 
-SignalProxy< boost::signal<void(LinkTreeItem* item, int column)> > LinkTreeWidget::sigItemChanged()
+SignalProxy<void(LinkTreeItem* item, int column)> LinkTreeWidget::sigItemChanged()
 {
     return impl->sigItemChanged;
 }
@@ -840,19 +929,19 @@ void LinkTreeWidget::onItemChanged(QTreeWidgetItem* item, int column)
 }
 
 
-SignalProxy< boost::signal<void()> > LinkTreeWidget::sigSelectionChanged()
+SignalProxy<void()> LinkTreeWidget::sigSelectionChanged()
 {
     return impl->sigSelectionChanged;
 }
 
 
-SignalProxy< boost::signal<void()> > LinkTreeWidget::sigSelectionChanged(BodyItemPtr bodyItem)
+SignalProxy<void()> LinkTreeWidget::sigSelectionChanged(BodyItem* bodyItem)
 {
     return impl->sigSelectionChangedOf(bodyItem);
 }
 
 
-boost::signal<void()>& LinkTreeWidgetImpl::sigSelectionChangedOf(BodyItemPtr bodyItem)
+Signal<void()>& LinkTreeWidgetImpl::sigSelectionChangedOf(BodyItem* bodyItem)
 {
     BodyItemInfoPtr info = getBodyItemInfo(bodyItem);
     if(info){
@@ -862,19 +951,44 @@ boost::signal<void()>& LinkTreeWidgetImpl::sigSelectionChangedOf(BodyItemPtr bod
 }
 
 
-const std::vector<int>& LinkTreeWidget::getSelectedLinkIndices()
+int LinkTreeWidget::selectedLinkIndex() const
 {
-    return impl->getSelectedLinkIndices(impl->currentBodyItem);
+    return impl->selectedLinkIndex(impl->currentBodyItem);
 }
 
 
-const std::vector<int>& LinkTreeWidget::getSelectedLinkIndices(BodyItemPtr bodyItem)
+int LinkTreeWidget::selectedLinkIndex(BodyItem* bodyItem) const
 {
-    return impl->getSelectedLinkIndices(bodyItem);
+    return impl->selectedLinkIndex(bodyItem);
 }
 
 
-const std::vector<int>& LinkTreeWidgetImpl::getSelectedLinkIndices(BodyItemPtr bodyItem)
+int LinkTreeWidgetImpl::selectedLinkIndex(BodyItem* bodyItem) const
+{
+    BodyItemInfoPtr info = const_cast<LinkTreeWidgetImpl*>(this)->getBodyItemInfo(bodyItem);
+    if(info){
+        dynamic_bitset<>::size_type index = info->selection.find_first();
+        if(index != dynamic_bitset<>::npos){
+            return index;
+        }
+    }
+    return -1;
+}
+
+
+const std::vector<int>& LinkTreeWidget::selectedLinkIndices()
+{
+    return impl->selectedLinkIndices(impl->currentBodyItem);
+}
+
+
+const std::vector<int>& LinkTreeWidget::selectedLinkIndices(BodyItem* bodyItem)
+{
+    return impl->selectedLinkIndices(bodyItem);
+}
+
+
+const std::vector<int>& LinkTreeWidgetImpl::selectedLinkIndices(BodyItem* bodyItem)
 {
     BodyItemInfoPtr info = getBodyItemInfo(bodyItem);
     if(info){
@@ -891,19 +1005,19 @@ const std::vector<int>& LinkTreeWidgetImpl::getSelectedLinkIndices(BodyItemPtr b
 }
 
 
-const boost::dynamic_bitset<>& LinkTreeWidget::getLinkSelection()
+const boost::dynamic_bitset<>& LinkTreeWidget::linkSelection()
 {
-    return impl->getLinkSelection(impl->currentBodyItem);
+    return impl->linkSelection(impl->currentBodyItem);
 }
 
 
-const boost::dynamic_bitset<>& LinkTreeWidget::getLinkSelection(BodyItemPtr bodyItem)
+const boost::dynamic_bitset<>& LinkTreeWidget::linkSelection(BodyItem* bodyItem)
 {
-    return impl->getLinkSelection(bodyItem);
+    return impl->linkSelection(bodyItem);
 }
 
 
-const boost::dynamic_bitset<>& LinkTreeWidgetImpl::getLinkSelection(BodyItemPtr bodyItem)
+const boost::dynamic_bitset<>& LinkTreeWidgetImpl::linkSelection(BodyItem* bodyItem)
 {
     BodyItemInfoPtr info = getBodyItemInfo(bodyItem);
     if(info){
@@ -929,7 +1043,7 @@ void LinkTreeWidgetImpl::setExpansionState(const LinkTreeItem* item, bool on)
 {
     if(listingMode == LinkTreeWidget::LINK_TREE || listingMode == LinkTreeWidget::JOINT_TREE){
         if(item->link()){
-            currentBodyItemInfo->linkExpansions[item->link()->index] = on;
+            currentBodyItemInfo->linkExpansions[item->link()->index()] = on;
         }
 
     } else if(listingMode == LinkTreeWidget::PART_TREE){
@@ -991,13 +1105,13 @@ void LinkTreeWidget::enableArchiveOfCurrentBodyItem(bool on)
 }
 
 
-bool LinkTreeWidget::makeSingleSelection(BodyItemPtr bodyItem, int linkIndex)
+bool LinkTreeWidget::makeSingleSelection(BodyItem* bodyItem, int linkIndex)
 {
     return impl->makeSingleSelection(bodyItem, linkIndex);
 }
 
 
-bool LinkTreeWidgetImpl::makeSingleSelection(BodyItemPtr& bodyItem, int linkIndex)
+bool LinkTreeWidgetImpl::makeSingleSelection(BodyItem* bodyItem, int linkIndex)
 {
     BodyItemInfoPtr info = getBodyItemInfo(bodyItem);
 
@@ -1044,7 +1158,7 @@ bool LinkTreeWidget::storeState(Archive& archive)
 bool LinkTreeWidgetImpl::storeState(Archive& archive)
 {
     if(listingModeCombo.isEnabled()){
-        archive.write("listingMode", listingModeCombo.currentOrgText().toStdString(), YAML_DOUBLE_QUOTED);
+        archive.write("listingMode", listingModeCombo.currentOrgText().toStdString(), DOUBLE_QUOTED);
     }
 
     if(isArchiveOfCurrentBodyItemEnabled){
@@ -1053,20 +1167,20 @@ bool LinkTreeWidgetImpl::storeState(Archive& archive)
 
     if(isCacheEnabled && !bodyItemInfoCache.empty()){
 
-        YamlSequencePtr bodyItemNodes = new YamlSequence();
+        ListingPtr bodyItemNodes = new Listing();
 
         for(BodyItemInfoMap::iterator it = bodyItemInfoCache.begin(); it != bodyItemInfoCache.end(); ++it){
 
-            BodyItemPtr bodyItem = it->first;
+            BodyItem* bodyItem = it->first;
             BodyItemInfo& info = *it->second;
-            YamlMappingPtr bodyItemNode = new YamlMapping();
+            MappingPtr bodyItemNode = new Mapping();
             bool isEmpty = true;
 
-            bodyItemNode->write("id", archive.getItemId(bodyItem));
+            bodyItemNode->insert("id", archive.getItemId(bodyItem));
             
-            const vector<int>& indices = getSelectedLinkIndices(bodyItem);
+            const vector<int>& indices = selectedLinkIndices(bodyItem);
             if(!indices.empty()){
-                YamlSequence& selected = *bodyItemNode->createFlowStyleSequence("selectedLinks");
+                Listing& selected = *bodyItemNode->createFlowStyleListing("selectedLinks");
                 int n = indices.size();
                 for(int i=0; i < n; ++i){
                     selected.append(indices[i], 20, n);
@@ -1077,7 +1191,7 @@ bool LinkTreeWidgetImpl::storeState(Archive& archive)
             int n = info.linkExpansions.size();
             int m = n - info.linkExpansions.count();
             if(m > 0){
-                YamlSequence& nonExpanded = *bodyItemNode->createFlowStyleSequence("nonExpandedLinks");
+                Listing& nonExpanded = *bodyItemNode->createFlowStyleListing("nonExpandedLinks");
                 for(int i=0; i < n; ++i){
                     if(!info.linkExpansions[i]){
                         nonExpanded.append(i, 20, m);
@@ -1088,9 +1202,9 @@ bool LinkTreeWidgetImpl::storeState(Archive& archive)
 
             n = info.expandedParts.size();
             if(n > 0){
-                YamlSequence& expanded = *bodyItemNode->createFlowStyleSequence("expandedParts");
+                Listing& expanded = *bodyItemNode->createFlowStyleListing("expandedParts");
                 for(std::set<string>::iterator p = info.expandedParts.begin(); p != info.expandedParts.end(); ++p){
-                    expanded.append(*p, 10, n, YAML_DOUBLE_QUOTED);
+                    expanded.append(*p, 10, n, DOUBLE_QUOTED);
                 }
                 isEmpty = false;
             }
@@ -1111,24 +1225,25 @@ bool LinkTreeWidgetImpl::storeState(Archive& archive)
 
 bool LinkTreeWidget::restoreState(const Archive& archive)
 {
-    return impl->restoreState(archive);
+    archive.addPostProcess(boost::bind(&LinkTreeWidgetImpl::restoreState, impl, boost::ref(archive)));
+    return true;
 }
 
 
 bool LinkTreeWidgetImpl::restoreState(const Archive& archive)
 {
     if(isCacheEnabled){
-        const YamlSequence& nodes = *archive.findSequence("bodyItems");
+        const Listing& nodes = *archive.findListing("bodyItems");
         if(nodes.isValid() && !nodes.empty()){
             for(int i=0; i < nodes.size(); ++i){
-                const YamlMapping& node = *nodes[i].toMapping();
-                int id = node.get("id", -1);
-                if(id >= 0){
+                const Mapping& node = *nodes[i].toMapping();
+                ValueNode* id = node.find("id");
+                if(id->isValid()){
                     BodyItem* bodyItem = archive.findItem<BodyItem>(id);
                     if(bodyItem){
                         int numLinks = bodyItem->body()->numLinks();
                         BodyItemInfoPtr info = getBodyItemInfo(bodyItem);
-                        const YamlSequence& selected = *node.findSequence("selectedLinks");
+                        const Listing& selected = *node.findListing("selectedLinks");
                         if(selected.isValid()){
                             info->setNumLinks(numLinks, true);
                             for(int i=0; i < selected.size(); ++i){
@@ -1138,9 +1253,9 @@ bool LinkTreeWidgetImpl::restoreState(const Archive& archive)
                                 }
                             }
                         }
-                        const YamlSequence& expanded = *node.findSequence("expandedParts");
+                        const Listing& expanded = *node.findListing("expandedParts");
                         for(int i=0; i < expanded.size(); ++i){
-                                info->expandedParts.insert(expanded[i]);
+                            info->expandedParts.insert(expanded[i]);
                         }
                     }
                 }
