@@ -17,8 +17,9 @@
 #include <cnoid/MainWindow>
 #include <cnoid/SpinBox>
 #include <cnoid/Separator>
-#include <cnoid/Button>
-#include <QDialog>
+#include <cnoid/Buttons>
+#include <cnoid/CheckBox>
+#include <cnoid/Dialog>
 #include <QDialogButtonBox>
 #include <boost/bind.hpp>
 #include <boost/format.hpp>
@@ -30,59 +31,60 @@ using namespace boost;
 using namespace cnoid;
 
 namespace {
-    const bool TRACE_FUNCTIONS = false;
+const bool TRACE_FUNCTIONS = false;
 }
 
 namespace cnoid {
 
-    class BodyMotionGenerationSetupDialog : public QDialog
-    {
-    public:
-        QVBoxLayout* vbox;
+class BodyMotionGenerationSetupDialog : public Dialog
+{
+public:
+    QVBoxLayout* vbox;
         
-        DoubleSpinBox timeScaleRatioSpin;
-        DoubleSpinBox preInitialDurationSpin;
-        DoubleSpinBox postFinalDurationSpin;
-        CheckBox onlyTimeBarRangeCheck;
-        CheckBox newBodyItemCheck;
+    DoubleSpinBox timeScaleRatioSpin;
+    DoubleSpinBox preInitialDurationSpin;
+    DoubleSpinBox postFinalDurationSpin;
+    CheckBox onlyTimeBarRangeCheck;
+    CheckBox newBodyItemCheck;
         
-        CheckBox stealthyStepCheck;
-        DoubleSpinBox stealthyHeightRatioThreshSpin;
-        DoubleSpinBox flatLiftingHeightSpin;
-        DoubleSpinBox flatLandingHeightSpin;
-        DoubleSpinBox impactReductionHeightSpin;
-        DoubleSpinBox impactReductionTimeSpin;
+    CheckBox stealthyStepCheck;
+    DoubleSpinBox stealthyHeightRatioThreshSpin;
+    DoubleSpinBox flatLiftingHeightSpin;
+    DoubleSpinBox flatLandingHeightSpin;
+    DoubleSpinBox impactReductionHeightSpin;
+    DoubleSpinBox impactReductionTimeSpin;
 
-        CheckBox autoZmpCheck;
-        DoubleSpinBox minZmpTransitionTimeSpin;
-        DoubleSpinBox zmpCenteringTimeThreshSpin;
-        DoubleSpinBox zmpTimeMarginBeforeLiftingSpin;
+    CheckBox autoZmpCheck;
+    DoubleSpinBox minZmpTransitionTimeSpin;
+    DoubleSpinBox zmpCenteringTimeThreshSpin;
+    DoubleSpinBox zmpTimeMarginBeforeLiftingSpin;
+    DoubleSpinBox zmpMaxDistanceFromCenterSpin;
         
-        CheckBox se3Check;
-        CheckBox lipSyncMixCheck;
+    CheckBox se3Check;
+    CheckBox lipSyncMixCheck;
 
-        void addSeparator(QVBoxLayout* vbox){
-            vbox->addSpacing(4);
-            vbox->addWidget(new HSeparator());
-            vbox->addSpacing(2);
-        }
+    void addSeparator(QVBoxLayout* vbox){
+        vbox->addSpacing(4);
+        vbox->addWidget(new HSeparator());
+        vbox->addSpacing(2);
+    }
 
-        void addSeparator(QVBoxLayout* vbox, QWidget* widget) {
-            vbox->addSpacing(4);
-            vbox->addLayout(new HSeparatorBox(widget));
-            vbox->addSpacing(2);
-        }
+    void addSeparator(QVBoxLayout* vbox, QWidget* widget) {
+        vbox->addSpacing(4);
+        vbox->addLayout(new HSeparatorBox(widget));
+        vbox->addSpacing(2);
+    }
 
-        QHBoxLayout* newRow(QVBoxLayout* vbox) {
-            QHBoxLayout* hbox = new QHBoxLayout();
-            hbox->setSpacing(2);
-            hbox->setContentsMargins(2, 2, 2, 2);
-            vbox->addLayout(hbox);
-            return hbox;
-        }
+    QHBoxLayout* newRow(QVBoxLayout* vbox) {
+        QHBoxLayout* hbox = new QHBoxLayout();
+        hbox->setSpacing(2);
+        hbox->setContentsMargins(2, 2, 2, 2);
+        vbox->addLayout(hbox);
+        return hbox;
+    }
         
-        BodyMotionGenerationSetupDialog() : QDialog(MainWindow::instance()){
-
+    BodyMotionGenerationSetupDialog()
+        {
             setWindowTitle(_("Body Motion Generation Setup"));
 
             vbox = new QVBoxLayout();
@@ -219,8 +221,17 @@ namespace cnoid {
             zmpTimeMarginBeforeLiftingSpin.setValue(0.0);
             hbox->addWidget(&zmpTimeMarginBeforeLiftingSpin);
             hbox->addWidget(new QLabel(_("[s]")));
+
+            hbox->addSpacing(8);
+            hbox->addWidget(new QLabel(_("Max distance from center")));
+            zmpMaxDistanceFromCenterSpin.setDecimals(3);
+            zmpMaxDistanceFromCenterSpin.setRange(0.001, 0.999);
+            zmpMaxDistanceFromCenterSpin.setSingleStep(0.001);
+            zmpMaxDistanceFromCenterSpin.setValue(0.02);
+            hbox->addWidget(&zmpMaxDistanceFromCenterSpin);
+            hbox->addWidget(new QLabel(_("[m]")));
             hbox->addStretch();
-            
+
             addSeparator(vbox);
             hbox = newRow(vbox);
 
@@ -245,48 +256,51 @@ namespace cnoid {
         }
         
 
-        void storeState(Archive& archive){
-            archive.write("timeScaleRatio", timeScaleRatioSpin.value());
-            archive.write("preInitialDuration", preInitialDurationSpin.value());
-            archive.write("postFinalDuration", postFinalDurationSpin.value());
-            archive.write("onlyTimeBarRange", onlyTimeBarRangeCheck.isChecked());
-            archive.write("makeNewBodyItem", newBodyItemCheck.isChecked());
-            archive.write("stealthyStepMode", stealthyStepCheck.isChecked());
-            archive.write("stealthyHeightRatioThresh", stealthyHeightRatioThreshSpin.value());
-            archive.write("flatLiftingHeight", flatLiftingHeightSpin.value());
-            archive.write("flatLandingHeight", flatLandingHeightSpin.value());
-            archive.write("impactReductionHeight", impactReductionHeightSpin.value());
-            archive.write("impactReductionTime", impactReductionTimeSpin.value());
-            archive.write("autoZmp", autoZmpCheck.isChecked());
-            archive.write("minZmpTransitionTime", minZmpTransitionTimeSpin.value());
-            archive.write("zmpCenteringTimeThresh", zmpCenteringTimeThreshSpin.value());
-            archive.write("zmpTimeMarginBeforeLiftingSpin", zmpTimeMarginBeforeLiftingSpin.value());
-            archive.write("allLinkPositions", se3Check.isChecked());
-            archive.write("lipSyncMix", lipSyncMixCheck.isChecked());
-        }
+    void storeState(Archive& archive){
+        archive.write("timeScaleRatio", timeScaleRatioSpin.value());
+        archive.write("preInitialDuration", preInitialDurationSpin.value());
+        archive.write("postFinalDuration", postFinalDurationSpin.value());
+        archive.write("onlyTimeBarRange", onlyTimeBarRangeCheck.isChecked());
+        archive.write("makeNewBodyItem", newBodyItemCheck.isChecked());
+        archive.write("stealthyStepMode", stealthyStepCheck.isChecked());
+        archive.write("stealthyHeightRatioThresh", stealthyHeightRatioThreshSpin.value());
+        archive.write("flatLiftingHeight", flatLiftingHeightSpin.value());
+        archive.write("flatLandingHeight", flatLandingHeightSpin.value());
+        archive.write("impactReductionHeight", impactReductionHeightSpin.value());
+        archive.write("impactReductionTime", impactReductionTimeSpin.value());
+        archive.write("autoZmp", autoZmpCheck.isChecked());
+        archive.write("minZmpTransitionTime", minZmpTransitionTimeSpin.value());
+        archive.write("zmpCenteringTimeThresh", zmpCenteringTimeThreshSpin.value());
+        archive.write("zmpTimeMarginBeforeLiftingSpin", zmpTimeMarginBeforeLiftingSpin.value());
+        archive.write("zmpMaxDistanceFromCenter", zmpMaxDistanceFromCenterSpin.value());
+        archive.write("allLinkPositions", se3Check.isChecked());
+        archive.write("lipSyncMix", lipSyncMixCheck.isChecked());
+    }
 
-        void restoreState(const Archive& archive){
-            timeScaleRatioSpin.setValue(archive.get("timeScaleRatio", timeScaleRatioSpin.value()));
-            preInitialDurationSpin.setValue(archive.get("preInitialDuration", preInitialDurationSpin.value()));
-            postFinalDurationSpin.setValue(archive.get("postFinalDuration", postFinalDurationSpin.value()));
-            onlyTimeBarRangeCheck.setChecked(archive.get("onlyTimeBarRange", onlyTimeBarRangeCheck.isChecked()));
-            newBodyItemCheck.setChecked(archive.get("makeNewBodyItem", newBodyItemCheck.isChecked()));
-            stealthyStepCheck.setChecked(archive.get("stealthyStepMode", stealthyStepCheck.isChecked()));
-            stealthyHeightRatioThreshSpin.setValue(archive.get("stealthyHeightRatioThresh", stealthyHeightRatioThreshSpin.value()));
-            flatLiftingHeightSpin.setValue(archive.get("flatLiftingHeight", flatLiftingHeightSpin.value()));
-            flatLandingHeightSpin.setValue(archive.get("flatLandingHeight", flatLandingHeightSpin.value()));
-            impactReductionHeightSpin.setValue(archive.get("impactReductionHeight", impactReductionHeightSpin.value()));
-            impactReductionTimeSpin.setValue(archive.get("impactReductionTime", impactReductionTimeSpin.value()));
+    void restoreState(const Archive& archive){
+        timeScaleRatioSpin.setValue(archive.get("timeScaleRatio", timeScaleRatioSpin.value()));
+        preInitialDurationSpin.setValue(archive.get("preInitialDuration", preInitialDurationSpin.value()));
+        postFinalDurationSpin.setValue(archive.get("postFinalDuration", postFinalDurationSpin.value()));
+        onlyTimeBarRangeCheck.setChecked(archive.get("onlyTimeBarRange", onlyTimeBarRangeCheck.isChecked()));
+        newBodyItemCheck.setChecked(archive.get("makeNewBodyItem", newBodyItemCheck.isChecked()));
+        stealthyStepCheck.setChecked(archive.get("stealthyStepMode", stealthyStepCheck.isChecked()));
+        stealthyHeightRatioThreshSpin.setValue(archive.get("stealthyHeightRatioThresh", stealthyHeightRatioThreshSpin.value()));
+        flatLiftingHeightSpin.setValue(archive.get("flatLiftingHeight", flatLiftingHeightSpin.value()));
+        flatLandingHeightSpin.setValue(archive.get("flatLandingHeight", flatLandingHeightSpin.value()));
+        impactReductionHeightSpin.setValue(archive.get("impactReductionHeight", impactReductionHeightSpin.value()));
+        impactReductionTimeSpin.setValue(archive.get("impactReductionTime", impactReductionTimeSpin.value()));
 
-            autoZmpCheck.setChecked(archive.get("autoZmp", autoZmpCheck.isChecked()));
-            minZmpTransitionTimeSpin.setValue(archive.get("minZmpTransitionTime", minZmpTransitionTimeSpin.value()));
-            zmpCenteringTimeThreshSpin.setValue(archive.get("zmpCenteringTimeThresh", zmpCenteringTimeThreshSpin.value()));
-            zmpTimeMarginBeforeLiftingSpin.setValue(archive.get("zmpTimeMarginBeforeLiftingSpin", zmpTimeMarginBeforeLiftingSpin.value()));
+        autoZmpCheck.setChecked(archive.get("autoZmp", autoZmpCheck.isChecked()));
+        minZmpTransitionTimeSpin.setValue(archive.get("minZmpTransitionTime", minZmpTransitionTimeSpin.value()));
+        zmpCenteringTimeThreshSpin.setValue(archive.get("zmpCenteringTimeThresh", zmpCenteringTimeThreshSpin.value()));
+        zmpTimeMarginBeforeLiftingSpin.setValue(archive.get("zmpTimeMarginBeforeLiftingSpin", zmpTimeMarginBeforeLiftingSpin.value()));
+        zmpMaxDistanceFromCenterSpin.setValue(archive.get("zmpMaxDistanceFromCenter", zmpMaxDistanceFromCenterSpin.value()));            
             
-            se3Check.setChecked(archive.get("allLinkPositions", se3Check.isChecked()));
-            lipSyncMixCheck.setChecked(archive.get("lipSyncMix", lipSyncMixCheck.isChecked()));
-        }
-    };
+        se3Check.setChecked(archive.get("allLinkPositions", se3Check.isChecked()));
+        lipSyncMixCheck.setChecked(archive.get("lipSyncMix", lipSyncMixCheck.isChecked()));
+    }
+};
+
 }
     
 
@@ -304,6 +318,9 @@ void BodyMotionGenerationBar::initializeInstance(ExtensionManager* ext)
         bar->autoInterpolationUpdateCheck = mm.addCheckItem(_("Automatic Interpolation Update"));
         bar->autoInterpolationUpdateCheck->setChecked(true);
 
+        bar->autoGenerationForNewBodyCheck = mm.addCheckItem(_("Automatic Generation for a New Body"));
+        bar->autoGenerationForNewBodyCheck->setChecked(true);
+
         mm.addSeparator();
     
         initialized = true;
@@ -319,32 +336,35 @@ BodyMotionGenerationBar* BodyMotionGenerationBar::instance()
 
 
 BodyMotionGenerationBar::BodyMotionGenerationBar()
-  : ToolBar("BodyMotionGenerationBar")
+    : ToolBar("BodyMotionGenerationBar")
 {
+    setVisibleByDefault(true);
+
     bodyMotionPoseProvider = new BodyMotionPoseProvider();
     poseProviderToBodyMotionConverter = new PoseProviderToBodyMotionConverter();
     timeBar = TimeBar::instance();
     setup = new BodyMotionGenerationSetupDialog();
+    balancer = 0;
 
     addButton(QIcon(":/PoseSeq/icons/trajectory-generation.png"), _("Generate body motions"))
-        ->sigClicked().connect(bind(&BodyMotionGenerationBar::onGenerationButtonClicked, this));
+        ->sigClicked().connect(boost::bind(&BodyMotionGenerationBar::onGenerationButtonClicked, this));
 
     interpolationParameterWidgetsConnection.add(
         setup->timeScaleRatioSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
     
     interpolationParameterWidgetsConnection.add(
         setup->preInitialDurationSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     interpolationParameterWidgetsConnection.add(
         setup->postFinalDurationSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     
     interpolationParameterWidgetsConnection.add(
         setup->onlyTimeBarRangeCheck.sigToggled().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
     
     autoGenerationToggle = addToggleButton(QIcon(":/PoseSeq/icons/auto-update.png"), _("Automatic Balance Adjustment Mode"));
     autoGenerationToggle->setChecked(false);
@@ -353,51 +373,55 @@ BodyMotionGenerationBar::BodyMotionGenerationBar()
     balancerToggle->setEnabled(false);
     balancerToggle->setChecked(false);
 
-    addButton(QIcon(":/Base/icons/setup.png"))->sigClicked().connect(bind(&BodyMotionGenerationSetupDialog::show, setup));
+    addButton(QIcon(":/Base/icons/setup.png"))->sigClicked().connect(boost::bind(&BodyMotionGenerationSetupDialog::show, setup));
     
     interpolationParameterWidgetsConnection.add(
         setup->stealthyStepCheck.sigToggled().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     interpolationParameterWidgetsConnection.add(
         setup->stealthyHeightRatioThreshSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
     
     interpolationParameterWidgetsConnection.add(
         setup->flatLiftingHeightSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     interpolationParameterWidgetsConnection.add(
         setup->flatLandingHeightSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
     
     interpolationParameterWidgetsConnection.add(
         setup->impactReductionHeightSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     interpolationParameterWidgetsConnection.add(
         setup->impactReductionTimeSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
     
     interpolationParameterWidgetsConnection.add(
         setup->autoZmpCheck.sigToggled().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     interpolationParameterWidgetsConnection.add(
         setup->minZmpTransitionTimeSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     interpolationParameterWidgetsConnection.add(
         setup->zmpCenteringTimeThreshSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
     interpolationParameterWidgetsConnection.add(
         setup->zmpTimeMarginBeforeLiftingSpin.sigValueChanged().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 
+    interpolationParameterWidgetsConnection.add(
+        setup->zmpMaxDistanceFromCenterSpin.sigValueChanged().connect(
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+    
     interpolationParameterWidgetsConnection.add(
         setup->lipSyncMixCheck.sigToggled().connect(
-            bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
+            boost::bind(&BodyMotionGenerationBar::notifyInterpolationParametersChanged, this)));
 }
 
 
@@ -420,11 +444,11 @@ void BodyMotionGenerationBar::onGenerationButtonClicked()
     ItemList<Item> selectedItems = ItemTreeView::mainInstance()->selectedItems<Item>();
 
     for(size_t i=0; i < selectedItems.size(); ++i){
-        PoseSeqItem* poseSeqItem = dynamic_cast<PoseSeqItem*>(selectedItems[i]);
+        PoseSeqItem* poseSeqItem = selectedItems.get<PoseSeqItem>(i);
         if(poseSeqItem){
             motionItems.insert(poseSeqItem->bodyMotionItem());
         } else {
-            BodyMotionItem* motionItem = dynamic_cast<BodyMotionItem*>(selectedItems[i]);
+            BodyMotionItem* motionItem = selectedItems.get<BodyMotionItem>(i);
             if(motionItem){
                 motionItems.insert(motionItem);
             }
@@ -456,20 +480,23 @@ void BodyMotionGenerationBar::onGenerationButtonClicked()
 }
 
 
-void BodyMotionGenerationBar::setBalancer(BalancerFunc func, QWidget* panel)
+void BodyMotionGenerationBar::setBalancer(Balancer* balancer)
 {
-    applyBalancer = func;
-    balancerToggle->setEnabled(applyBalancer != 0);
-    balancerPanel = panel;
-    setup->vbox->addWidget(panel);
+    this->balancer = balancer;
+    balancerToggle->setEnabled(balancer != 0);
+    if(balancer){
+        setup->vbox->addWidget(balancer->panel());
+    }
 }
 
 
 void BodyMotionGenerationBar::unsetBalancer()
 {
-    applyBalancer.clear();
     balancerToggle->setEnabled(false);
-    setup->layout()->removeWidget(balancerPanel);
+    if(balancer){
+        setup->layout()->removeWidget(balancer->panel());
+        balancer = 0;
+    }
 }
 
 
@@ -478,8 +505,8 @@ bool BodyMotionGenerationBar::shapeBodyMotion
 {
     bool result = false;
     
-    if(balancerToggle->isChecked() && applyBalancer){
-        result = applyBalancer(body, provider, motionItem, putMessages);
+    if(balancerToggle->isChecked() && balancer){
+        result = balancer->apply(body, provider, motionItem, putMessages);
     } else {
         result = shapeBodyMotionWithSimpleInterpolation(body, provider, motionItem);
     }
@@ -504,7 +531,6 @@ bool BodyMotionGenerationBar::shapeBodyMotionWithSimpleInterpolation
     bool result = poseProviderToBodyMotionConverter->convert(body, provider, *motion);
     
     if(result){
-        motionItem->updateChildItemLineup();
         motionItem->notifyUpdate();
     }
     return result;
@@ -513,20 +539,26 @@ bool BodyMotionGenerationBar::shapeBodyMotionWithSimpleInterpolation
 
 bool BodyMotionGenerationBar::storeState(Archive& archive)
 {
+    archive.write("autoGenerationForNewBody", autoGenerationForNewBodyCheck->isChecked());
     archive.write("balancer", balancerToggle->isChecked());
     archive.write("autoGeneration", autoGenerationToggle->isChecked());
     setup->storeState(archive);
+    if(balancer){
+        balancer->storeState(archive);
+    }
     return true;
 }
 
 
 bool BodyMotionGenerationBar::restoreState(const Archive& archive)
 {
+    autoGenerationForNewBodyCheck->setChecked(archive.get("autoGenerationForNewBody", autoGenerationForNewBodyCheck->isChecked()));
     balancerToggle->setChecked(archive.get("balancer", balancerToggle->isChecked()));
     autoGenerationToggle->setChecked(archive.get("autoGeneration", autoGenerationToggle->isChecked()));
-
     setup->restoreState(archive);
-
+    if(balancer){
+        balancer->restoreState(archive);
+    }
     return true;
 }
 
@@ -549,6 +581,12 @@ bool BodyMotionGenerationBar::isAutoGenerationMode() const
 }
 
 
+bool BodyMotionGenerationBar::isAutoGenerationForNewBodyEnabled() const
+{
+    return autoGenerationForNewBodyCheck->isChecked();
+}
+
+
 double BodyMotionGenerationBar::timeScaleRatio() const
 {
     return setup->timeScaleRatioSpin.value();
@@ -622,6 +660,12 @@ double BodyMotionGenerationBar::zmpTimeMarginBeforeLifting() const
 }
 
 
+double BodyMotionGenerationBar::zmpMaxDistanceFromCenter() const
+{
+    return setup->zmpMaxDistanceFromCenterSpin.value();
+}
+
+
 bool BodyMotionGenerationBar::isSe3Enabled() const
 {
     return setup->se3Check.isChecked();
