@@ -7,8 +7,11 @@
 #include "PoseSeqViewBase.h"
 #include "PronunSymbol.h"
 #include <cnoid/ScrollBar>
+#include <cnoid/CheckBox>
 #include <cnoid/Separator>
 #include <cnoid/EigenUtil>
+#include <cnoid/ViewManager>
+#include <cnoid/ItemTreeView>
 #include <QGridLayout>
 #include <QHeaderView>
 #include <QPaintEvent>
@@ -25,199 +28,199 @@ using namespace cnoid;
 
 namespace {
 
-    const bool TRACE_FUNCTIONS = false;
-    const bool FAST_DRAW_MODE = false;
+const bool TRACE_FUNCTIONS = false;
+const bool FAST_DRAW_MODE = false;
 
-    const double leftMargin = 0.2;
+const double leftMargin = 0.2;
 
-    class ScrollBarEx : public ScrollBar
-    {
-    public:
-        ScrollBarEx(Qt::Orientation orientation) : ScrollBar(orientation) { }
-        virtual void sliderChange(SliderChange change){
-            ScrollBar::sliderChange(change);
-            if(change == QAbstractSlider::SliderStepsChange){
-                if((maximum() - minimum()) == 0){
-                    hide();
-                } else {
-                    show();
-                }
+class ScrollBarEx : public ScrollBar
+{
+public:
+    ScrollBarEx(Qt::Orientation orientation) : ScrollBar(orientation) { }
+    virtual void sliderChange(SliderChange change){
+        ScrollBar::sliderChange(change);
+        if(change == QAbstractSlider::SliderStepsChange){
+            if((maximum() - minimum()) == 0){
+                hide();
+            } else {
+                show();
             }
         }
-    };
+    }
+};
 }
 
 namespace cnoid {
 
-    class PoseRollViewImpl : public PoseSeqViewBase
-    {
-    public:
-        PoseRollViewImpl(PoseRollView* self);
-        ~PoseRollViewImpl();
-
-        PoseRollView* self;
-        QGridLayout* gridLayout;
-        ScrollBarEx* treeWidgetVerticalScrollBar;
-        QWidget* screen;
-        QVector<qreal> dash;
-        QPainter painter;
-        QPen markerPen;
-        QPen pronumSymbolPen;
-        QPen selectedMarkerPen;
-        QPen gridPen;
-        QPen cursorPen;
-
-        ToolButton commandMenuButton;
-        Menu commandMenu;
-        MenuManager commandMenuManager;
-        Action* lipSyncCheck;
-
-        QLabel poseNameLabel;
-        DoubleSpinBox currentTimeSpin;
-        signals::connection currentTimeSpinConnection;
-        DoubleSpinBox poseTimeSpin;
-        signals::connection poseTimeSpinConnection;
-        DoubleSpinBox poseTTimeSpin;
-        signals::connection poseTTimeSpinConnection;
-
-        DoubleScrollBar* hScrollBar;
-        signals::connection hScrollBarChangedConnection;
-        bool isTmpScrollBlocked;
-
-        double left;
-        double right;
-        double treeTopOnScreen;
-        double screenWidth;
-        double screenHeight;
-        double treeBottomOnScreen;
-        double timeToScreenX;
-        double gridStep;
-
-        double timeLength;
-        DoubleSpinBox timeLengthSpin;
-        DoubleSpinBox gridIntervalSpin;
-
-        bool updateRowRectsNeeded;
-            
-        struct RowInfo {
-            RowInfo() { }
-            bool isVisible;
-            int visibleRowIndex;
-            int y;
-            int height;
-            int linkIndex;
-            int jointId;
-        };
-        vector<RowInfo> itemIndexToRowInfoMap;
-                
-        vector<LinkTreeItem*> visibleRows;
-        vector<LinkTreeItem*> linkIndexToVisibleRowAncestorMap;
-        LinkTreeItem* visibleRowAncestorOfZmp;
-
-        struct RowRenderInfo {
-            bool rendered;
-            double lastPoseTime;
-        };
-        vector<RowRenderInfo> rowRenderInfos;
-
-        double pointerX;
-        double pointerY;
-        double pressedScreenX;
-        double dragOrgLeft;
-        double dragOrgScale;
-        double pickDistance;
-        PoseSeq::iterator pickedPoseIter;
-        enum PickedPart { PICK_NONE, PICK_LEFT, PICK_BODY, PICK_RIGHT };
-        PickedPart pickedPart;
-        double pickedTime;
-
-        enum DragMode { DRAG_NONE, DRAG_POSES, DRAG_TRANSITION_TIME, DRAG_TIME_CURSOR, DRAG_SCALING } dragMode;
-        enum DragState { DS_INITIAL, DS_DRAGGED } dragState;
-
-        double draggingPosesOrgTime;
-            
-        // for key pose marker rendering
-        PoseSeq::iterator markerPoseIter;
-        double markerTime0;
-        double markerX0;
-        double markerX1;
-        double markerY0;
-        double markerY1;
-        bool isMarkerPronunSymbol;
-        bool isMarkerSelected;
-
-        void initialize();
-        QHBoxLayout* layoutOperationParts();
-        void setupScreen();
-            
-        virtual void setCurrentPoseSeqItem(PoseSeqItemPtr poseSeqItem);
-        void requestRowRectsUpdate();
-        void onTimeLengthChanged(double value);
-        void onPoseTimeSpinChanged(double value);
-        void onPoseTTimeSpinChanged(double value);
-        virtual void onInsertPoseButtonClicked();
-        virtual void onPoseInserted(PoseSeq::iterator it, bool isMoving);
-        virtual void onPoseRemoving(PoseSeq::iterator it, bool isMoving);
-        virtual void onPoseModified(PoseSeq::iterator it);
-        void onCurrentTimeSpinChanged(double value);
-        void setCurrentTime(double time, bool blockScroll = false);
-        virtual bool onTimeChanged(double time);
-        void setTimeOfScreenLeft(double time, bool changeScrollBar = true, bool forceChange = false);
-        void onHScrollbarChanged(double value);
-
-        void onGridResolutionChanged(double value);
-
-        virtual void onLinkTreeUpdateRequest(bool isInitialCreation);
-        void updateRowRects();
-        void updateRowRectsSub(QTreeWidgetItem* treeWidgetItem);
-        LinkTreeItem* getFirstVisibleAncestor(const LinkTreeItem* item);
-        bool checkIfPoseHasRow(const PosePtr& pose, const LinkTreeItem* item);
-        double searchLastPoseTime(const LinkTreeItem* item);
-        void processKeyPoseMarkersSub(LinkTreeItem* item, boost::function<void()> func);
-        void processKeyPoseMarkers(boost::function<void()> func);
-            
-        bool onScreenPaintEvent(QPaintEvent* event);        
-        void drawBackground();
-        void drawTimeCursor();
-        void drawKeyPoseMarker();
-
-        virtual void onTimeScaleChanged();
-            
-        virtual void onSelectedPosesModified();
-        void pickPose();
-        void pickPoseSub();
-            
-        void onScreenFontChanged();
-        bool onScreenResizeEvent(QResizeEvent* event);
-        bool onScreenMouseButtonPressEvent(QMouseEvent* event);
-        void pickPoseOnButtonPress(bool isAdding);
-        bool onScreenMouseButtonReleaseEvent(QMouseEvent* event);
-        bool onScreenMouseMoveEvent(QMouseEvent* event);
-        void pickPoseOnMotionNotify();
-        void dragSelectedPoses();
-        void dragTransitionTime();
-        void dragScaling();
-        bool onScreenKeyPressEvent(QKeyEvent* event);
-        bool onScreenKeyReleaseEvent(QKeyEvent* event);
-        void selectPrevPose(bool isAdding);
-        void selectNextPose(bool isAdding);
-        void onMenuButtonClicked();
-
-        bool storeState(Archive& archive);
-        bool restoreState(const Archive& archive);
-    };
-}
-
-
-PoseRollView* PoseRollView::instance()
+class PoseRollViewImpl : public PoseSeqViewBase
 {
-	static PoseRollView* instance = new PoseRollView();
-	return instance;
+public:
+    PoseRollViewImpl(PoseRollView* self);
+    ~PoseRollViewImpl();
+
+    PoseRollView* self;
+    QGridLayout* gridLayout;
+    ScrollBarEx* treeWidgetVerticalScrollBar;
+    QWidget* screen;
+    QVector<qreal> dash;
+    QPainter painter;
+    QPen markerPen;
+    QPen pronumSymbolPen;
+    QPen selectedMarkerPen;
+    QPen gridPen;
+    QPen cursorPen;
+
+    ToolButton commandMenuButton;
+    Menu commandMenu;
+    MenuManager commandMenuManager;
+    Action* lipSyncCheck;
+
+    QLabel poseNameLabel;
+    DoubleSpinBox currentTimeSpin;
+    Connection currentTimeSpinConnection;
+    DoubleSpinBox poseTimeSpin;
+    Connection poseTimeSpinConnection;
+    DoubleSpinBox poseTTimeSpin;
+    Connection poseTTimeSpinConnection;
+
+    DoubleScrollBar* hScrollBar;
+    Connection hScrollBarChangedConnection;
+    bool isTmpScrollBlocked;
+
+    double left;
+    double right;
+    double treeTopOnScreen;
+    double screenWidth;
+    double screenHeight;
+    double treeBottomOnScreen;
+    double timeToScreenX;
+    double barLength;
+
+    double timeLength;
+    DoubleSpinBox timeLengthSpin;
+    DoubleSpinBox gridIntervalSpin;
+
+    bool updateRowRectsNeeded;
+            
+    struct RowInfo {
+        RowInfo() { }
+        bool isVisible;
+        int visibleRowIndex;
+        int y;
+        int height;
+        int linkIndex;
+        int jointId;
+    };
+    vector<RowInfo> itemIndexToRowInfoMap;
+                
+    vector<LinkTreeItem*> visibleRows;
+    vector<LinkTreeItem*> linkIndexToVisibleRowAncestorMap;
+    LinkTreeItem* visibleRowAncestorOfZmp;
+
+    struct RowRenderInfo {
+        bool rendered;
+        double lastPoseTime;
+    };
+    vector<RowRenderInfo> rowRenderInfos;
+
+    double pointerX;
+    double pointerY;
+    double pressedScreenX;
+    double dragOrgLeft;
+    double dragOrgScale;
+    double pickDistance;
+    PoseSeq::iterator pickedPoseIter;
+    enum PickedPart { PICK_NONE, PICK_LEFT, PICK_BODY, PICK_RIGHT };
+    PickedPart pickedPart;
+    double pickedTime;
+
+    enum DragMode { DRAG_NONE, DRAG_POSES, DRAG_TRANSITION_TIME, DRAG_TIME_CURSOR, DRAG_SCALING } dragMode;
+    enum DragState { DS_INITIAL, DS_DRAGGED } dragState;
+
+    double draggingPosesOrgTime;
+            
+    // for key pose marker rendering
+    PoseSeq::iterator markerPoseIter;
+    double markerTime0;
+    double markerX0;
+    double markerX1;
+    double markerY0;
+    double markerY1;
+    bool isMarkerPronunSymbol;
+    bool isMarkerSelected;
+
+    void initialize();
+    QHBoxLayout* layoutOperationParts();
+    void setupScreen();
+            
+    virtual void setCurrentPoseSeqItem(PoseSeqItemPtr poseSeqItem);
+    void requestRowRectsUpdate();
+    void onTimeLengthChanged(double value);
+    void onPoseTimeSpinChanged(double value);
+    void onPoseTTimeSpinChanged(double value);
+    virtual void onInsertPoseButtonClicked();
+    virtual void onPoseInserted(PoseSeq::iterator it, bool isMoving);
+    virtual void onPoseRemoving(PoseSeq::iterator it, bool isMoving);
+    virtual void onPoseModified(PoseSeq::iterator it);
+    void onCurrentTimeSpinChanged(double value);
+    void setCurrentTime(double time, bool blockScroll = false);
+    virtual bool onTimeChanged(double time);
+    void setTimeOfScreenLeft(double time, bool changeScrollBar = true, bool forceChange = false);
+    void onHScrollbarChanged(double value);
+
+    void onGridResolutionChanged(double value);
+
+    virtual void onLinkTreeUpdateRequest(bool isInitialCreation);
+    void updateRowRects();
+    void updateRowRectsSub(QTreeWidgetItem* treeWidgetItem);
+    LinkTreeItem* getFirstVisibleAncestor(const LinkTreeItem* item);
+    bool checkIfPoseHasRow(const PosePtr& pose, const LinkTreeItem* item);
+    double searchLastPoseTime(const LinkTreeItem* item);
+    void processKeyPoseMarkersSub(LinkTreeItem* item, boost::function<void()> func);
+    void processKeyPoseMarkers(boost::function<void()> func);
+            
+    bool onScreenPaintEvent(QPaintEvent* event);        
+    void drawBackground();
+    void drawTimeCursor();
+    void drawKeyPoseMarker();
+
+    virtual void onTimeScaleChanged();
+            
+    virtual void onSelectedPosesModified();
+    void pickPose();
+    void pickPoseSub();
+            
+    void onScreenFontChanged();
+    bool onScreenResizeEvent(QResizeEvent* event);
+    bool onScreenMouseButtonPressEvent(QMouseEvent* event);
+    void pickPoseOnButtonPress(bool isAdding);
+    bool onScreenMouseButtonReleaseEvent(QMouseEvent* event);
+    bool onScreenMouseMoveEvent(QMouseEvent* event);
+    void pickPoseOnMotionNotify();
+    void dragSelectedPoses();
+    void dragTransitionTime();
+    void dragScaling();
+    bool onScreenKeyPressEvent(QKeyEvent* event);
+    bool onScreenKeyReleaseEvent(QKeyEvent* event);
+    void selectPrevPose(bool isAdding);
+    void selectNextPose(bool isAdding);
+    void onMenuButtonClicked();
+
+    bool storeState(Archive& archive);
+    bool restoreState(const Archive& archive);
+};
 }
 
+
+void PoseRollView::initializeClass(ExtensionManager* ext)
+{
+    ext->viewManager().registerClass<PoseRollView>(
+        "PoseRollView", N_("Pose Roll"), ViewManager::SINGLE_OPTIONAL);
+}
+
+
 PoseRollView::PoseRollView()
 {
-    setName(N_("Pose Roll"));
     setDefaultLayoutArea(View::BOTTOM);
     setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
 
@@ -258,20 +261,20 @@ void PoseRollViewImpl::initialize()
     linkTreeWidget->setVerticalScrollBar(treeWidgetVerticalScrollBar);
     linkTreeWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     treeWidgetVerticalScrollBar->sigValueChanged().connect(
-        bind(&PoseRollViewImpl::requestRowRectsUpdate, this));
+        boost::bind(&PoseRollViewImpl::requestRowRectsUpdate, this));
 
     // setup the link tree view model
     linkTreeWidget->sigItemExpanded().connect(
-        bind(&PoseRollViewImpl::requestRowRectsUpdate, this));
+        boost::bind(&PoseRollViewImpl::requestRowRectsUpdate, this));
     linkTreeWidget->sigItemCollapsed().connect(
-        bind(&PoseRollViewImpl::requestRowRectsUpdate, this));
+        boost::bind(&PoseRollViewImpl::requestRowRectsUpdate, this));
     treeBox->addWidget(linkTreeWidget, 1);
 
     hScrollBar = new DoubleScrollBar(Qt::Horizontal);
     hScrollBar->setSingleStep(0.1);
     hScrollBar->setRange(-leftMargin, timeLength + (2.0 / timeToScreenX));
     hScrollBarChangedConnection = hScrollBar->sigValueChanged().connect(
-        bind(&PoseRollViewImpl::onHScrollbarChanged, this, _1));
+        boost::bind(&PoseRollViewImpl::onHScrollbarChanged, this, _1));
 
     QHBoxLayout* hbox = new QHBoxLayout();
     hbox->setSpacing(8);
@@ -293,24 +296,26 @@ void PoseRollViewImpl::initialize()
     self->setLayout(vbox);
 
     commandMenuManager.addItem(_("Select specified key poses"))->sigTriggered().connect(
-        bind(&PoseRollViewImpl::onSelectSpecifiedKeyPosesActivated, this));
+        boost::bind(&PoseRollViewImpl::onSelectSpecifiedKeyPosesActivated, this));
     commandMenuManager.addItem(_("Adjust step positions"))->sigTriggered().connect(
-        bind(&PoseRollViewImpl::onAdjustStepPositionsActivated, this));
+        boost::bind(&PoseRollViewImpl::onAdjustStepPositionsActivated, this));
     commandMenuManager.addItem(_("Adjust waist positions of selected key poses"))->sigTriggered().connect(
-        bind(&PoseRollViewImpl::onAdjustWaistPositionActivated, this));
+        boost::bind(&PoseRollViewImpl::onAdjustWaistPositionActivated, this));
     commandMenuManager.addItem(_("Rotate yaw orientations"))->sigTriggered().connect(
-        bind(&PoseRollViewImpl::onRotateYawOrientationsActivated, this));
+        boost::bind(&PoseRollViewImpl::onRotateYawOrientationsActivated, this));
     commandMenuManager.addItem(_("Update key poses with balanced trajectories"))->sigTriggered().connect(
-        bind(&PoseRollViewImpl::onUpdateKeyposesWithBalancedTrajectoriesActivated, this));
+        boost::bind(&PoseRollViewImpl::onUpdateKeyposesWithBalancedTrajectoriesActivated, this));
     commandMenuManager.addItem(_("Flip poses against the x-z plane"))->sigTriggered().connect(
-        bind(&PoseRollViewImpl::onFlipPosesActivated, this));
+        boost::bind(&PoseRollViewImpl::onFlipPosesActivated, this));
 
     lipSyncCheck = commandMenuManager.addCheckItem(_("Show lip-sync elements"));
-    lipSyncCheck->sigToggled().connect(bind(&QWidget::update, screen));
+    lipSyncCheck->sigToggled().connect(boost::bind(&QWidget::update, screen));
 
     commandMenuButton.setText(_("Menu"));
     commandMenuButton.setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
-    commandMenuButton.sigClicked().connect(bind(&PoseRollViewImpl::onMenuButtonClicked, this));
+    commandMenuButton.sigClicked().connect(boost::bind(&PoseRollViewImpl::onMenuButtonClicked, this));
+
+    onItemSelectionChanged(ItemTreeView::instance()->selectedItems());
 }
 
 
@@ -345,7 +350,7 @@ QHBoxLayout* PoseRollViewImpl::layoutOperationParts()
     currentTimeSpin.setRange(0.0, 999.999);
     currentTimeSpin.setSingleStep(0.005);
     currentTimeSpinConnection = currentTimeSpin.sigValueChanged().connect(
-        bind(&PoseRollViewImpl::onCurrentTimeSpinChanged, this, _1));
+        boost::bind(&PoseRollViewImpl::onCurrentTimeSpinChanged, this, _1));
     hbox->addWidget(&currentTimeSpin);
 
     hbox->addWidget(new QLabel(" / "));
@@ -356,7 +361,7 @@ QHBoxLayout* PoseRollViewImpl::layoutOperationParts()
     timeLengthSpin.setSingleStep(1.0);
     timeLengthSpin.setValue(timeLength);
     timeLengthSpin.sigValueChanged().connect(
-        bind(&PoseRollViewImpl::onTimeLengthChanged, this, _1));
+        boost::bind(&PoseRollViewImpl::onTimeLengthChanged, this, _1));
     hbox->addWidget(&timeLengthSpin);
 
     hbox->addWidget(&timeSyncCheck);
@@ -375,7 +380,7 @@ QHBoxLayout* PoseRollViewImpl::layoutOperationParts()
     poseTimeSpin.setRange(0.0, 999.999);
     poseTimeSpin.setSingleStep(0.005);
     poseTimeSpinConnection = poseTimeSpin.sigValueChanged().connect(
-        bind(&PoseRollViewImpl::onPoseTimeSpinChanged, this, _1));
+        boost::bind(&PoseRollViewImpl::onPoseTimeSpinChanged, this, _1));
     hbox->addWidget(&poseTimeSpin);
 
     hbox->addWidget(new QLabel(_("TT:")));
@@ -386,7 +391,7 @@ QHBoxLayout* PoseRollViewImpl::layoutOperationParts()
     poseTTimeSpin.setRange(0.0, 9.999);
     poseTTimeSpin.setSingleStep(0.005);
     poseTTimeSpinConnection = poseTTimeSpin.sigValueChanged().connect(
-        bind(&PoseRollViewImpl::onPoseTTimeSpinChanged, this, _1));
+        boost::bind(&PoseRollViewImpl::onPoseTTimeSpinChanged, this, _1));
     hbox->addWidget(&poseTTimeSpin);
 
     hbox->addWidget(&deleteButton);
@@ -397,7 +402,7 @@ QHBoxLayout* PoseRollViewImpl::layoutOperationParts()
     gridIntervalSpin.setRange(1.0, 10.0);
     gridIntervalSpin.setSingleStep(1.0);
     gridIntervalSpin.sigValueChanged().connect(
-        bind(&PoseRollViewImpl::onGridResolutionChanged, this, _1));
+        boost::bind(&PoseRollViewImpl::onGridResolutionChanged, this, _1));
     hbox->addWidget(&gridIntervalSpin);
 
     hbox->addStretch();
@@ -413,13 +418,16 @@ void PoseRollViewImpl::setupScreen()
     screen->installEventFilter(self);
     screen->setBackgroundRole(QPalette::Base);
     screen->setAutoFillBackground(true);
+
+    dragMode = DRAG_NONE;
+    dragState = DS_INITIAL;
     
     left = -leftMargin;
     right = 0.0;
     screenWidth = 0.0;
     screenHeight = 0.0;
     timeToScreenX = 120.0;
-    gridStep = 1.0;
+    barLength = 1.0;
 
     dash.push_back(2.0);
     dash.push_back(2.0);
@@ -477,6 +485,12 @@ bool PoseRollView::eventFilter(QObject* obj, QEvent* event)
 }
 
 
+PoseSeqItem* PoseRollView::currentPoseSeqItem()
+{
+    return impl->currentPoseSeqItem.get();
+}
+
+
 void PoseRollViewImpl::setCurrentPoseSeqItem(PoseSeqItemPtr poseSeqItem)
 {
     BodyPtr prevBody = body;
@@ -489,6 +503,7 @@ void PoseRollViewImpl::setCurrentPoseSeqItem(PoseSeqItemPtr poseSeqItem)
         if(timeLengthSpin.value() < upper){
             timeLengthSpin.setValue(upper);
         }
+        barLength = poseSeqItem->barLength();
     }
 
     if(body != prevBody){
@@ -549,10 +564,6 @@ void PoseRollViewImpl::onPoseTTimeSpinChanged(double value)
     }
 }
 
-void PoseRollView::onInsertPoseButtonClicked()
-{
-	return impl->onInsertPoseButtonClicked();
-}
 
 void PoseRollViewImpl::onInsertPoseButtonClicked()
 {
@@ -690,7 +701,6 @@ void PoseRollViewImpl::onHScrollbarChanged(double value)
 
 void PoseRollViewImpl::onGridResolutionChanged(double value)
 {
-    gridStep = 1.0 / value;
     screen->update();
 }
 
@@ -759,9 +769,9 @@ void PoseRollViewImpl::updateRowRectsSub(QTreeWidgetItem* treeWidgetItem)
                 visibleRowAncestorOfZmp = rowInfo.isVisible ? item : getFirstVisibleAncestor(item);
             }
         } else {
-            rowInfo.linkIndex = link->index;
-            rowInfo.jointId = link->jointId;
-            linkIndexToVisibleRowAncestorMap[link->index] = rowInfo.isVisible ? item : getFirstVisibleAncestor(item);
+            rowInfo.linkIndex = link->index();
+            rowInfo.jointId = link->jointId();
+            linkIndexToVisibleRowAncestorMap[link->index()] = rowInfo.isVisible ? item : getFirstVisibleAncestor(item);
         }
     }
 
@@ -900,7 +910,7 @@ void PoseRollViewImpl::processKeyPoseMarkers(boost::function<void()> func)
             for(int i=0; i < n; ++i){
                 Link* joint = body->joint(i);
                 if(pose->isJointValid(i) && joint->isValid()){
-                    processKeyPoseMarkersSub(linkIndexToVisibleRowAncestorMap[joint->index], func);
+                    processKeyPoseMarkersSub(linkIndexToVisibleRowAncestorMap[joint->index()], func);
                 }
             }
             for(Pose::LinkInfoMap::iterator it = pose->ikLinkBegin(); it != pose->ikLinkEnd(); ++it){
@@ -941,7 +951,7 @@ bool PoseRollViewImpl::onScreenPaintEvent(QPaintEvent* event)
     painter.setClipping(true);
 
     if(seq){
-        processKeyPoseMarkers(bind(&PoseRollViewImpl::drawKeyPoseMarker, this));
+        processKeyPoseMarkers(boost::bind(&PoseRollViewImpl::drawKeyPoseMarker, this));
     }
 
     painter.setClipping(false);
@@ -986,8 +996,12 @@ void PoseRollViewImpl::drawBackground()
     }
 
     // draw time grid lines
-    double t = floor(left + 0.9999); // for avoiding -0.0
+    //double t = floor(left + 0.9999); // for avoiding -0.0
 
+    double step = barLength / gridIntervalSpin.value();
+    double t = floor(left / step) * step;
+    if(t != left) t += step;
+    
     while(true){
 
         double x = timeToScreenX * (t - left);
@@ -1005,7 +1019,7 @@ void PoseRollViewImpl::drawBackground()
 
         painter.drawLine(x, treeTopOnScreen, x, rowBottom);
 
-        t += gridStep;
+        t += step;
     }
 }
 
@@ -1101,7 +1115,7 @@ void PoseRollViewImpl::pickPose()
         pickedPoseIter = seq->end();
         pickDistance = std::numeric_limits<double>::max();
         pickedPart = PICK_NONE;
-        processKeyPoseMarkers(bind(&PoseRollViewImpl::pickPoseSub, this));
+        processKeyPoseMarkers(boost::bind(&PoseRollViewImpl::pickPoseSub, this));
     }
 }
 
@@ -1333,6 +1347,14 @@ bool PoseRollViewImpl::onScreenKeyPressEvent(QKeyEvent* event)
 {
     bool handled = false;
 
+    if(event->key() == Qt::Key_Space){
+        QMouseEvent mouseEvent(QEvent::MouseButtonPress,
+                               QPoint(pointerX, pointerY),
+                               Qt::MidButton, Qt::MidButton,
+                               event->modifiers());
+        return onScreenMouseButtonPressEvent(&mouseEvent);
+    }
+
     bool isCtrlActive = (event->modifiers() & Qt::ControlModifier);
 
     if(isCtrlActive){
@@ -1393,6 +1415,14 @@ bool PoseRollViewImpl::onScreenKeyPressEvent(QKeyEvent* event)
 
 bool PoseRollViewImpl::onScreenKeyReleaseEvent(QKeyEvent* event)
 {
+    if(event->key() == Qt::Key_Space){
+        QMouseEvent mouseEvent(QEvent::MouseButtonPress,
+                               QPoint(pointerX, pointerY),
+                               Qt::MidButton, Qt::MidButton,
+                               event->modifiers());
+        return onScreenMouseButtonReleaseEvent(&mouseEvent);
+    }
+    
     return false;
 }
 
