@@ -10,7 +10,7 @@
 #include <cnoid/MessageView>
 #include <cnoid/LinkPath>
 #include <cnoid/BodyItem>
-#include <cnoid/LeggedBody>
+#include <cnoid/LeggedBodyHelper>
 #include <boost/bind.hpp>
 #include <boost/format.hpp>
 #include <iostream>
@@ -22,55 +22,55 @@ using namespace cnoid;
 
 namespace {
 
-    const bool TRACE_FUNCTIONS = false;
+const bool TRACE_FUNCTIONS = false;
 
-    bool loadPoseSeqItem(PoseSeqItem* item, const std::string& filename, std::ostream& os, Item* parentItem)
-    {
-        bool loaded = false;
-        BodyItem* bodyItem = parentItem->findOwnerItem<BodyItem>(true);
-        if(bodyItem){
-            item->clearEditHistory();
-            loaded = item->poseSeq()->load(filename, bodyItem->body());
-            if(loaded){
-                const string& name = item->poseSeq()->name();
-                if(!name.empty()){
-                    item->setName(name);
-                }
-                if(item->poseSeq()->targetBodyName() != bodyItem->body()->name()){
-                    format m(_("Warning: the original target body %1% of \"%2%\" is"
-                               "different from the current target %3%."));
-                    os << str(m % item->poseSeq()->targetBodyName() % item->name() % bodyItem->body()->name());
-                }
-                item->notifyUpdate();
-            } else {
-                os << item->poseSeq()->errorMessage();
+bool loadPoseSeqItem(PoseSeqItem* item, const std::string& filename, std::ostream& os, Item* parentItem)
+{
+    bool loaded = false;
+    BodyItem* bodyItem = parentItem->findOwnerItem<BodyItem>(true);
+    if(bodyItem){
+        item->clearEditHistory();
+        loaded = item->poseSeq()->load(filename, bodyItem->body());
+        if(loaded){
+            const string& name = item->poseSeq()->name();
+            if(!name.empty()){
+                item->setName(name);
             }
+            if(item->poseSeq()->targetBodyName() != bodyItem->body()->name()){
+                format m(_("Warning: the original target body %1% of \"%2%\" is"
+                           "different from the current target %3%."));
+                os << str(m % item->poseSeq()->targetBodyName() % item->name() % bodyItem->body()->name());
+            }
+            item->notifyUpdate();
         } else {
-            os << _("PoseSeqItem must be loaded as a child of a BodyItem");
+            os << item->poseSeq()->errorMessage();
         }
-        return loaded;
+    } else {
+        os << _("PoseSeqItem must be loaded as a child of a BodyItem");
     }
+    return loaded;
+}
     
-    bool savePoseSeqItem(PoseSeqItem* item, const std::string& filename, std::ostream& os, Item* parentItem)
-    {
-        BodyItem* bodyItem = parentItem->findOwnerItem<BodyItem>(true);
-        if(bodyItem){
-            return item->poseSeq()->save(filename, bodyItem->body());
-        } else {
-            os << "PoseSeqItem to save must be a child of a BodyItem ";
-        }
-        return false;
+bool savePoseSeqItem(PoseSeqItem* item, const std::string& filename, std::ostream& os, Item* parentItem)
+{
+    BodyItem* bodyItem = parentItem->findOwnerItem<BodyItem>(true);
+    if(bodyItem){
+        return item->poseSeq()->save(filename, bodyItem->body());
+    } else {
+        os << "PoseSeqItem to save must be a child of a BodyItem ";
     }
+    return false;
+}
 
-    bool exportTalkPluginFormat(PoseSeqItem* item, const std::string& filename)
-    {
-        return item->poseSeq()->exportTalkPluginFile(filename);
-    }
+bool exportTalkPluginFormat(PoseSeqItem* item, const std::string& filename)
+{
+    return item->poseSeq()->exportTalkPluginFile(filename);
+}
 
-    bool exportFaceControllerFormat(PoseSeqItem* item, const std::string& filename)
-    {
-        return item->poseSeq()->exportSeqFileForFaceController(filename);
-    }    
+bool exportFaceControllerFormat(PoseSeqItem* item, const std::string& filename)
+{
+    return item->poseSeq()->exportSeqFileForFaceController(filename);
+}    
 }
 
 
@@ -87,10 +87,10 @@ void PoseSeqItem::initializeClass(ExtensionManager* ext)
             _("Pose Sequence"), "POSE-SEQ-YAML", "pseq", loadPoseSeqItem, savePoseSeqItem);
         im.addSaver<PoseSeqItem>(
             _("Talk Plugin File"), "TALK-PLUGIN-FORMAT", "talk",
-            bind(exportTalkPluginFormat, _1, _2), ItemManager::PRIORITY_CONVERSION);
+            boost::bind(exportTalkPluginFormat, _1, _2), ItemManager::PRIORITY_CONVERSION);
         im.addSaver<PoseSeqItem>(
             _("Seq File for the Face Controller"), "FACE-CONTROLLER-SEQ-FORMAT", "poseseq",
-            bind(exportFaceControllerFormat, _1, _2), ItemManager::PRIORITY_CONVERSION);
+            boost::bind(exportFaceControllerFormat, _1, _2), ItemManager::PRIORITY_CONVERSION);
 
         initialized = true;
     }
@@ -101,6 +101,8 @@ PoseSeqItem::PoseSeqItem()
     : seq(new PoseSeq())
 {
     init();
+
+    barLength_ = 1.0;
 }
 
 
@@ -109,6 +111,8 @@ PoseSeqItem::PoseSeqItem(const PoseSeqItem& org)
       seq(new PoseSeq(*org.seq))
 {
     init();
+
+    barLength_ = org.barLength_;
 }
 
 
@@ -125,8 +129,6 @@ void PoseSeqItem::init()
 
     clearEditHistory();
 
-    sigPositionChanged().connect(bind(&PoseSeqItem::onPositionChanged, this));
-
     generationBar = BodyMotionGenerationBar::instance();
 
     isSelectedPoseMoving = false;
@@ -143,7 +145,7 @@ PoseSeqItem::~PoseSeqItem()
 void PoseSeqItem::setName(const std::string& name)
 {
     seq->setName(name);
-    setInconsistencyWithLastAccessedFile();
+    suggestFileUpdate();
     Item::setName(name);
 }
 
@@ -152,49 +154,57 @@ void PoseSeqItem::onPositionChanged()
 {
     if(!sigInterpolationParametersChangedConnection.connected()){
         sigInterpolationParametersChangedConnection =
-            BodyMotionGenerationBar::instance()->sigInterpolationParametersChanged().connect(
-                bind(&PoseSeqItem::updateInterpolationParameters, this));
+            generationBar->sigInterpolationParametersChanged().connect(
+                boost::bind(&PoseSeqItem::updateInterpolationParameters, this));
         updateInterpolationParameters();
     }
 
-    BodyItem* prevOwnerBodyItem = ownerBodyItem;
-    
+    BodyItemPtr prevBodyItem = ownerBodyItem;
     ownerBodyItem = findOwnerItem<BodyItem>();
+    if(ownerBodyItem == prevBodyItem){
+        return;
+    }
+        
+    if(!ownerBodyItem){
+        interpolator_->setBody(0);
 
-    if(ownerBodyItem){
-        BodyPtr body = ownerBodyItem->body();
+    } else {
+        Body* body = ownerBodyItem->body();
 
         if(seq->targetBodyName().empty()){
             seq->setTargetBodyName(body->name());
-        } else if(prevOwnerBodyItem && (seq->targetBodyName() != body->name())){
-            convert(prevOwnerBodyItem->body());
+        } else if(prevBodyItem && (seq->targetBodyName() != body->name())){
+            convert(prevBodyItem->body());
         }
 
         interpolator_->setBody(body);
 
-        const YamlSequence& linearInterpolationJoints =
-            *ownerBodyItem->body()->info()->findSequence("linearInterpolationJoints");
+        const Listing& linearInterpolationJoints =
+            *ownerBodyItem->body()->info()->findListing("linearInterpolationJoints");
         if(linearInterpolationJoints.isValid()){
             for(int i=0; i < linearInterpolationJoints.size(); ++i){
                 Link* link = body->link(linearInterpolationJoints[i].toString());
                 if(link){
-                    interpolator_->setLinearInterpolationJoint(link->jointId);
+                    interpolator_->setLinearInterpolationJoint(link->jointId());
                 }
             }
         }
-        LeggedBody* legged = dynamic_cast<LeggedBody*>(ownerBodyItem->body());
-        if(legged){
+
+        LeggedBodyHelperPtr legged = getLeggedBodyHelper(ownerBodyItem->body());
+        if(legged->isValid()){
             for(int i=0; i < legged->numFeet(); ++i){
-                const LeggedBody::FootInfo& info = legged->footInfo(i);
-                interpolator_->addFootLink(info.link->index, info.soleCenter);
+                interpolator_->addFootLink(legged->footLink(i)->index(), legged->centerOfSoleLocal(i));
             }
         }
 
         interpolator_->setLipSyncShapes(*ownerBodyItem->body()->info()->findMapping("lipSyncShapes"));
         bodyMotionItem_->motion()->setNumParts(interpolator_->body()->numJoints());
-        bodyMotionItem_->notifyUpdate();
-    } else {
-        interpolator_->setBody(0);
+
+        if(generationBar->isAutoGenerationForNewBodyEnabled()){
+            updateTrajectory(true);
+        } else {
+            bodyMotionItem_->notifyUpdate();
+        }
     }
 }
 
@@ -206,23 +216,23 @@ void PoseSeqItem::convert(BodyPtr orgBody)
         return;
     }
     
-    const YamlSequence& convInfoTop = *ownerBodyItem->body()->info()->findSequence("poseConversionInfo");
+    const Listing& convInfoTop = *ownerBodyItem->body()->info()->findListing("poseConversionInfo");
     if(convInfoTop.isValid()){
         for(int i=0; i < convInfoTop.size(); ++i){
-            const YamlMapping& convInfo = *convInfoTop[i].toMapping();
-            const YamlSequence& targets = *convInfo["targetBodies"].toSequence();
+            const Mapping& convInfo = *convInfoTop[i].toMapping();
+            const Listing& targets = *convInfo["targetBodies"].toListing();
             for(int j=0; j < targets.size(); ++j){
                 if(targets[j].toString() == orgBody->name()){
 
                     beginEditing();
                     if(endEditing(convertSub(orgBody, convInfo))){
                         
-                        clearLastAccessInformation();
+                        clearFileInformation();
                         BodyPtr body = ownerBodyItem->body();
                         seq->setTargetBodyName(body->name());
                         format m(_("Pose seq \"%1%\" has been converted. Its target has been changed from %2% to %3%"));
                         MessageView::mainInstance()->notify(str(m % name() % orgBody->name() % body->name()));
-
+                        
                         return;
                     }
                 }
@@ -232,12 +242,12 @@ void PoseSeqItem::convert(BodyPtr orgBody)
 }
 
 
-bool PoseSeqItem::convertSub(BodyPtr orgBody, const YamlMapping& convInfo)
+bool PoseSeqItem::convertSub(BodyPtr orgBody, const Mapping& convInfo)
 {
     bool converted = false;
     
-    const YamlSequence& jointMap = *convInfo.findSequence("jointMap");
-    const YamlMapping& linkMap = *convInfo.findMapping("linkMap");
+    const Listing& jointMap = *convInfo.findListing("jointMap");
+    const Mapping& linkMap = *convInfo.findMapping("linkMap");
     BodyPtr body = ownerBodyItem->body();
     
     for(PoseSeq::iterator p = seq->begin(); p != seq->end(); ++p){
@@ -272,7 +282,7 @@ bool PoseSeqItem::convertSub(BodyPtr orgBody, const YamlMapping& convInfo)
                 for(Pose::LinkInfoMap::const_iterator q = orgPose->ikLinkBegin(); q != orgPose->ikLinkEnd(); ++q){
                     Link* orgLink = orgBody->link(q->first);
                     string linkName;
-                    YamlNode* linkNameNode = linkMap.find(orgLink->name());
+                    ValueNode* linkNameNode = linkMap.find(orgLink->name());
                     if(linkNameNode->isValid()){
                         linkName = linkNameNode->toString();
                     } else {
@@ -281,7 +291,7 @@ bool PoseSeqItem::convertSub(BodyPtr orgBody, const YamlMapping& convInfo)
                     Link* link = body->link(linkName);
                     if(link){
                         const Pose::LinkInfo& orgLinkInfo = q->second;
-                        Pose::LinkInfo* linkInfo = pose->addIkLink(link->index);
+                        Pose::LinkInfo* linkInfo = pose->addIkLink(link->index());
                         linkInfo->p = orgLinkInfo.p;
                         linkInfo->R = orgLinkInfo.R;
                         linkInfo->setStationaryPoint(orgLinkInfo.isStationaryPoint());
@@ -290,7 +300,7 @@ bool PoseSeqItem::convertSub(BodyPtr orgBody, const YamlMapping& convInfo)
                         }
                         linkInfo->setSlave(orgLinkInfo.isSlave());
                         if(orgLinkInfo.isBaseLink()){
-                            baseLinkIndex = link->index;
+                            baseLinkIndex = link->index();
                         }
                     }
                 }
@@ -310,7 +320,7 @@ bool PoseSeqItem::convertSub(BodyPtr orgBody, const YamlMapping& convInfo)
 }
 
 
-ItemPtr PoseSeqItem::doDuplicate() const
+Item* PoseSeqItem::doDuplicate() const
 {
     return new PoseSeqItem(*this);
 }
@@ -328,7 +338,10 @@ void PoseSeqItem::updateInterpolationParameters()
 
     interpolator_->enableAutoZmpAdjustmentMode(generationBar->isAutoZmpAdjustmentMode());
     interpolator_->setZmpAdjustmentParameters(
-        generationBar->minZmpTransitionTime(), generationBar->zmpCenteringTimeThresh(), generationBar->zmpTimeMarginBeforeLifting());
+        generationBar->minZmpTransitionTime(),
+        generationBar->zmpCenteringTimeThresh(),
+        generationBar->zmpTimeMarginBeforeLifting(),
+        generationBar->zmpMaxDistanceFromCenter());
 
     interpolator_->enableLipSyncMix(generationBar->isLipSyncMixMode());
 }        
@@ -366,10 +379,10 @@ void PoseSeqItem::beginEditing()
 
     if(editConnections.empty()){
         editConnections = seq->connectSignalSet(
-            bind(&PoseSeqItem::onInserted, this, _1, _2),
-            bind(&PoseSeqItem::onRemoving, this, _1, _2),
-            bind(&PoseSeqItem::onModifying, this, _1),
-            bind(&PoseSeqItem::onModified, this, _1));
+            boost::bind(&PoseSeqItem::onInserted, this, _1, _2),
+            boost::bind(&PoseSeqItem::onRemoving, this, _1, _2),
+            boost::bind(&PoseSeqItem::onModifying, this, _1),
+            boost::bind(&PoseSeqItem::onModified, this, _1));
     }
 }
 
@@ -392,7 +405,7 @@ bool PoseSeqItem::endEditing(bool actuallyModified)
             editHistories.resize(currentHistory);
             editHistories.push_back(newHistory);
             currentHistory = editHistories.size();
-            setInconsistencyWithLastAccessedFile();
+            suggestFileUpdate();
         } 
     }
     modifyingPoseIter = seq->end();
@@ -499,7 +512,7 @@ bool PoseSeqItem::undo()
             current->setMaxTransitionTime(p->maxTransitionTime());
         }
         editConnections.unblock();
-        setInconsistencyWithLastAccessedFile();
+        suggestFileUpdate();
         return true;
     }
     return false;
@@ -522,7 +535,7 @@ bool PoseSeqItem::redo()
             current->setMaxTransitionTime(p->maxTransitionTime());
         }
         editConnections.unblock();
-        setInconsistencyWithLastAccessedFile();
+        suggestFileUpdate();
         return true;
     }
     return false;
@@ -533,7 +546,7 @@ bool PoseSeqItem::updateKeyPosesWithBalancedTrajectories(std::ostream& os)
 {
     BodyMotionPtr motion = bodyMotionItem()->motion();
     MultiValueSeqPtr qseq = motion->jointPosSeq();
-    MultiAffine3SeqPtr pseq = motion->linkPosSeq();
+    MultiSE3SeqPtr pseq = motion->linkPosSeq();
 
     double length = seq->endingTime();
     
@@ -556,13 +569,13 @@ bool PoseSeqItem::updateKeyPosesWithBalancedTrajectories(std::ostream& os)
             
             int nj = pose->numJoints();
             int frame = qseq->frameOfTime(p->time());
-            MultiValueSeq::View q = qseq->frame(frame);
+            MultiValueSeq::Frame q = qseq->frame(frame);
             for(int i=0; i < nj; ++i){
                 if(pose->isJointValid(i)){
                     pose->setJointPosition(i, q[i]);
                 }
             }
-            MultiAffine3Seq::View pos = pseq->frame(frame);
+            MultiSE3Seq::Frame pos = pseq->frame(frame);
             for(Pose::LinkInfoMap::iterator q = pose->ikLinkBegin(); q != pose->ikLinkEnd(); ++q){
                 int linkIndex = q->first;
                 Pose::LinkInfo& info = q->second;
@@ -586,15 +599,17 @@ bool PoseSeqItem::updateKeyPosesWithBalancedTrajectories(std::ostream& os)
 
 void PoseSeqItem::doPutProperties(PutPropertyFunction& putProperty)
 {
-    putProperty(_("targetBody"), seq->targetBodyName());
+    putProperty(_("Target body"), seq->targetBodyName());
+    putProperty(_("Bar length"), barLength_, changeProperty(barLength_));
 }
 
 
 bool PoseSeqItem::store(Archive& archive)
 {
     if(overwrite()){
-        archive.writeRelocatablePath("filename", lastAccessedFileName());
-        archive.write("format", lastAccessedFileFormatId());
+        archive.writeRelocatablePath("filename", filePath());
+        archive.write("format", fileFormat());
+        archive.write("barLength", barLength_);
         return true;
     }
     return false;
@@ -606,6 +621,7 @@ bool PoseSeqItem::restore(const Archive& archive)
     std::string filename, formatId;
     if(archive.readRelocatablePath("filename", filename) && archive.read("format", formatId)){
         if(load(filename, archive.currentParentItem(), formatId)){
+            archive.read("barLength", barLength_);
             return true;
         }
     }
