@@ -4,8 +4,8 @@
 */
 
 #include "PronunSymbol.h"
-#include <cnoid/YamlNodes>
-#include <cnoid/YamlWriter>
+#include <cnoid/ValueTree>
+#include <cnoid/YAMLWriter>
 
 using namespace std;
 using namespace cnoid;
@@ -37,14 +37,14 @@ PoseUnit* PronunSymbol::duplicate()
 }
 
 
-bool PronunSymbol::restore(const YamlMapping& archive, const BodyPtr body)
+bool PronunSymbol::restore(const Mapping& archive, const BodyPtr body)
 {
     return true;
 }
 
 
-void PronunSymbol::store(YamlMapping& archive, const BodyPtr body) const
+void PronunSymbol::store(Mapping& archive, const BodyPtr body) const
 {
     archive.write("type", "PronunSymbol");
-    archive.write("name", name(), YAML_DOUBLE_QUOTED);
+    archive.write("name", name(), DOUBLE_QUOTED);
 }
